' =========================================================
'   
'                Create Picture Format Task
'
'                          a SAS Custom Task Example 
'
' Copyright (C) 2006 by SAS Institute Inc., Cary, NC 27513
'
' ---------------------------------------------------------
' This panel shows the ranges currently defined within the
' picture format. It is from this panel that new ranges are
' added, modified and removed.
' The OTHER and missing items are also defined here.
' A feature of this window is the ability to test the
' picture format as it is currently defined by specifying
' values to be formatted based in the current ranges.
' =========================================================

Imports System.Collections
Imports System.Drawing
Imports System.Globalization
Imports System.Text
Imports System.Windows.Forms

Namespace SASVB.CustomTasks
    Public Class DefinitionPanel
        Inherits System.Windows.Forms.UserControl

#Region " Public Events "
        Public Event ParametersUpdated(ByVal sender As Object, ByVal ea As System.EventArgs)
#End Region

#Region " Member Variables "
        Private _Consumer As SAS.Shared.AddIns.ISASTaskConsumer = Nothing

        ' 0 = Numeric, 1 = Date, 2 = Time, 3 = DateTime
        Private _PictureType As Int32 = 0

        Private _ServerName As String = "Local"

        Private _DefaultWidth As Int32 = 12

        Private _TestValues() As Double = New Double() {}
#End Region

#Region " Properties "
        ' The definition parameters object is used to pass the
        ' range definitions between the view to the model.
        Public Property DefinitionParams() As DefinitionParameters
            Get
                ' We need to define a new definition parameter object
                ' which will be passed back with all the latest range
                ' values in it.
                Dim returnDP As DefinitionParameters = New DefinitionParameters

                ' Just to be sure, we'll clear out the ArrayList
                ' before we insert the new items into it.
                returnDP.RangeItems.Clear()
                ' Get each RangeItem for the ranges defined within the
                ' ListView and clone it into a new Rangeitem for adding
                ' to the ArrayList we'll be returning. 
                For Each lvi As ListViewItem In lvwRanges.Items
                    returnDP.RangeItems.Add(CType(lvi.Tag, RangeItem).Clone())
                Next

                ' Return the 'missing' label information.
                returnDP.UseMissingLabel = chkUseMissingLabel.Checked
                returnDP.MissingLabel = txtMissingLabel.Text

                ' And the OTHER label information.
                returnDP.UseOTHERLabel = chkUseOtherLabel.Checked
                returnDP.OTHERLabel = txtOtherLabel.Text

                ' Lastly, we need to record any test values that have
                ' been defined within the TestRanges dialog. Start by
                ' resizing the TestValues Double array to be just big
                ' enough to hold the defined values.
                ReDim returnDP.TestValues(_TestValues.GetLength(0) - 1)
                ' Now loop through the source array and copy the double
                ' values to the new array.
                For index As Int32 = 0 To _TestValues.GetLength(0) - 1
                    returnDP.TestValues(index) = _TestValues(index)
                Next

                ' The definition parameters object is ready to return.
                Return returnDP
            End Get

            Set(ByVal Value As DefinitionParameters)
                Dim lvi As ListViewItem

                ' Remove existing range items from the list view control
                For rangeItemIdx As Int32 = lvwRanges.Items.Count - 1 To 0 Step -1
                    lvwRanges.Items(rangeItemIdx).Tag = Nothing
                    lvwRanges.Items.RemoveAt(rangeItemIdx)
                Next

                ' Add the new set of range items to the list view control.
                For Each ri As RangeItem In Value.RangeItems
                    lvi = New ListViewItem
                    lvi.Tag = ri
                    ' Propogate the item's values to the subitems within the
                    ' list view item.
                    PopulateListItem(lvi)
                    ' Add the new item to the list view control.
                    lvwRanges.Items.Add(lvi)
                Next

                ' Update the controls associated with the 'missing' label. 
                chkUseMissingLabel.Checked = Value.UseMissingLabel
                txtMissingLabel.Text = Value.MissingLabel

                ' Update the controls associated with the 'OTHER' label.
                chkUseOtherLabel.Checked = Value.UseOTHERLabel
                txtOtherLabel.Text = Value.OTHERLabel

                ' Make a copy of the Double test values.
                ReDim _TestValues(Value.TestValues.GetLength(0) - 1)
                For index As Int32 = 0 To Value.TestValues.GetLength(0) - 1
                    _TestValues(index) = Value.TestValues(index)
                Next

                ' Make sure that the first of the items is selected. Note
                ' that there is ALWAYS at least one item in the Ranges
                ' list view - even if the whole lot is undefined.
                lvwRanges.Items(0).Selected = True

                ' Update the button controls based on the currently
                ' selected item's type.
                UpdateButtonStates()
            End Set
        End Property

        ' PictureType is needed so that we know how to display the Double
        ' values that define the ranges as well as what controls we need
        ' to show to allow the user to change the range values.
        Public WriteOnly Property PictureType() As Int32
            Set(ByVal Value As Int32)
                ' The only valid values are 0 = Numeric, 1 = Date, 2 = Time,
                ' or 3 = DateTime
                If (Value < 0 Or Value > 3) Then
                    Throw New Exception("PictureType value expected to be 0, 1, 2 or 3.")
                End If

                ' We only need to do something if the value has actually
                ' changed from what it was last time.
                If (_PictureType <> Value) Then
                    ' Store the new value.
                    _PictureType = Value

                    ' Loop through each list view item and update the
                    ' displayed values based on the new picture type.
                    For Each lvi As ListViewItem In lvwRanges.Items
                        PopulateListItem(lvi)
                    Next
                End If
            End Set
        End Property

        ' Associated Server for the task. This is used in the
        ' definition panel when testing the format.
        Public WriteOnly Property ServerName() As String
            Set(ByVal Value As String)
                ' Store the name of the server associated with the
                ' task.
                _ServerName = Value
            End Set
        End Property

        ' Default width for the formatted values. This is used in the
        ' definition panel when testing the format.
        Public WriteOnly Property DefaultWidth() As Int32
            Set(ByVal Value As Int32)
                ' The default width value must be from 1 to 40.
                If (Value < 1 Or Value > 40) Then
                    Throw New Exception("Default width value expected to be betweeen 1 and 40 (inclusive).")
                End If

                ' Store the default width for now.
                _DefaultWidth = Value
            End Set
        End Property

        ' The consumer interface for the task. This is used in the
        ' definition panel when testing the format.
        Public WriteOnly Property Consumer() As SAS.Shared.AddIns.ISASTaskConsumer
            Set(ByVal Value As SAS.Shared.AddIns.ISASTaskConsumer)
                ' Simply store the consumer for now.
                _Consumer = Value
            End Set
        End Property

        ' A flag which indicates whether the panel is in a valid
        ' state and is therefore runnable.
        Public ReadOnly Property IsAbleToBeRun() As Boolean
            Get
                ' To be valid, there must be at least one defined
                ' range, or the 'missing' label is enabled, or the
                ' OTHER label is enabled.
                Return (lvwRanges.Items.Count > 1 Or _
                        CType(lvwRanges.Items(0).Tag, RangeItem).IsUndefinedRange = False Or _
                        chkUseMissingLabel.Checked = True Or _
                        chkUseOtherLabel.Checked = True)
            End Get
        End Property
#End Region

#Region " Windows Form Designer generated code "

        Public Sub New()
            MyBase.New()

            'This call is required by the Windows Form Designer.
            InitializeComponent()

            'Initialise the various member variables and structures.
            InitializeData()

            ' Set the buttons to their initial enabled/disabled states.
            UpdateButtonStates()
        End Sub

        'UserControl overrides dispose to clean up the component list.
        Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
            If disposing Then
                If Not (components Is Nothing) Then
                    components.Dispose()
                End If
            End If
            MyBase.Dispose(disposing)
        End Sub

        'Required by the Windows Form Designer
        Private components As System.ComponentModel.IContainer

        'NOTE: The following procedure is required by the Windows Form Designer
        'It can be modified using the Windows Form Designer.  
        'Do not modify it using the code editor.
        Friend WithEvents colMask As System.Windows.Forms.ColumnHeader
        Friend WithEvents colStartValue As System.Windows.Forms.ColumnHeader
        Friend WithEvents colStartInclusive As System.Windows.Forms.ColumnHeader
        Friend WithEvents colEndValue As System.Windows.Forms.ColumnHeader
        Friend WithEvents colPrefix As System.Windows.Forms.ColumnHeader
        Friend WithEvents colFillChar As System.Windows.Forms.ColumnHeader
        Friend WithEvents colNoEdit As System.Windows.Forms.ColumnHeader
        Friend WithEvents colEndInclusive As System.Windows.Forms.ColumnHeader
        Friend WithEvents colMultiplier As System.Windows.Forms.ColumnHeader
        Friend WithEvents btnTest As System.Windows.Forms.Button
        Friend WithEvents btnAddMask As System.Windows.Forms.Button
        Friend WithEvents btnRemoveMask As System.Windows.Forms.Button
        Friend WithEvents chkUseOtherLabel As System.Windows.Forms.CheckBox
        Friend WithEvents lblOtherLabel As System.Windows.Forms.Label
        Friend WithEvents txtOtherLabel As System.Windows.Forms.TextBox
        Friend WithEvents lvwRanges As System.Windows.Forms.ListView
        Friend WithEvents btnModifyMask As System.Windows.Forms.Button
        Friend WithEvents btnRemoveAllMasks As System.Windows.Forms.Button
        Friend WithEvents chkUseMissingLabel As System.Windows.Forms.CheckBox
        Friend WithEvents lblMissingLabel As System.Windows.Forms.Label
        Friend WithEvents txtMissingLabel As System.Windows.Forms.TextBox
        <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
            Me.lvwRanges = New System.Windows.Forms.ListView
            Me.colStartValue = New System.Windows.Forms.ColumnHeader
            Me.colStartInclusive = New System.Windows.Forms.ColumnHeader
            Me.colEndValue = New System.Windows.Forms.ColumnHeader
            Me.colEndInclusive = New System.Windows.Forms.ColumnHeader
            Me.colMask = New System.Windows.Forms.ColumnHeader
            Me.colNoEdit = New System.Windows.Forms.ColumnHeader
            Me.colPrefix = New System.Windows.Forms.ColumnHeader
            Me.colFillChar = New System.Windows.Forms.ColumnHeader
            Me.colMultiplier = New System.Windows.Forms.ColumnHeader
            Me.btnTest = New System.Windows.Forms.Button
            Me.btnAddMask = New System.Windows.Forms.Button
            Me.btnRemoveMask = New System.Windows.Forms.Button
            Me.chkUseMissingLabel = New System.Windows.Forms.CheckBox
            Me.lblMissingLabel = New System.Windows.Forms.Label
            Me.txtMissingLabel = New System.Windows.Forms.TextBox
            Me.chkUseOtherLabel = New System.Windows.Forms.CheckBox
            Me.lblOtherLabel = New System.Windows.Forms.Label
            Me.txtOtherLabel = New System.Windows.Forms.TextBox
            Me.btnRemoveAllMasks = New System.Windows.Forms.Button
            Me.btnModifyMask = New System.Windows.Forms.Button
            Me.SuspendLayout()
            '
            'lvwRanges
            '
            Me.lvwRanges.Anchor = CType((((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Bottom) _
                        Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.lvwRanges.Columns.AddRange(New System.Windows.Forms.ColumnHeader() {Me.colStartValue, Me.colStartInclusive, Me.colEndValue, Me.colEndInclusive, Me.colMask, Me.colNoEdit, Me.colPrefix, Me.colFillChar, Me.colMultiplier})
            Me.lvwRanges.FullRowSelect = True
            Me.lvwRanges.HideSelection = False
            Me.lvwRanges.Location = New System.Drawing.Point(8, 8)
            Me.lvwRanges.MultiSelect = False
            Me.lvwRanges.Name = "lvwRanges"
            Me.lvwRanges.Size = New System.Drawing.Size(568, 160)
            Me.lvwRanges.TabIndex = 0
            Me.lvwRanges.View = System.Windows.Forms.View.Details
            '
            'colStartValue
            '
            Me.colStartValue.Text = "Start Value"
            Me.colStartValue.TextAlign = System.Windows.Forms.HorizontalAlignment.Center
            Me.colStartValue.Width = 80
            '
            'colStartInclusive
            '
            Me.colStartInclusive.Text = "Incl."
            Me.colStartInclusive.TextAlign = System.Windows.Forms.HorizontalAlignment.Center
            Me.colStartInclusive.Width = 40
            '
            'colEndValue
            '
            Me.colEndValue.Text = "End Value"
            Me.colEndValue.Width = 80
            '
            'colEndInclusive
            '
            Me.colEndInclusive.Text = "Incl."
            Me.colEndInclusive.TextAlign = System.Windows.Forms.HorizontalAlignment.Center
            Me.colEndInclusive.Width = 40
            '
            'colMask
            '
            Me.colMask.Text = "Mask"
            Me.colMask.Width = 100
            '
            'colNoEdit
            '
            Me.colNoEdit.Text = "Label"
            Me.colNoEdit.TextAlign = System.Windows.Forms.HorizontalAlignment.Center
            Me.colNoEdit.Width = 45
            '
            'colPrefix
            '
            Me.colPrefix.Text = "Prefix"
            Me.colPrefix.TextAlign = System.Windows.Forms.HorizontalAlignment.Center
            Me.colPrefix.Width = 45
            '
            'colFillChar
            '
            Me.colFillChar.Text = "Fill"
            Me.colFillChar.TextAlign = System.Windows.Forms.HorizontalAlignment.Center
            Me.colFillChar.Width = 45
            '
            'colMultiplier
            '
            Me.colMultiplier.Text = "Multiplier"
            Me.colMultiplier.Width = 65
            '
            'btnTest
            '
            Me.btnTest.Anchor = CType((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.btnTest.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.btnTest.Location = New System.Drawing.Point(504, 296)
            Me.btnTest.Name = "btnTest"
            Me.btnTest.Size = New System.Drawing.Size(72, 24)
            Me.btnTest.TabIndex = 11
            Me.btnTest.Text = "&Test..."
            '
            'btnAddMask
            '
            Me.btnAddMask.Anchor = CType((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Left), System.Windows.Forms.AnchorStyles)
            Me.btnAddMask.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.btnAddMask.Location = New System.Drawing.Point(8, 168)
            Me.btnAddMask.Name = "btnAddMask"
            Me.btnAddMask.Size = New System.Drawing.Size(80, 24)
            Me.btnAddMask.TabIndex = 1
            Me.btnAddMask.Text = "&Add..."
            '
            'btnRemoveMask
            '
            Me.btnRemoveMask.Anchor = CType((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Left), System.Windows.Forms.AnchorStyles)
            Me.btnRemoveMask.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.btnRemoveMask.Location = New System.Drawing.Point(168, 168)
            Me.btnRemoveMask.Name = "btnRemoveMask"
            Me.btnRemoveMask.Size = New System.Drawing.Size(80, 24)
            Me.btnRemoveMask.TabIndex = 3
            Me.btnRemoveMask.Text = "&Remove"
            '
            'chkUseMissingLabel
            '
            Me.chkUseMissingLabel.Anchor = CType(((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.chkUseMissingLabel.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.chkUseMissingLabel.Location = New System.Drawing.Point(8, 200)
            Me.chkUseMissingLabel.Name = "chkUseMissingLabel"
            Me.chkUseMissingLabel.Size = New System.Drawing.Size(568, 16)
            Me.chkUseMissingLabel.TabIndex = 5
            Me.chkUseMissingLabel.Text = "Specif&y a label to be used for missing values"
            '
            'lblMissingLabel
            '
            Me.lblMissingLabel.Anchor = CType((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Left), System.Windows.Forms.AnchorStyles)
            Me.lblMissingLabel.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.lblMissingLabel.Location = New System.Drawing.Point(32, 224)
            Me.lblMissingLabel.Name = "lblMissingLabel"
            Me.lblMissingLabel.Size = New System.Drawing.Size(192, 16)
            Me.lblMissingLabel.TabIndex = 6
            Me.lblMissingLabel.Text = "La&bel for missing values:"
            '
            'txtMissingLabel
            '
            Me.txtMissingLabel.Anchor = CType(((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.txtMissingLabel.Location = New System.Drawing.Point(224, 222)
            Me.txtMissingLabel.Name = "txtMissingLabel"
            Me.txtMissingLabel.Size = New System.Drawing.Size(352, 20)
            Me.txtMissingLabel.TabIndex = 7
            Me.txtMissingLabel.Text = ""
            '
            'chkUseOtherLabel
            '
            Me.chkUseOtherLabel.Anchor = CType(((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.chkUseOtherLabel.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.chkUseOtherLabel.Location = New System.Drawing.Point(8, 248)
            Me.chkUseOtherLabel.Name = "chkUseOtherLabel"
            Me.chkUseOtherLabel.Size = New System.Drawing.Size(568, 16)
            Me.chkUseOtherLabel.TabIndex = 8
            Me.chkUseOtherLabel.Text = "For all other values, &use a common label"
            '
            'lblOtherLabel
            '
            Me.lblOtherLabel.Anchor = CType((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Left), System.Windows.Forms.AnchorStyles)
            Me.lblOtherLabel.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.lblOtherLabel.Location = New System.Drawing.Point(32, 272)
            Me.lblOtherLabel.Name = "lblOtherLabel"
            Me.lblOtherLabel.Size = New System.Drawing.Size(192, 16)
            Me.lblOtherLabel.TabIndex = 9
            Me.lblOtherLabel.Text = "Label for &other values:"
            '
            'txtOtherLabel
            '
            Me.txtOtherLabel.Anchor = CType(((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.txtOtherLabel.Location = New System.Drawing.Point(224, 270)
            Me.txtOtherLabel.Name = "txtOtherLabel"
            Me.txtOtherLabel.Size = New System.Drawing.Size(352, 20)
            Me.txtOtherLabel.TabIndex = 10
            Me.txtOtherLabel.Text = ""
            '
            'btnRemoveAllMasks
            '
            Me.btnRemoveAllMasks.AccessibleDescription = ""
            Me.btnRemoveAllMasks.Anchor = CType((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Left), System.Windows.Forms.AnchorStyles)
            Me.btnRemoveAllMasks.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.btnRemoveAllMasks.Location = New System.Drawing.Point(272, 168)
            Me.btnRemoveAllMasks.Name = "btnRemoveAllMasks"
            Me.btnRemoveAllMasks.Size = New System.Drawing.Size(80, 24)
            Me.btnRemoveAllMasks.TabIndex = 4
            Me.btnRemoveAllMasks.Text = "Remo&ve All"
            '
            'btnModifyMask
            '
            Me.btnModifyMask.Anchor = CType((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Left), System.Windows.Forms.AnchorStyles)
            Me.btnModifyMask.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.btnModifyMask.Location = New System.Drawing.Point(88, 168)
            Me.btnModifyMask.Name = "btnModifyMask"
            Me.btnModifyMask.Size = New System.Drawing.Size(80, 24)
            Me.btnModifyMask.TabIndex = 2
            Me.btnModifyMask.Text = "&Modify..."
            '
            'DefinitionPanel
            '
            Me.Controls.Add(Me.txtMissingLabel)
            Me.Controls.Add(Me.lblMissingLabel)
            Me.Controls.Add(Me.chkUseMissingLabel)
            Me.Controls.Add(Me.txtOtherLabel)
            Me.Controls.Add(Me.lblOtherLabel)
            Me.Controls.Add(Me.chkUseOtherLabel)
            Me.Controls.Add(Me.btnRemoveMask)
            Me.Controls.Add(Me.btnAddMask)
            Me.Controls.Add(Me.btnTest)
            Me.Controls.Add(Me.lvwRanges)
            Me.Controls.Add(Me.btnRemoveAllMasks)
            Me.Controls.Add(Me.btnModifyMask)
            Me.Name = "DefinitionPanel"
            Me.Size = New System.Drawing.Size(584, 328)
            Me.ResumeLayout(False)

        End Sub

#End Region

#Region " Child Control Event Handlers "
        Private Sub btnAddMask_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnAddMask.Click
            Dim dlg As RangeEditor = New RangeEditor
            Dim selItemIndex As Integer = lvwRanges.SelectedIndices.Item(0)
            Dim selRange As RangeItem = CType(lvwRanges.Items(selItemIndex).Tag, RangeItem)

            dlg.AddMode = True
            dlg.PictureType = _PictureType
            dlg.SetRanges(selRange.Clone(), selRange)
            If (dlg.ShowDialog() = DialogResult.OK) Then
                ' Determine what adjustments need to be made to the
                ' original item based on the new range item's values.
                Dim newRange As RangeItem = dlg.RangeItem

                ' Start by checking to see if the new range contains
                ' a mask value. If it doesn't, then we've got nothing
                ' else to do.
                If (newRange.IsUndefinedRange = True) Then
                    Return
                End If

                ' Now, does the new range encompass the original
                ' range's starting value?
                If Not ((selRange.Low = newRange.Low And selRange.Low = True) Or _
                    (selRange.Low = newRange.Low And selRange.Low = False And _
                     selRange.StartValue = newRange.StartValue And _
                     selRange.IncludeStartValue = newRange.IncludeStartValue)) Then

                    ' The original and new ranges' start values do not
                    ' match which means that we need to insert a new
                    ' unspecified range to precede the new range.
                    Dim workRange As RangeItem = New RangeItem
                    workRange.Low = selRange.Low
                    workRange.StartValue = selRange.StartValue
                    workRange.IncludeStartValue = selRange.IncludeStartValue
                    workRange.High = False
                    workRange.EndValue = newRange.StartValue
                    workRange.IncludeEndValue = (newRange.IncludeStartValue = False)
                    Dim newlvi As ListViewItem = New ListViewItem
                    ' Load up the list item's subitems with the new range's values
                    newlvi.Tag = workRange
                    PopulateListItem(newlvi)
                    lvwRanges.Items.Insert(selItemIndex, newlvi)
                    selItemIndex = selItemIndex + 1
                End If

                ' Next we need to see if the new range includes the original
                ' range's end value.
                If Not ((selRange.High = newRange.High And selRange.High = True) Or _
                    (selRange.High = newRange.High And selRange.High = False And _
                     selRange.EndValue = newRange.EndValue And _
                     selRange.IncludeEndValue = newRange.IncludeEndValue)) Then

                    ' The original and new ranges' end values do not
                    ' match which means that we need to insert a new
                    ' unspecified range to follow the new range.
                    Dim workRange As RangeItem = New RangeItem
                    workRange.Low = False
                    workRange.StartValue = newRange.EndValue
                    workRange.IncludeStartValue = (newRange.IncludeEndValue = False)
                    workRange.High = selRange.High
                    workRange.EndValue = selRange.EndValue
                    workRange.IncludeEndValue = selRange.IncludeEndValue
                    Dim newlvi As ListViewItem = New ListViewItem
                    ' Load up the list item's subitems with the new range's values
                    newlvi.Tag = workRange
                    PopulateListItem(newlvi)
                    If (selItemIndex + 1 >= lvwRanges.Items.Count) Then
                        lvwRanges.Items.Add(newlvi)
                    Else
                        lvwRanges.Items.Insert(selItemIndex + 1, newlvi)
                    End If
                End If

                ' Now replace the values in the selected list view item 
                ' with the new ones and we're done.
                lvwRanges.Items(selItemIndex).Tag = newRange
                PopulateListItem(lvwRanges.Items(selItemIndex))
                lvwRanges.Items(selItemIndex).Selected = True

                UpdateButtonStates()
            End If
        End Sub

        Private Sub lvwRanges_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles lvwRanges.SelectedIndexChanged
            UpdateButtonStates()
        End Sub

        Private Sub btnModifyMask_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnModifyMask.Click
            Dim dlg As RangeEditor = New RangeEditor
            Dim selItemIndex As Integer = lvwRanges.SelectedIndices.Item(0)
            Dim selRange As RangeItem = CType(lvwRanges.Items(selItemIndex).Tag, RangeItem)
            Dim limitRange As RangeItem = selRange.Clone()


            ' We need to start by creating the limit range object. Consider
            ' the situation where we have one range defined as:
            '     Low -< 0 = Undefined
            '     0 - 100 = "999"
            '     100 <- High = Undefined
            ' If we were to select the defined range (0 - 100) and wanted to
            ' modify it, then the range could be modified anywhere from Low
            ' to High, without conflicting with any other defined range.
            ' In this next example, if the third item was to be modified,
            ' then the limit range would be limited from 100 <- High. 
            '     Low -< 0 = Undefined
            '     0 - 100 = "999"
            '     100 <- 150 = "999*"
            '     150 <- High = Undefined
            '
            ' Start by checking for an undefined range preceding the 
            ' selected item
            If (selItemIndex > 0) Then
                Dim workRange As RangeItem

                workRange = CType(lvwRanges.Items(selItemIndex - 1).Tag, RangeItem)
                If (workRange.IsUndefinedRange = True) Then
                    limitRange.Low = workRange.Low
                    limitRange.StartValue = workRange.StartValue
                    limitRange.IncludeStartValue = workRange.IncludeStartValue
                End If
            End If

            ' Now see if there is an undefined range that follows the 
            ' selected item
            If (selItemIndex < lvwRanges.Items.Count - 1) Then
                Dim workRange As RangeItem

                workRange = CType(lvwRanges.Items(selItemIndex + 1).Tag, RangeItem)
                If (workRange.IsUndefinedRange = True) Then
                    limitRange.High = workRange.High
                    limitRange.EndValue = workRange.EndValue
                    limitRange.IncludeEndValue = workRange.IncludeEndValue
                End If
            End If

            ' Now that we have the limit range defined, we're ready to
            ' initialize the range editor dialog and display it.
            dlg.AddMode = False
            dlg.PictureType = _PictureType
            dlg.SetRanges(limitRange, selRange)
            If (dlg.ShowDialog() = DialogResult.OK) Then
                ' Check if we need to remove the item due to there
                ' being no mask defined for it.
                If (dlg.RangeItem.IsUndefinedRange = True) Then
                    ' So we do need to remove the range. Initialize
                    ' the non-range parts.
                    selRange.Mask = String.Empty
                    selRange.NoEdit = False
                    selRange.UsePrefix = False
                    selRange.Prefix = String.Empty
                    selRange.UseFillChar = False
                    selRange.FillChar = " "
                    selRange.UseMultiplier = False
                    selRange.Multiplier = 1.0
                    ' Now if there was an undefined range following the
                    ' selected item, then we need to merge it with the
                    ' selected item's range and delete the item from the
                    ' control.
                    If (selRange.IsEndEqualTo(limitRange) = False) Then
                        ' There was an undefined range following, so
                        ' we need to use its upper limit.
                        selRange.High = limitRange.High
                        selRange.EndValue = limitRange.EndValue
                        selRange.IncludeEndValue = limitRange.IncludeEndValue
                        ' Remove the reference to the range item.
                        lvwRanges.Items(selItemIndex + 1).Tag = Nothing
                        ' Remove the range item itself.
                        lvwRanges.Items.RemoveAt(selItemIndex + 1)
                    End If
                    ' Likewise, if there was an undefined range preceding
                    ' the selected item, then we need to merge it with the
                    ' selected item's range and delete the item from the
                    ' control.
                    If (selRange.IsStartEqualTo(limitRange) = False) Then
                        ' There was an undefined range, so we need to
                        ' use its lower limit.
                        selRange.Low = limitRange.Low
                        selRange.StartValue = limitRange.StartValue
                        selRange.IncludeStartValue = limitRange.IncludeStartValue
                        ' Remove the reference to the range item.
                        lvwRanges.Items(selItemIndex - 1).Tag = Nothing
                        ' Remove the range item itself.
                        lvwRanges.Items.RemoveAt(selItemIndex - 1)
                        ' Since we've now removed the item before the
                        ' selceted one, we need to decrement selItemIndex
                        ' to reflect this.
                        selItemIndex = selItemIndex - 1
                    End If

                    PopulateListItem(lvwRanges.Items(selItemIndex))
                Else
                    ' OK, so we haven't had to remove the range -  that
                    ' means that it needs to be updated. This could involve
                    ' inserting, modifying or removing undefined ranges 
                    ' around it.
                    '
                    ' We'll start by dealing with the range before the
                    ' selected item, then the one that comes after it, and
                    ' finally updating the selected range itself.
                    '
                    ' Determine if the low part of the range has changed.
                    If (dlg.RangeItem.IsStartEqualTo(selRange) = False) Then
                        ' Something has changed. What we do next differs
                        ' depending on whether there is already an
                        ' undefined range or not.
                        If (limitRange.IsStartEqualTo(selRange) = True) Then
                            ' There was not previously an undefined range,
                            ' so we need to create one.
                            Dim newRange As RangeItem = New RangeItem

                            newRange.Low = limitRange.Low
                            newRange.StartValue = limitRange.StartValue
                            newRange.IncludeStartValue = limitRange.IncludeStartValue
                            newRange.High = dlg.RangeItem.Low
                            newRange.EndValue = dlg.RangeItem.StartValue
                            newRange.IncludeEndValue = (dlg.RangeItem.IncludeStartValue = False)
                            ' Create a new list view item for the new 
                            ' undefined range.
                            Dim newlvi As ListViewItem = New ListViewItem
                            ' Associate the new range with the new list 
                            ' view item.
                            newlvi.Tag = newRange
                            ' Populate the list view item's subitems with the 
                            ' new range's values.
                            PopulateListItem(newlvi)
                            ' Insert the new range into the list view.
                            lvwRanges.Items.Insert(selItemIndex, newlvi)
                            ' Since the new item is inserted before the
                            ' selected range, we need to adjust our index.
                            selItemIndex = selItemIndex + 1
                        Else
                            ' There was an undefined range preceding the
                            ' selected range, so we need to determine if
                            ' it is still needed or not.
                            If (dlg.RangeItem.IsStartEqualTo(limitRange) = True) Then
                                ' The start of the range appears to have 
                                ' been expanded to encompass the undefined
                                ' range, so we no longer need it.
                                lvwRanges.Items(selItemIndex - 1).Tag = Nothing
                                lvwRanges.Items.RemoveAt(selItemIndex - 1)
                                selItemIndex = selItemIndex - 1
                            Else
                                ' At this point it appears that we just need to
                                ' adjust the end value of the existing
                                ' undefined range.
                                Dim oldRange As RangeItem

                                oldRange = CType(lvwRanges.Items(selItemIndex - 1).Tag, RangeItem)
                                oldRange.High = dlg.RangeItem.Low
                                oldRange.EndValue = dlg.RangeItem.StartValue
                                oldRange.IncludeEndValue = (dlg.RangeItem.IncludeStartValue = False)
                                ' Update the new values displayed within the
                                ' list view item's subitems.
                                PopulateListItem(lvwRanges.Items(selItemIndex - 1))
                            End If
                        End If
                    End If

                    ' Determine if the high part of the range has changed.
                    If (dlg.RangeItem.IsEndEqualTo(selRange) = False) Then
                        ' Something has changed. What we do next differs
                        ' depending on whether there is already an
                        ' undefined range or not.
                        If (limitRange.IsEndEqualTo(selRange) = True) Then
                            ' There was not previously an undefined range,
                            ' so we need to create one.
                            Dim newRange As RangeItem = New RangeItem

                            newRange.Low = dlg.RangeItem.High
                            newRange.StartValue = dlg.RangeItem.EndValue
                            newRange.IncludeStartValue = (dlg.RangeItem.IncludeEndValue = False)
                            newRange.High = limitRange.High
                            newRange.EndValue = limitRange.EndValue
                            newRange.IncludeEndValue = limitRange.IncludeEndValue
                            ' Create a new list view item for the new 
                            ' undefined range.
                            Dim newlvi As ListViewItem = New ListViewItem
                            ' Associate the new range with the new list 
                            ' view item.
                            newlvi.Tag = newRange
                            ' Populate the list view item's subitems with the 
                            ' new range's values.
                            PopulateListItem(newlvi)
                            ' Insert the new range into the list view.
                            If (selItemIndex + 1 >= lvwRanges.Items.Count) Then
                                lvwRanges.Items.Add(newlvi)
                            Else
                                lvwRanges.Items.Insert(selItemIndex + 1, newlvi)
                            End If
                        Else
                            ' There was an undefined range following the
                            ' selected range, so we need to determine if
                            ' it is still needed or not.
                            If (dlg.RangeItem.IsEndEqualTo(limitRange) = True) Then
                                ' The end of the range appears to have 
                                ' been expanded to encompass the undefined
                                ' range, so we no longer need it.
                                lvwRanges.Items(selItemIndex + 1).Tag = Nothing
                                lvwRanges.Items.RemoveAt(selItemIndex + 1)
                            Else
                                ' At this point it appears that we just need to
                                ' adjust the start value of the existing
                                ' undefined range.
                                Dim oldRange As RangeItem

                                oldRange = CType(lvwRanges.Items(selItemIndex + 1).Tag, RangeItem)
                                oldRange.Low = dlg.RangeItem.Low
                                oldRange.StartValue = dlg.RangeItem.EndValue
                                oldRange.IncludeStartValue = (dlg.RangeItem.IncludeEndValue = False)
                                ' Update the new values displayed within the
                                ' list view item's subitems.
                                PopulateListItem(lvwRanges.Items(selItemIndex + 1))
                            End If
                        End If
                    End If

                    ' And now finally, we just need to update the originally
                    ' selected range itself.
                    lvwRanges.Items(selItemIndex).Tag = dlg.RangeItem
                    PopulateListItem(lvwRanges.Items(selItemIndex))
                End If

                UpdateButtonStates()
            End If
        End Sub

        Private Sub btnRemoveMask_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnRemoveMask.Click
            Dim selItemIndex As Integer = lvwRanges.SelectedIndices.Item(0)
            Dim selRange As RangeItem = CType(lvwRanges.Items(selItemIndex).Tag, RangeItem)
            Dim workRange As RangeItem


            ' To remove the currently selected range item we need to
            ' determine if either of the items before or after are
            ' undefined ranges. If they are, then we need to collapse
            ' the consecutive undefined ranges down into a single 
            ' range which encompasses the range we're removing.
            '
            ' Start by attempting to get a preceding undefined range.
            If (selItemIndex > 0) Then
                workRange = CType(lvwRanges.Items(selItemIndex - 1).Tag, RangeItem)
                If (workRange.IsUndefinedRange = True) Then
                    ' Make the start of the selected range the start
                    ' of the undefined range which precedes it.
                    selRange.Low = workRange.Low
                    selRange.StartValue = workRange.StartValue
                    selRange.IncludeStartValue = workRange.IncludeStartValue
                    ' And then delete the undefined range.
                    lvwRanges.Items(selItemIndex - 1).Tag = Nothing
                    lvwRanges.Items.RemoveAt(selItemIndex - 1)
                    ' Because we removed an earlier item from the table
                    ' our selected item's index has now changed. We need
                    ' to update our index to reflect this.
                    selItemIndex = selItemIndex - 1
                End If
            End If

            ' Next attempt to get an undefined range which follows the
            ' selected item.
            If (selItemIndex + 1 < lvwRanges.Items.Count) Then
                workRange = CType(lvwRanges.Items(selItemIndex + 1).Tag, RangeItem)
                If (workRange.IsUndefinedRange = True) Then
                    ' Make the end of the selected range the end
                    ' of the undefined range which follows it.
                    selRange.High = workRange.High
                    selRange.EndValue = workRange.EndValue
                    selRange.IncludeEndValue = workRange.IncludeEndValue
                    ' And then delete the undefined range.
                    lvwRanges.Items(selItemIndex + 1).Tag = Nothing
                    lvwRanges.Items.RemoveAt(selItemIndex + 1)
                End If
            End If

            ' Now we just need to turn the selected item into an
            ' undefined range and update the associated list view
            ' item.
            selRange.Mask = String.Empty
            selRange.NoEdit = False
            selRange.UsePrefix = False
            selRange.Prefix = String.Empty
            selRange.UseFillChar = False
            selRange.FillChar = " "
            selRange.UseMultiplier = False
            selRange.Multiplier = 1.0

            PopulateListItem(lvwRanges.Items(selItemIndex))

            lvwRanges.Items(selItemIndex).Selected = True

            UpdateButtonStates()
        End Sub

        Private Sub btnRemoveAllMasks_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnRemoveAllMasks.Click
            ' If we're clearing out all the items within the list of
            ' ranges then we destroy all of them except one and then
            ' initialize the sole survivor to the low-high range.
            Dim i As Int32
            For i = lvwRanges.Items.Count - 1 To 1 Step -1
                lvwRanges.Items(i).Tag = Nothing
                lvwRanges.Items.RemoveAt(i)
            Next
            CType(lvwRanges.Items(0).Tag, RangeItem).Reset()

            PopulateListItem(lvwRanges.Items(0))

            lvwRanges.Items(0).Selected = True

            UpdateButtonStates()
        End Sub

        Private Sub btnTest_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnTest.Click
            Dim dlg As TestRanges = New TestRanges

            ' Load up the various properties needed by the TestRanges form.
            dlg.Consumer = _Consumer
            dlg.PictureType = _PictureType
            dlg.ServerName = _ServerName
            dlg.DefaultWidth = _DefaultWidth
            dlg.TestValues = _TestValues
            dlg.FormatCode = PicFormat.BuildSASCodeForRange(DefinitionParams, _PictureType)

            ' Show the form.
            dlg.ShowDialog(Me)

            ' Save the current test values for use next time.
            _TestValues = dlg.TestValues
        End Sub

        Private Sub chkUseOtherLabel_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles chkUseOtherLabel.CheckedChanged
            txtOtherLabel.Enabled = chkUseOtherLabel.Checked
            ' Make any button states changes based on this new selection.
            UpdateButtonStates()
            ' Tell those that are interested that one of the 
            ' properties has changed. 
            RaiseEvent ParametersUpdated(Me, System.EventArgs.Empty)
        End Sub

        Private Sub txtOtherLabel_Validating(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles txtOtherLabel.Validating
            If (chkUseOtherLabel.Checked = True) Then
                ' Tell those that are interested that one of the 
                ' properties has changed. 
                RaiseEvent ParametersUpdated(Me, System.EventArgs.Empty)
            End If
        End Sub

        Private Sub chkUseMissingLabel_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles chkUseMissingLabel.CheckedChanged
            txtMissingLabel.Enabled = chkUseMissingLabel.Checked
            ' Make any button states changes based on this new selection.
            UpdateButtonStates()
            ' Tell those that are interested that one of the 
            ' properties has changed. 
            RaiseEvent ParametersUpdated(Me, System.EventArgs.Empty)
        End Sub

        Private Sub txtMissingLabel_Validating(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles txtMissingLabel.Validating
            If (chkUseMissingLabel.Checked = True) Then
                ' Tell those that are interested that one of the 
                ' properties has changed. 
                RaiseEvent ParametersUpdated(Me, System.EventArgs.Empty)
            End If
        End Sub
#End Region

#Region " Private Helper Methods "
        ' Sets the enabled/disabled states of the various pushbuttons
        ' depending on the current selected item and the ranges
        ' currently defined.
        Private Sub UpdateButtonStates()
            Dim selItems As ListView.SelectedListViewItemCollection = lvwRanges.SelectedItems

            ' If the number of items selected in the list view control
            ' is not 1, then we disable the add, modify and remove
            ' buttons.
            If (selItems.Count <> 1) Then
                btnAddMask.Enabled = False
                btnModifyMask.Enabled = False
                btnRemoveMask.Enabled = False
            Else
                Dim ri As RangeItem
                ri = selItems(0).Tag
                If (ri.Mask.Trim() = String.Empty And ri.NoEdit = False) Then
                    btnAddMask.Enabled = True
                    btnModifyMask.Enabled = False
                    btnRemoveMask.Enabled = False
                Else
                    btnAddMask.Enabled = False
                    btnModifyMask.Enabled = True
                    btnRemoveMask.Enabled = True
                End If
            End If

            ' There has to be at least two ranges before the Remove All
            ' button is enabled.
            btnRemoveAllMasks.Enabled = (lvwRanges.Items.Count > 1)
            ' If the panel is ready to run, then the Test button is
            ' ready to be pushed!
            btnTest.Enabled = IsAbleToBeRun

            '
            ' NOTE: IT IS BELIEVED THAT THE FOLLOWING STATEMENT IS NOT NEEDED
            '
            ' RaiseEvent ParametersUpdated(Me, System.EventArgs.Empty)
        End Sub

        ' This helper method takes a list view item with RangeItem object
        ' in the Tag member and populates the list view item's subitems based
        ' on the values in the RangeItem object.
        Private Sub PopulateListItem(ByVal listViewItem As ListViewItem)
            Dim riWork As RangeItem = CType(listViewItem.Tag, RangeItem)
            ' Determine if the item is an undefined range.
            Dim undefinedItem As Boolean = (riWork.Mask.Trim() = String.Empty And riWork.NoEdit = False)

            ' We want to be in control of the subitem font styles
            ' on an individual subitem basis.
            listViewItem.UseItemStyleForSubItems = False

            ' Make sure we have enough subitems defined for the
            ' current list view item.
            If (listViewItem.SubItems.Count < lvwRanges.Columns.Count) Then
                Dim colIndex As Integer
                For colIndex = listViewItem.SubItems.Count To lvwRanges.Columns.Count - 1
                    listViewItem.SubItems.Add(String.Empty)
                Next colIndex
            End If

            ' Configure the Start Value subitem.
            If (riWork.Low = True) Then
                listViewItem.SubItems.Item(0).Text = "Low"
                listViewItem.SubItems.Item(0).Font = New Font(lvwRanges.Font, FontStyle.Italic)
            Else
                ' The format of the value displayed depends on the current
                ' PictureType setting.
                Select Case (_PictureType)
                    Case 1
                        ' Date
                        listViewItem.SubItems.Item(0).Text = UtilityMethods.SASDateToString(CInt(riWork.StartValue), False)
                    Case 2
                        ' Time
                        listViewItem.SubItems.Item(0).Text = UtilityMethods.SASTimeToString(riWork.StartValue, False)
                    Case 3
                        ' DateTime
                        listViewItem.SubItems.Item(0).Text = UtilityMethods.SASDateTimeToString(riWork.StartValue, False)
                    Case Else
                        ' Anything else is just formatted as a number.
                        listViewItem.SubItems.Item(0).Text = riWork.StartValue.ToString()
                End Select
                ' Use the standard font for the listview control.
                listViewItem.SubItems.Item(0).Font = lvwRanges.Font
            End If

            ' Format the Include Start Value subitem
            If (riWork.Low = True) Then
                ' If "LOW" is displayed then the include is implied. 
                listViewItem.SubItems.Item(1).Text = String.Empty
            Else
                If (riWork.IncludeStartValue = True) Then
                    listViewItem.SubItems.Item(1).Text = "Yes"
                Else
                    listViewItem.SubItems.Item(1).Text = "No"
                End If
            End If

            ' Configure the End Value subitem.
            If (riWork.High = True) Then
                listViewItem.SubItems.Item(2).Text = "High"
                listViewItem.SubItems.Item(2).Font = New Font(lvwRanges.Font, FontStyle.Italic)
            Else
                ' The format of the value displayed depends on the current
                ' PictureType setting.
                Select Case (_PictureType)
                    Case 1
                        ' Date
                        listViewItem.SubItems.Item(2).Text = UtilityMethods.SASDateToString(CInt(riWork.EndValue), False)
                    Case 2
                        ' Time
                        listViewItem.SubItems.Item(2).Text = UtilityMethods.SASTimeToString(riWork.EndValue, False)
                    Case 3
                        ' DateTime
                        listViewItem.SubItems.Item(2).Text = UtilityMethods.SASDateTimeToString(riWork.EndValue, False)
                    Case Else
                        ' Anything else is just formatted as a number.
                        listViewItem.SubItems.Item(2).Text = riWork.EndValue.ToString()
                End Select
                ' Use the standard font for the listview control.
                listViewItem.SubItems.Item(2).Font = lvwRanges.Font
            End If

            ' Format the Include End Value subitem
            If (riWork.High = True) Then
                ' If "HIGH" is displayed then the include is implied. 
                listViewItem.SubItems.Item(3).Text = String.Empty
            Else
                If (riWork.IncludeEndValue = True) Then
                    listViewItem.SubItems.Item(3).Text = "Yes"
                Else
                    listViewItem.SubItems.Item(3).Text = "No"
                End If
            End If

            ' Add the Mask string subitem
            If (undefinedItem = True) Then
                ' If the range is undefined then we mark it as such.
                listViewItem.SubItems.Item(4).Text = "Undefined"
                listViewItem.SubItems.Item(4).Font = New Font(lvwRanges.Font, FontStyle.Italic)
            Else
                listViewItem.SubItems.Item(4).Text = riWork.Mask
                listViewItem.SubItems.Item(4).Font = lvwRanges.Font
            End If

            ' Add the NOEDIT subitem
            If (undefinedItem = True) Then
                listViewItem.SubItems.Item(5).Text = String.Empty
            Else
                If (riWork.NoEdit = True) Then
                    listViewItem.SubItems.Item(5).Text = "Yes"
                Else
                    listViewItem.SubItems.Item(5).Text = "No"
                End If
            End If
            listViewItem.SubItems.Item(5).Font = lvwRanges.Font

            ' Add the Prefix subitem
            If (undefinedItem = True) Then
                listViewItem.SubItems.Item(6).Text = String.Empty
                listViewItem.SubItems.Item(6).Font = lvwRanges.Font
            Else
                If (riWork.UsePrefix = False Or riWork.Prefix.Trim() = String.Empty) Then
                    listViewItem.SubItems.Item(6).Text = "None"
                    listViewItem.SubItems.Item(6).Font = New Font(lvwRanges.Font, FontStyle.Italic)
                Else
                    listViewItem.SubItems.Item(6).Text = riWork.Prefix
                    listViewItem.SubItems.Item(6).Font = lvwRanges.Font
                End If
            End If

            ' Add the Fill Character subitem
            If (undefinedItem = True) Then
                listViewItem.SubItems.Item(7).Text = String.Empty
                listViewItem.SubItems.Item(7).Font = lvwRanges.Font
            Else
                If (riWork.UseFillChar = False Or riWork.FillChar = " ") Then
                    listViewItem.SubItems.Item(7).Text = "None"
                    listViewItem.SubItems.Item(7).Font = New Font(lvwRanges.Font, FontStyle.Italic)
                Else
                    listViewItem.SubItems.Item(7).Text = riWork.FillChar.ToString()
                    listViewItem.SubItems.Item(7).Font = lvwRanges.Font
                End If
            End If

            ' Add the Multiplier subitem
            If (undefinedItem = True) Then
                listViewItem.SubItems.Item(8).Text = String.Empty
                listViewItem.SubItems.Item(8).Font = lvwRanges.Font
            Else
                If (riWork.UseMultiplier = False Or riWork.Multiplier = 1.0) Then
                    listViewItem.SubItems.Item(8).Text = "None"
                    listViewItem.SubItems.Item(8).Font = New Font(lvwRanges.Font, FontStyle.Italic)
                Else
                    listViewItem.SubItems.Item(8).Text = riWork.Multiplier.ToString()
                    listViewItem.SubItems.Item(8).Font = lvwRanges.Font
                End If
            End If
        End Sub

        Private Sub InitializeData()
            ' Remove all current items from the collection.
            Dim lvi As ListViewItem
            For Each lvi In lvwRanges.Items
                lvi.Tag = Nothing
            Next
            lvwRanges.Items.Clear()

            ' Initialize the collection with the first
            ' data item which encompases all values.
            Dim riInitial As RangeItem = New RangeItem
            Dim liInitialItem As ListViewItem = New ListViewItem
            liInitialItem.Tag = riInitial
            PopulateListItem(liInitialItem)
            lvwRanges.Items.Add(liInitialItem)
            liInitialItem.Selected = True
        End Sub
#End Region

    End Class
End Namespace