' =========================================================
'   
'                Create Picture Format Task
'
'                          a SAS Custom Task Example 
'
' Copyright (C) 2006 by SAS Institute Inc., Cary, NC 27513
'
' ---------------------------------------------------------
' This panel provides the UI to allow the user to change
' the general options for the format. These options include
' The server and library to be used to store the format, 
' minimum, maximum and default widths for the format,
' format type and name.
' =========================================================

Imports System.Text
Imports System.Text.RegularExpressions
Imports System.Windows.Forms

Namespace SASVB.CustomTasks
    Public Class GeneralPanel
        Inherits System.Windows.Forms.UserControl

#Region " Constants "
        Private Const MAXIMUM_WidthValue As Integer = 40

        Private Const DEFAULT_DefaultWidth As Integer = 12
        Private Const DEFAULT_MinimumWidth As Integer = 1
        Private Const DEFAULT_MaximumWidth As Integer = MAXIMUM_WidthValue
#End Region

#Region " Private Member Variables "
        Private _DefaultWidth As Integer = DEFAULT_DefaultWidth
        Private _MinimumWidth As Integer = DEFAULT_MinimumWidth
        Private _MaximumWidth As Integer = DEFAULT_MaximumWidth

        Private _ServerName As String = String.Empty
        Private _LibraryName As String = String.Empty

        Private _Consumer As SAS.Shared.AddIns.ISASTaskConsumer = Nothing
#End Region

#Region " Properties "
        Public Property Consumer() As SAS.Shared.AddIns.ISASTaskConsumer2
            Get
                Return _Consumer
            End Get
            Set(ByVal Value As SAS.Shared.AddIns.ISASTaskConsumer2)
                ' We only need to do something if there has been a
                ' change in the consumer.
                If Not (_Consumer Is Value) Then
                    ' Save the new consumer value.
                    _Consumer = Value
                    If Not (_Consumer Is Nothing) Then
                        ' The value supplied is not NULL so we need to
                        ' prepare to load the server names defined by
                        ' the consumer.
                        Dim currentServer As String
                        If (_ServerName = String.Empty) Then
                            ' If there is no current server then we use
                            ' the currently assigned server.
                            currentServer = _Consumer.AssignedServer.Trim()
                        Else
                            currentServer = _ServerName
                        End If

                        ' Now actually do the server name enumeration and
                        ' provide the server name that we wish to have
                        ' selected as the current server.
                        If (LoadServers(currentServer) = True) Then
                            ' Since we appear to have successfully loaded
                            ' all the server names and selected the current
                            ' server within the list, it's now time to load
                            ' the libraries.
                            _ServerName = currentServer
                            LoadLibraries()
                            If (_LibraryName <> String.Empty) Then
                                ' Now we need to try to select the current library
                                ' within the list.
                                Dim libraryIdx As Int32
                                While (libraryIdx < combLibrary.Items.Count)
                                    If (CType(combLibrary.Items(libraryIdx), String).Trim() = _LibraryName.Trim()) Then
                                        libraryIdx = libraryIdx + 10000
                                    Else
                                        libraryIdx = libraryIdx + 1
                                    End If
                                End While

                                If (libraryIdx >= 10000) Then
                                    ' Select the library in the combobox
                                    combLibrary.SelectedIndex = libraryIdx - 10000
                                Else
                                    _LibraryName = combLibrary.SelectedText
                                End If
                            End If
                        Else
                            ' Because we couldn't load the servers and/or
                            ' select the current server, then we need to
                            ' clear the list of available libraries.
                            combLibrary.Items.Clear()
                        End If
                    Else
                        ' The consumer was NULL so we need to clear the server
                        ' and library comboboxes.
                        combServer.Items.Clear()
                        combLibrary.Items.Clear()
                    End If
                End If
            End Set
        End Property

        Public Property GeneralParams() As GeneralParameters
            Get
                ' We don't keep a copy of the GeneralParameters object
                ' to be returned. Instead, we create it dynamically
                ' whenever it is needed.
                Dim returnGP As GeneralParameters = New GeneralParameters

                returnGP.FormatName = txtFormatName.Text
                returnGP.PictureType = combPictureType.SelectedIndex
                returnGP.ServerName = _ServerName
                returnGP.LibraryName = _LibraryName
                returnGP.UseDefaultWidth = chkUseDefaultWidth.Checked
                returnGP.DefaultWidth = _DefaultWidth
                returnGP.UseMinimumWidth = chkUseMinimumWidth.Checked
                returnGP.MinimumWidth = _MinimumWidth
                returnGP.UseMaximumWidth = chkUseMaximumWidth.Checked
                returnGP.MaximumWidth = _MaximumWidth

                Return returnGP
            End Get
            Set(ByVal Value As GeneralParameters)
                ' Load up all of the form's controls with the associated
                ' values from the supplied GeneralParameters object.

                txtFormatName.Text = Value.FormatName.Trim()

                combPictureType.SelectedIndex = Value.PictureType

                'Server name
                If (Value.ServerName.Trim().Length > 0 And _
                    _ServerName <> Value.ServerName.Trim()) Then
                    If Not (_Consumer Is Nothing) Then
                        ' Check that the server name is in the list
                        ' of available servers
                        Dim serverIdx As Int32
                        While (serverIdx < combServer.Items.Count)
                            If (CType(combServer.Items(serverIdx), String).Trim() = Value.ServerName.Trim()) Then
                                serverIdx = serverIdx + 10000
                            Else
                                serverIdx = serverIdx + 1
                            End If
                        End While

                        If (serverIdx < 10000) Then
                            MessageBox.Show(Me, "The requested server """ + Value.ServerName + """ is not defined within" + vbCrLf + _
                                                "the current repository.", "Cannot Locate Server", MessageBoxButtons.OK, MessageBoxIcon.Information)
                            Return
                        End If

                        ' Select the server in the combobox
                        _ServerName = Value.ServerName.Trim()
                        combServer.SelectedIndex = serverIdx - 10000
                    Else
                        _ServerName = Value.ServerName.Trim()
                    End If
                End If

                ' Library Name
                If (Value.LibraryName.Trim().Length > 0 And _
                    combLibrary.SelectedItem.ToString() <> Value.LibraryName.Trim()) Then
                    If Not (_Consumer Is Nothing) Then
                        ' Check that the library name is in the list
                        ' of available libraries
                        Dim libraryIdx As Int32
                        While (libraryIdx < combLibrary.Items.Count)
                            If (CType(combLibrary.Items(libraryIdx), String).Trim() = Value.LibraryName.Trim()) Then
                                libraryIdx = libraryIdx + 10000
                            Else
                                libraryIdx = libraryIdx + 1
                            End If
                        End While

                        If (libraryIdx < 10000) Then
                            MessageBox.Show(Me, "The requested library """ + Value.LibraryName + """ is not defined on the" + vbCrLf + _
                                                "server " + _ServerName + ".", "Cannot Locate Library", MessageBoxButtons.OK, MessageBoxIcon.Information)
                            Return
                        End If

                        ' Select the library in the combobox
                        _LibraryName = Value.LibraryName.Trim()
                        combLibrary.SelectedIndex = libraryIdx - 10000
                    Else
                        _LibraryName = Value.LibraryName.Trim()
                    End If
                End If

                ' Use Default Width
                chkUseDefaultWidth.Checked = Value.UseDefaultWidth
                ' Default Width value
                If (_DefaultWidth < 1 Or _DefaultWidth > MAXIMUM_WidthValue) Then
                    Throw New ArgumentOutOfRangeException("DefaultWidth", Value.DefaultWidth, "The Default Width property value must be between 1 and 40")
                End If
                If (_MinimumWidth > Value.DefaultWidth) Then
                    _MinimumWidth = Value.DefaultWidth
                    If (chkUseMinimumWidth.Checked = True) Then
                        txtMinimumWidth.Text = _MinimumWidth.ToString()
                    End If
                End If
                If (_MaximumWidth < Value.DefaultWidth) Then
                    _MaximumWidth = Value.DefaultWidth
                    If (chkUseMaximumWidth.Checked = True) Then
                        txtMaximumWidth.Text = _MaximumWidth.ToString()
                    End If
                End If
                _DefaultWidth = Value.DefaultWidth
                If (chkUseDefaultWidth.Checked = True) Then
                    txtDefaultWidth.Text = _DefaultWidth.ToString()
                End If

                ' Use Minimum Width
                chkUseMinimumWidth.Checked = Value.UseMinimumWidth

                ' Minimum Width value
                If (_MinimumWidth < 1 Or _MinimumWidth > MAXIMUM_WidthValue) Then
                    Throw New ArgumentOutOfRangeException("MinimumWidth", Value.MinimumWidth, "The Minimum Width property value must be between 1 and 40")
                End If
                If (_MaximumWidth < Value.MinimumWidth) Then
                    _MaximumWidth = Value.MinimumWidth
                    If (chkUseMaximumWidth.Checked = True) Then
                        txtMaximumWidth.Text = _MaximumWidth.ToString()
                    End If
                End If
                If (_DefaultWidth < Value.MinimumWidth) Then
                    _DefaultWidth = Value.MinimumWidth
                    If (chkUseDefaultWidth.Checked = True) Then
                        txtDefaultWidth.Text = _DefaultWidth.ToString()
                    End If
                End If
                _MinimumWidth = Value.MinimumWidth
                If (chkUseMinimumWidth.Checked = True) Then
                    txtMinimumWidth.Text = _MinimumWidth.ToString()
                End If

                ' Use Maximum Width
                chkUseMaximumWidth.Checked = Value.UseMaximumWidth

                ' Maximum Width value
                If (_MaximumWidth < 1 Or _MaximumWidth > MAXIMUM_WidthValue) Then
                    Throw New ArgumentOutOfRangeException("MaximumWidth", Value.MaximumWidth, "The Maximum Width property value must be between 1 and 40")
                End If
                If (_MinimumWidth > Value.MaximumWidth) Then
                    _MinimumWidth = Value.MaximumWidth
                    If (chkUseMinimumWidth.Checked = True) Then
                        txtMinimumWidth.Text = _MinimumWidth.ToString()
                    End If
                End If
                If (_DefaultWidth > Value.MaximumWidth) Then
                    _DefaultWidth = Value.MaximumWidth
                    If (chkUseDefaultWidth.Checked = True) Then
                        txtDefaultWidth.Text = _DefaultWidth.ToString()
                    End If
                End If
                _MaximumWidth = Value.MaximumWidth
                If (chkUseMaximumWidth.Checked = True) Then
                    txtMaximumWidth.Text = _MaximumWidth.ToString()
                End If
            End Set
        End Property

        Public ReadOnly Property IsAbleToBeRun() As Boolean
            Get
                ' We are in a runnable state if a valid format name
                ' has been specified, and each of the picture type,
                ' server and library comboboxes has an item selected.
                Return (txtFormatName.Text.Trim().Length > 0 And _
                        combPictureType.SelectedIndex <> -1 And _
                        combServer.SelectedIndex <> -1 And _
                        combLibrary.SelectedIndex <> -1)
            End Get
        End Property
#End Region

#Region " Public Events "
        Public Event ParametersUpdated(ByVal sender As Object, ByVal ea As System.EventArgs)
        Public Event PictureTypeChanged(ByVal sender As Object, ByVal newType As Int32)
        Public Event ServerNameChanged(ByVal sender As Object, ByVal newServerName As String)
        Public Event DefaultWidthChanged(ByVal sender As Object, ByVal newDefaultWidth As Int32)
#End Region

#Region " Windows Form Designer generated code "

        Public Sub New()
            MyBase.New()

            'This call is required by the Windows Form Designer.
            InitializeComponent()

            'Initialize the Picture Type combobox to Numeric
            combPictureType.SelectedItem = 0

            ' Start off with a blank format name and default width
            txtFormatName.Text = String.Empty
            txtDefaultWidth.Text = String.Empty
            'Initialize the Minimum and Maximum value fields
            txtMinimumWidth.Text = _MinimumWidth.ToString()
            txtMaximumWidth.Text = _MaximumWidth.ToString()

        End Sub

        'UserControl overrides dispose to clean up the component list.
        Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
            If disposing Then
                If Not (components Is Nothing) Then
                    components.Dispose()
                End If
            End If
            MyBase.Dispose(disposing)
        End Sub

        'Required by the Windows Form Designer
        Private components As System.ComponentModel.IContainer

        'NOTE: The following procedure is required by the Windows Form Designer
        'It can be modified using the Windows Form Designer.  
        'Do not modify it using the code editor.
        Friend WithEvents lblFormatName As System.Windows.Forms.Label
        Friend WithEvents gboxWidths As System.Windows.Forms.GroupBox
        Friend WithEvents lblDefaultWidth As System.Windows.Forms.Label
        Friend WithEvents lblMinimumWidth As System.Windows.Forms.Label
        Friend WithEvents lblMaximumWidth As System.Windows.Forms.Label
        Public WithEvents txtFormatName As System.Windows.Forms.TextBox
        Public WithEvents txtDefaultWidth As System.Windows.Forms.TextBox
        Public WithEvents txtMinimumWidth As System.Windows.Forms.TextBox
        Public WithEvents txtMaximumWidth As System.Windows.Forms.TextBox
        Friend WithEvents chkUseDefaultWidth As System.Windows.Forms.CheckBox
        Friend WithEvents chkUseMinimumWidth As System.Windows.Forms.CheckBox
        Friend WithEvents chkUseMaximumWidth As System.Windows.Forms.CheckBox
        Friend WithEvents gboxFormatLocation As System.Windows.Forms.GroupBox
        Friend WithEvents lblServer As System.Windows.Forms.Label
        Friend WithEvents combServer As System.Windows.Forms.ComboBox
        Friend WithEvents lblLibrary As System.Windows.Forms.Label
        Friend WithEvents combLibrary As System.Windows.Forms.ComboBox
        Friend WithEvents combPictureType As System.Windows.Forms.ComboBox
        Friend WithEvents lblPictureType As System.Windows.Forms.Label
        <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
            Me.lblFormatName = New System.Windows.Forms.Label
            Me.txtFormatName = New System.Windows.Forms.TextBox
            Me.gboxWidths = New System.Windows.Forms.GroupBox
            Me.chkUseDefaultWidth = New System.Windows.Forms.CheckBox
            Me.txtDefaultWidth = New System.Windows.Forms.TextBox
            Me.lblDefaultWidth = New System.Windows.Forms.Label
            Me.lblMinimumWidth = New System.Windows.Forms.Label
            Me.txtMinimumWidth = New System.Windows.Forms.TextBox
            Me.lblMaximumWidth = New System.Windows.Forms.Label
            Me.txtMaximumWidth = New System.Windows.Forms.TextBox
            Me.chkUseMinimumWidth = New System.Windows.Forms.CheckBox
            Me.chkUseMaximumWidth = New System.Windows.Forms.CheckBox
            Me.gboxFormatLocation = New System.Windows.Forms.GroupBox
            Me.combLibrary = New System.Windows.Forms.ComboBox
            Me.lblLibrary = New System.Windows.Forms.Label
            Me.combServer = New System.Windows.Forms.ComboBox
            Me.lblServer = New System.Windows.Forms.Label
            Me.lblPictureType = New System.Windows.Forms.Label
            Me.combPictureType = New System.Windows.Forms.ComboBox
            Me.gboxWidths.SuspendLayout()
            Me.gboxFormatLocation.SuspendLayout()
            Me.SuspendLayout()
            '
            'lblFormatName
            '
            Me.lblFormatName.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.lblFormatName.Location = New System.Drawing.Point(8, 6)
            Me.lblFormatName.Name = "lblFormatName"
            Me.lblFormatName.Size = New System.Drawing.Size(120, 16)
            Me.lblFormatName.TabIndex = 0
            Me.lblFormatName.Text = "F&ormat name:"
            '
            'txtFormatName
            '
            Me.txtFormatName.Location = New System.Drawing.Point(136, 4)
            Me.txtFormatName.Name = "txtFormatName"
            Me.txtFormatName.Size = New System.Drawing.Size(192, 20)
            Me.txtFormatName.TabIndex = 1
            Me.txtFormatName.Text = ""
            '
            'gboxWidths
            '
            Me.gboxWidths.Controls.Add(Me.chkUseDefaultWidth)
            Me.gboxWidths.Controls.Add(Me.txtDefaultWidth)
            Me.gboxWidths.Controls.Add(Me.lblDefaultWidth)
            Me.gboxWidths.Controls.Add(Me.lblMinimumWidth)
            Me.gboxWidths.Controls.Add(Me.txtMinimumWidth)
            Me.gboxWidths.Controls.Add(Me.lblMaximumWidth)
            Me.gboxWidths.Controls.Add(Me.txtMaximumWidth)
            Me.gboxWidths.Controls.Add(Me.chkUseMinimumWidth)
            Me.gboxWidths.Controls.Add(Me.chkUseMaximumWidth)
            Me.gboxWidths.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.gboxWidths.Location = New System.Drawing.Point(8, 56)
            Me.gboxWidths.Name = "gboxWidths"
            Me.gboxWidths.Size = New System.Drawing.Size(376, 176)
            Me.gboxWidths.TabIndex = 2
            Me.gboxWidths.TabStop = False
            Me.gboxWidths.Text = "Width values"
            '
            'chkUseDefaultWidth
            '
            Me.chkUseDefaultWidth.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.chkUseDefaultWidth.Location = New System.Drawing.Point(16, 24)
            Me.chkUseDefaultWidth.Name = "chkUseDefaultWidth"
            Me.chkUseDefaultWidth.Size = New System.Drawing.Size(352, 16)
            Me.chkUseDefaultWidth.TabIndex = 0
            Me.chkUseDefaultWidth.Text = "Specify a d&efault width to use if none is supplied"
            '
            'txtDefaultWidth
            '
            Me.txtDefaultWidth.Enabled = False
            Me.txtDefaultWidth.Location = New System.Drawing.Point(128, 46)
            Me.txtDefaultWidth.Name = "txtDefaultWidth"
            Me.txtDefaultWidth.Size = New System.Drawing.Size(80, 20)
            Me.txtDefaultWidth.TabIndex = 2
            Me.txtDefaultWidth.Text = ""
            '
            'lblDefaultWidth
            '
            Me.lblDefaultWidth.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.lblDefaultWidth.Location = New System.Drawing.Point(32, 48)
            Me.lblDefaultWidth.Name = "lblDefaultWidth"
            Me.lblDefaultWidth.Size = New System.Drawing.Size(96, 16)
            Me.lblDefaultWidth.TabIndex = 1
            Me.lblDefaultWidth.Text = "&Default:"
            '
            'lblMinimumWidth
            '
            Me.lblMinimumWidth.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.lblMinimumWidth.Location = New System.Drawing.Point(32, 96)
            Me.lblMinimumWidth.Name = "lblMinimumWidth"
            Me.lblMinimumWidth.Size = New System.Drawing.Size(96, 16)
            Me.lblMinimumWidth.TabIndex = 4
            Me.lblMinimumWidth.Text = "&Minimum:"
            '
            'txtMinimumWidth
            '
            Me.txtMinimumWidth.Enabled = False
            Me.txtMinimumWidth.Location = New System.Drawing.Point(128, 94)
            Me.txtMinimumWidth.Name = "txtMinimumWidth"
            Me.txtMinimumWidth.Size = New System.Drawing.Size(80, 20)
            Me.txtMinimumWidth.TabIndex = 5
            Me.txtMinimumWidth.Text = ""
            '
            'lblMaximumWidth
            '
            Me.lblMaximumWidth.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.lblMaximumWidth.Location = New System.Drawing.Point(32, 144)
            Me.lblMaximumWidth.Name = "lblMaximumWidth"
            Me.lblMaximumWidth.Size = New System.Drawing.Size(96, 16)
            Me.lblMaximumWidth.TabIndex = 7
            Me.lblMaximumWidth.Text = "Ma&ximum:"
            '
            'txtMaximumWidth
            '
            Me.txtMaximumWidth.Enabled = False
            Me.txtMaximumWidth.Location = New System.Drawing.Point(128, 142)
            Me.txtMaximumWidth.Name = "txtMaximumWidth"
            Me.txtMaximumWidth.Size = New System.Drawing.Size(80, 20)
            Me.txtMaximumWidth.TabIndex = 8
            Me.txtMaximumWidth.Text = ""
            '
            'chkUseMinimumWidth
            '
            Me.chkUseMinimumWidth.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.chkUseMinimumWidth.Location = New System.Drawing.Point(16, 72)
            Me.chkUseMinimumWidth.Name = "chkUseMinimumWidth"
            Me.chkUseMinimumWidth.Size = New System.Drawing.Size(352, 16)
            Me.chkUseMinimumWidth.TabIndex = 3
            Me.chkUseMinimumWidth.Text = "Specify a minim&um width value"
            '
            'chkUseMaximumWidth
            '
            Me.chkUseMaximumWidth.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.chkUseMaximumWidth.Location = New System.Drawing.Point(16, 120)
            Me.chkUseMaximumWidth.Name = "chkUseMaximumWidth"
            Me.chkUseMaximumWidth.Size = New System.Drawing.Size(352, 16)
            Me.chkUseMaximumWidth.TabIndex = 6
            Me.chkUseMaximumWidth.Text = "Specify a m&aximum width value"
            '
            'gboxFormatLocation
            '
            Me.gboxFormatLocation.Controls.Add(Me.combLibrary)
            Me.gboxFormatLocation.Controls.Add(Me.lblLibrary)
            Me.gboxFormatLocation.Controls.Add(Me.combServer)
            Me.gboxFormatLocation.Controls.Add(Me.lblServer)
            Me.gboxFormatLocation.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.gboxFormatLocation.Location = New System.Drawing.Point(8, 240)
            Me.gboxFormatLocation.Name = "gboxFormatLocation"
            Me.gboxFormatLocation.Size = New System.Drawing.Size(376, 80)
            Me.gboxFormatLocation.TabIndex = 3
            Me.gboxFormatLocation.TabStop = False
            Me.gboxFormatLocation.Text = "Location to store format"
            '
            'combLibrary
            '
            Me.combLibrary.Location = New System.Drawing.Point(128, 50)
            Me.combLibrary.Name = "combLibrary"
            Me.combLibrary.Size = New System.Drawing.Size(192, 21)
            Me.combLibrary.TabIndex = 3
            '
            'lblLibrary
            '
            Me.lblLibrary.Anchor = CType(((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.lblLibrary.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.lblLibrary.Location = New System.Drawing.Point(16, 52)
            Me.lblLibrary.Name = "lblLibrary"
            Me.lblLibrary.Size = New System.Drawing.Size(112, 16)
            Me.lblLibrary.TabIndex = 2
            Me.lblLibrary.Text = "&Library:"
            '
            'combServer
            '
            Me.combServer.Location = New System.Drawing.Point(128, 22)
            Me.combServer.Name = "combServer"
            Me.combServer.Size = New System.Drawing.Size(192, 21)
            Me.combServer.TabIndex = 1
            '
            'lblServer
            '
            Me.lblServer.Anchor = CType(((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.lblServer.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.lblServer.Location = New System.Drawing.Point(16, 24)
            Me.lblServer.Name = "lblServer"
            Me.lblServer.Size = New System.Drawing.Size(112, 16)
            Me.lblServer.TabIndex = 0
            Me.lblServer.Text = "&Server:"
            '
            'lblPictureType
            '
            Me.lblPictureType.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.lblPictureType.Location = New System.Drawing.Point(8, 30)
            Me.lblPictureType.Name = "lblPictureType"
            Me.lblPictureType.Size = New System.Drawing.Size(120, 16)
            Me.lblPictureType.TabIndex = 4
            Me.lblPictureType.Text = "Picture &type:"
            '
            'combPictureType
            '
            Me.combPictureType.Items.AddRange(New Object() {"Numeric", "Date", "Time", "DateTime"})
            Me.combPictureType.Location = New System.Drawing.Point(136, 28)
            Me.combPictureType.Name = "combPictureType"
            Me.combPictureType.Size = New System.Drawing.Size(192, 21)
            Me.combPictureType.TabIndex = 5
            '
            'GeneralPanel
            '
            Me.Controls.Add(Me.combPictureType)
            Me.Controls.Add(Me.lblPictureType)
            Me.Controls.Add(Me.gboxFormatLocation)
            Me.Controls.Add(Me.gboxWidths)
            Me.Controls.Add(Me.txtFormatName)
            Me.Controls.Add(Me.lblFormatName)
            Me.Name = "GeneralPanel"
            Me.Size = New System.Drawing.Size(608, 328)
            Me.Tag = "Type"
            Me.gboxWidths.ResumeLayout(False)
            Me.gboxFormatLocation.ResumeLayout(False)
            Me.ResumeLayout(False)

        End Sub

#End Region

#Region " Child Control Event Handlers "
        Private Sub chkUseDefaultWidth_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkUseDefaultWidth.CheckedChanged
            txtDefaultWidth.Enabled = chkUseDefaultWidth.Checked
            ' If the user wants to specify the default width then we
            ' want to show the current value in the textbox. Otherwise
            ' we want the textbox to be blank.
            If (chkUseDefaultWidth.Checked = True) Then
                txtDefaultWidth.Text = _DefaultWidth.ToString()
            Else
                txtDefaultWidth.Text = String.Empty
            End If

            ' Tell anyone that's interested that one of the panel's
            ' properties has changed.
            RaiseEvent ParametersUpdated(Me, System.EventArgs.Empty)
        End Sub

        Private Sub chkUseMinimumWidth_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkUseMinimumWidth.CheckedChanged
            txtMinimumWidth.Enabled = chkUseMinimumWidth.Checked
            ' If the user wants to specify the minimum width then we
            ' want to show the current value in the textbox. Otherwise
            ' we want the textbox to be blank.
            If (chkUseMinimumWidth.Checked = True) Then
                txtMinimumWidth.Text = _MinimumWidth.ToString()
            Else
                txtMinimumWidth.Text = DEFAULT_MinimumWidth.ToString()
            End If

            ' Tell anyone that's interested that one of the panel's
            ' properties has changed.
            RaiseEvent ParametersUpdated(Me, System.EventArgs.Empty)
        End Sub

        Private Sub chkUseMaximumWidth_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkUseMaximumWidth.CheckedChanged
            txtMaximumWidth.Enabled = chkUseMaximumWidth.Checked
            ' If the user wants to specify the maximum width then we
            ' want to show the current value in the textbox. Otherwise
            ' we want the textbox to be blank.
            If (chkUseMaximumWidth.Checked = True) Then
                txtMaximumWidth.Text = _MaximumWidth.ToString()
            Else
                txtMaximumWidth.Text = DEFAULT_MaximumWidth.ToString()
            End If

            ' Tell anyone that's interested that one of the panel's
            ' properties has changed.
            RaiseEvent ParametersUpdated(Me, System.EventArgs.Empty)
        End Sub

        Private Sub combServer_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles combServer.SelectedIndexChanged
            ' A new server has apparently been selected so we want
            ' to enumerate the available, writeable, libraries on
            ' the newly selected server.
            _ServerName = CType(combServer.SelectedItem, String)
            LoadLibraries()

            ' Tell anyone that's interested that one of the panel's
            ' properties has changed.
            RaiseEvent ParametersUpdated(Me, System.EventArgs.Empty)
            ' Also let them know that it was more specifically the
            ' server name that changed. This is needed primarily to
            ' update the server name used by the test dialog from
            ' the range definition panel.
            RaiseEvent ServerNameChanged(Me, _ServerName)
        End Sub

        Private Sub combLibrary_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles combLibrary.SelectedIndexChanged
            Dim libraryName As String

            ' A new library has been selected, so we want to record
            ' the library name for use later.
            libraryName = CType(combLibrary.SelectedItem, String)
            If (libraryName = String.Empty) Then
                ' We can't have a blank library name, so if it is
                ' blank, we'll force it to WORK.
                _LibraryName = "WORK"
            Else
                _LibraryName = libraryName
            End If
            RaiseEvent ParametersUpdated(Me, System.EventArgs.Empty)
        End Sub

        Private Sub combPictureType_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles combPictureType.SelectedIndexChanged
            ' Let everyone know that the selected picture type has
            ' changed.  This is needed primarily to update the 
            ' displayed values and input field types within the
            ' range definition panel and its range test form. 
            RaiseEvent PictureTypeChanged(Me, combPictureType.SelectedIndex)

            ' Tell anyone that's interested that one of the panel's
            ' properties has changed.
            RaiseEvent ParametersUpdated(Me, System.EventArgs.Empty)
        End Sub
#End Region

#Region " Form Overrides "
        Protected Overrides Sub OnLoad(ByVal e As System.EventArgs)
            ' By the time this Windows form loads we need to have a
            ' valid consumer assigned. If not, then we can't really
            ' continue.
            If (_Consumer Is Nothing) Then
                Throw New Exception("A valid Consumer is required at the time the form is displayed." + vbCrLf + _
                "The consumer's value is currently Nothing.")
            End If

            ' Enumerate the names of the available servers.
            LoadServers(_Consumer.AssignedServer.Trim())
        End Sub
#End Region

#Region " Control Validation Handlers "
        Private Sub txtFormatName_Validating(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles txtFormatName.Validating
            Dim formatName As String

            ' We need to make sure that the value entered for the format
            ' name conforms to the SAS rules for numeric format names.
            ' Those rules are that it must be 1 to 32 characters in
            ' length where the fist character is A to Z or an underscore,
            ' end with A to Z or an underscore. All other characters can be
            ' A to Z, 0 to 9 or an underscore.
            formatName = txtFormatName.Text.Trim()
            ' One thing we will allow at this point is for the format name
            ' to be blank. This is to allow focus to be moved from the
            ' Format Name textbox initially so that other controls can be
            ' manipulated before the format name is specified. The Finish
            ' button will remain disabled until a valid format name has
            ' been entered.
            If (formatName <> String.Empty) Then
                If (Regex.IsMatch(formatName, "^[A-Za-z_]([A-Za-z0-9_]{0,30}[A-Za-z_])*$") = False) Then
                    ' Set the event flag to signify that the validation
                    ' has failed.
                    e.Cancel = True
                    ' Let the user know and tell them the rules.
                    MessageBox.Show(Me, "The format name is not valid. It must meet the following criteria:" + vbCrLf + _
                                        "    * 1 to 32 characters in length" + vbCrLf + _
                                        "    * The first character must be ""A"" to ""Z"" or an underscore" + vbCrLf + _
                                        "    * The last character must be ""A"" to ""Z"" or an underscore" + vbCrLf + _
                                        "    * All other characters may be ""A"" to ""Z"", ""0"" to ""9"" or an underscore", _
                                        "Format Name Error", MessageBoxButtons.OK, MessageBoxIcon.Information)
                    Return
                End If
            End If

            ' Everything about the format name appears to be in order,
            ' so we set the event flag to indicate that the
            ' validation has succeeded.
            e.Cancel = False

            ' Let the world know that one of the parameters (properties)
            ' has been modified.
            RaiseEvent ParametersUpdated(Me, System.EventArgs.Empty)
        End Sub

        Private Sub txtDefaultWidth_Validating(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles txtDefaultWidth.Validating
            Dim defaultWidthString As String
            Dim defaultWidth As Int32

            defaultWidthString = txtDefaultWidth.Text.Trim()
            ' The value entered for the default width must be a
            ' valid integer.
            If (Regex.IsMatch(defaultWidthString, "^\d+[+-]?$|^[+-]?\d+$") = False) Then
                ' Set the event flag to signify that the validation
                ' has failed.
                e.Cancel = True
                MessageBox.Show(Me, "The default width value is not valid." + vbCrLf + _
                                    "It must be an integer value within the range " + _
                                    DEFAULT_MinimumWidth.ToString() + " to " + _
                                    DEFAULT_MaximumWidth.ToString() + ".", _
                                    "Default Width Error", MessageBoxButtons.OK, MessageBoxIcon.Information)
                Return
            End If

            defaultWidth = Int32.Parse(defaultWidthString, Globalization.NumberStyles.Integer)
            ' The default width must be >= 1 and <= 40 (both limit
            ' values are specified as Int32 constants).
            If (defaultWidth < DEFAULT_MinimumWidth Or defaultWidth > DEFAULT_MaximumWidth) Then
                ' Set the event flag to signify that the validation
                ' has failed.
                e.Cancel = True
                MessageBox.Show(Me, "The default width value is not valid." + vbCrLf + _
                                    "It must be an integer value within the range " + _
                                    DEFAULT_MinimumWidth.ToString() + " to " + _
                                    DEFAULT_MaximumWidth.ToString() + ".", _
                                    "Default Width Error", MessageBoxButtons.OK, MessageBoxIcon.Information)
                Return
            End If

            ' If a minimum width has been specified and the new default width is less
            ' than the minimum width value, or a maximum width has been specified 
            ' and the new default width is greater than the maximum width value, then
            ' we have a validation error.
            If ((chkUseMinimumWidth.Checked = True And defaultWidth < _MinimumWidth) Or _
                (chkUseMaximumWidth.Checked = True And defaultWidth > _MaximumWidth)) Then
                ' Set the event flag to signify that the validation
                ' has failed.
                e.Cancel = True
                MessageBox.Show(Me, "The default width value is not valid." + vbCrLf + _
                                    "Its value must be greater than or equal to the Minimum width and" + vbCrLf + _
                                    "less than or equal to the Maximum width values.", _
                                    "Default Width Error", MessageBoxButtons.OK, MessageBoxIcon.Information)
                Return
            End If

            ' The default width value appears to be fine,
            ' so we set the event flag to indicate that the
            ' validation has succeeded.
            e.Cancel = False

            ' Store the new default width value.
            _DefaultWidth = defaultWidth

            ' Now let the world know that one of the properties 
            ' (parameters) has changed.
            RaiseEvent ParametersUpdated(Me, System.EventArgs.Empty)
            ' Also tell anyone that's interested (primarily the
            ' test dialog from the range definitions panel) that
            ' the default width is the property that changed.
            RaiseEvent DefaultWidthChanged(Me, _DefaultWidth)
        End Sub

        Private Sub txtMinimumWidth_Validating(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles txtMinimumWidth.Validating
            Dim minimumWidthString As String
            Dim minimumWidth As Int32

            minimumWidthString = txtMinimumWidth.Text.Trim()
            ' The value entered for the minimum width must be a
            ' valid integer.
            If (Regex.IsMatch(minimumWidthString, "^\d+[+-]?$|^[+-]?\d+$") = False) Then
                ' Set the event flag to signify that the validation
                ' has failed.
                e.Cancel = True
                MessageBox.Show(Me, "The minimum width value is not valid." + vbCrLf + _
                                    "It must be an integer value within the range " + _
                                    DEFAULT_MinimumWidth.ToString() + " to " + _
                                    DEFAULT_MaximumWidth.ToString() + ".", _
                                    "Minimum Width Error", MessageBoxButtons.OK, MessageBoxIcon.Information)
                Return
            End If

            minimumWidth = Int32.Parse(minimumWidthString, Globalization.NumberStyles.Integer)
            ' The minimum width must be >= 1 and <= 40 (both limit
            ' values are specified as Int32 constants).
            If (minimumWidth < DEFAULT_MinimumWidth Or minimumWidth > DEFAULT_MaximumWidth) Then
                ' Set the event flag to signify that the validation
                ' has failed.
                e.Cancel = True
                MessageBox.Show(Me, "The minimum width value is not valid." + vbCrLf + _
                                    "It must be an integer value within the range " + _
                                    DEFAULT_MinimumWidth.ToString() + " to " + _
                                    DEFAULT_MaximumWidth.ToString() + ".", _
                                    "Minimum Width Error", MessageBoxButtons.OK, MessageBoxIcon.Information)
                Return
            End If

            ' If a default width has been specified and the new minimum width is 
            ' greater than the default width value, or a maximum width has been
            ' specified and the new minimum width is greater than the maximum width
            ' value, then we have a validation error.
            If ((chkUseDefaultWidth.Checked = True And minimumWidth > _DefaultWidth) Or _
                (chkUseMaximumWidth.Checked = True And minimumWidth > _MaximumWidth)) Then
                ' Set the event flag to signify that the validation
                ' has failed.
                e.Cancel = True
                MessageBox.Show(Me, "The minimum width value is not valid." + vbCrLf + _
                                    "Its value must be less than or equal to both the Default width and" + vbCrLf + _
                                    "the Maximum width values.", _
                                    "Minimum Width Error", MessageBoxButtons.OK, MessageBoxIcon.Information)
                Return
            End If

            ' The minimum width value appears to be fine,
            ' so we set the event flag to indicate that the
            ' validation has succeeded.
            e.Cancel = False

            ' Store the new minimum width value.
            _MinimumWidth = minimumWidth

            ' Now let the world know that one of the properties 
            ' (parameters) has changed.
            RaiseEvent ParametersUpdated(Me, System.EventArgs.Empty)
        End Sub

        Private Sub txtMaximumWidth_Validating(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles txtMaximumWidth.Validating
            Dim maximumWidthString As String
            Dim maximumWidth As Int32

            maximumWidthString = txtMaximumWidth.Text.Trim()
            ' The value entered for the maximum width must be a
            ' valid integer.
            If (Regex.IsMatch(maximumWidthString, "^\d+[+-]?$|^[+-]?\d+$") = False) Then
                ' Set the event flag to signify that the validation
                ' has failed.
                e.Cancel = True
                MessageBox.Show(Me, "The maximum width value is not valid." + vbCrLf + _
                                    "It must be an integer value within the range " + _
                                    DEFAULT_MinimumWidth.ToString() + " to " + _
                                    DEFAULT_MaximumWidth.ToString() + ".", _
                                    "Maximum Width Error", MessageBoxButtons.OK, MessageBoxIcon.Information)
                Return
            End If

            maximumWidth = Int32.Parse(maximumWidthString, Globalization.NumberStyles.Integer)
            ' The maximum width must be >= 1 and <= 40 (both limit
            ' values are specified as Int32 constants).
            If (maximumWidth < DEFAULT_MinimumWidth Or maximumWidth > DEFAULT_MaximumWidth) Then
                ' Set the event flag to signify that the validation
                ' has failed.
                e.Cancel = True
                MessageBox.Show(Me, "The maximum width value is not valid." + vbCrLf + _
                                    "It must be an integer value within the range " + _
                                    DEFAULT_MinimumWidth.ToString() + " to " + _
                                    DEFAULT_MaximumWidth.ToString() + ".", _
                                    "Maximum Width Error", MessageBoxButtons.OK, MessageBoxIcon.Information)
                Return
            End If

            ' If a default width has been specified and the new maximum width is 
            ' less than the default width value, or a minimum width has been
            ' specified and the new maximum width is less than the minimum width
            ' value, then we have a validation error.
            If ((chkUseDefaultWidth.Checked = True And maximumWidth < _DefaultWidth) Or _
                (chkUseMinimumWidth.Checked = True And maximumWidth < _MinimumWidth)) Then
                ' Set the event flag to signify that the validation
                ' has failed.
                e.Cancel = True
                MessageBox.Show(Me, "The maximum width value is not valid." + vbCrLf + _
                                    "Its value must be greater than or equal to both the Default width and" + vbCrLf + _
                                    "the Minimum width values.", _
                                    "Maximum Width Error", MessageBoxButtons.OK, MessageBoxIcon.Information)
                Return
            End If

            ' The maximum width value appears to be fine,
            ' so we set the event flag to indicate that the
            ' validation has succeeded.
            e.Cancel = False

            ' Store the new maximum width value.
            _MaximumWidth = maximumWidth

            ' Now let the world know that one of the properties 
            ' (parameters) has changed.
            RaiseEvent ParametersUpdated(Me, System.EventArgs.Empty)
        End Sub
#End Region

#Region " Internal Helper methods "
        Private Function LoadServers(ByVal newServer As String) As Boolean
            Dim consumer2 As SAS.Shared.AddIns.ISASTaskConsumer2
            Dim ServerList() As String
            Dim serverName As String
            Dim currentServerItem As Int32 = -1
            Dim localServerItem As Int32 = -1


            ' We can't do anything if we haven't got a Consumer to talk to!
            If (_Consumer Is Nothing) Then
                Throw New Exception("A valid Consumer was not available at the time the " + _
                                    "list of servers was to be enumerated. The consumer's " + _
                                    "value is currently Nothing.")
            End If

            ' Now, check if we can actually get some servers back from the consumer.
            If (_Consumer.Servers(ServerList) <= 0) Then
                Throw New Exception("There were no server names returned by the " + _
                                    "consumer.Servers() method. At least one server " + _
                                    "needs to be defined.")
            End If

            ' Clear out the existing names from the Server combobox.
            combServer.Items.Clear()

            ' Load up the names of the currently available servers.
            For Each serverName In ServerList
                combServer.Items.Add(serverName.Trim())
                ' If the current server if the one we've been told is to be
                ' the newly selected server, we just save the index number
                ' for now.
                If (serverName.Trim() = newServer.Trim()) Then
                    currentServerItem = combServer.Items.Count - 1
                End If
                If (serverName.Trim().ToUpper() = "LOCAL") Then
                    localServerItem = combServer.Items.Count - 1
                End If
            Next

            ' If we couldn't locate the new server within the list, then
            ' we attempt to fall back to the Local server.
            If (currentServerItem = -1) Then
                If (localServerItem = -1) Then
                    combServer.SelectedIndex = -1
                    Return False
                End If
                _ServerName = CType(combServer.Items(localServerItem), String)

                combServer.SelectedIndex = localServerItem
            Else
                ' The new server's item was located, so we want to store the
                ' new server name for use elsewhere, ready to return when the
                ' task is finally run. 
                _ServerName = newServer.Trim()

                ' Select the new current server 
                combServer.SelectedIndex = currentServerItem
            End If

            Return True
        End Function

        Private Function LoadLibraries() As Boolean
            Dim consumer2 As SAS.Shared.AddIns.ISASTaskConsumer2
            Dim LibraryList() As String
            Dim libraryName As String
            Dim workLibraryItem As Int32 = -1
            Dim currentLibraryItem As Int32 = -1


            ' Having a Consumer is a fairly essential part of this task.
            If (_Consumer Is Nothing) Then
                Throw New Exception("A valid Consumer is required at the time the form is displayed." + vbCrLf + _
                "The consumer's value is currently Nothing.")
            End If

            ' From the Consumer we want to get an ISASTaskConsumer2 interface.
            consumer2 = CType(_Consumer, SAS.Shared.AddIns.ISASTaskConsumer2)
            If (consumer2 Is Nothing) Then
                Throw New Exception("Unable to obtain an ISASTaskConsumer2 interface from the supplied consumer.")
            End If

            ' Attempt to get the names of all the writable libraries on the server.
            If (consumer2.Libraries(_ServerName, SAS.Shared.AddIns.LibraryType.ReadWrite, LibraryList) <= 0) Then
                Throw New Exception("There were no writeable library names returned by the " + _
                                    "ISASTaskConsumer2.Libraries() method for the server " + _
                                    _ServerName + ". At least one writeable library needs " + _
                                    "to be available.")
            End If

            ' Clear any existing items from the library combo box.
            combLibrary.Items.Clear()

            ' Load up the names of the writeable libraries on the
            ' current server.
            For Each libraryName In LibraryList
                combLibrary.Items.Add(libraryName.Trim())
                If (libraryName.Trim().ToUpper() = _LibraryName) Then
                    currentLibraryItem = combLibrary.Items.Count - 1
                End If
                If (libraryName.Trim().ToUpper() = "WORK") Then
                    workLibraryItem = combLibrary.Items.Count - 1
                End If
            Next

            If (currentLibraryItem <> -1) Then
                ' We found a matching item for the current library name
                ' so we make it the currently selected item in the list.
                combLibrary.SelectedIndex = currentLibraryItem
            Else
                ' There wasn't a matching library name on the current
                ' server, so we default the library name to WORK.
                combLibrary.SelectedIndex = workLibraryItem
            End If

            ' Of course, there is alwaysthe remote possibility that
            ' neither the current nor work libraries were found, in
            ' which case no library name will be selected within the
            ' list and and we return a False to the caller.
            If (currentLibraryItem = -1 And workLibraryItem = -1) Then
                Return False
            End If

            ' If we got here, then the libraries have been 
            ' successfully loaded and an item is selected.
            Return True
        End Function
#End Region

    End Class
End Namespace