' =========================================================
'   
'                Create Picture Format Task
'
'                          a SAS Custom Task Example 
'
' Copyright (C) 2006 by SAS Institute Inc., Cary, NC 27513
'
' ---------------------------------------------------------
' This is the task model which provides all of the
' ISASTask, ISASTaskAddIn and ISASTaskDescription interface
' implementations as required by the AddIn task
' specifications.
' =========================================================

Imports System
Imports System.Collections
Imports System.Globalization
Imports System.IO
Imports System.Text
Imports System.Windows.Forms
Imports System.Xml

Namespace SASVB.CustomTasks

#Region " Format Range Item Class "
    ' The Range Item class is used to store the definition
    ' of a single range. 
    Public Class RangeItem
        Public Low As Boolean
        Public StartValue As Double
        Public IncludeStartValue As Boolean
        Public High As Boolean
        Public EndValue As Double
        Public IncludeEndValue As Boolean
        Public Mask As String
        Public NoEdit As Boolean
        Public UseFillChar As Boolean
        Public FillChar As Char
        Public UsePrefix As Boolean
        Public Prefix As String
        Public UseMultiplier As Boolean
        Public Multiplier As Double

        Public Sub New()
            ' Use the Reset() method to initialise the item
            ' to its initial state.
            Reset()
        End Sub

        ' Returns a new object which is an exact copy of the
        ' current object.
        Public Function Clone() As RangeItem
            Dim newOne As RangeItem = New RangeItem

            newOne.Low = Low
            newOne.StartValue = StartValue
            newOne.IncludeStartValue = IncludeStartValue
            newOne.High = High
            newOne.EndValue = EndValue
            newOne.IncludeEndValue = IncludeEndValue
            newOne.Mask = Mask
            newOne.NoEdit = NoEdit
            newOne.UseFillChar = UseFillChar
            newOne.FillChar = FillChar
            newOne.UsePrefix = UsePrefix
            newOne.Prefix = Prefix
            newOne.UseMultiplier = UseMultiplier
            newOne.Multiplier = Multiplier

            Return newOne
        End Function

        ' Compares the start and end range of a specified RangeItem with
        ' the current instances start and end ranges.
        Public Function IsEqualTo(ByVal otherRange As RangeItem) As Boolean
            Return (IsStartEqualTo(otherRange) = True And _
                    IsEndEqualTo(otherRange) = True)
        End Function

        ' Compares the start range of a RangeItem with the current range
        ' item's start range.
        Public Function IsStartEqualTo(ByVal otherRange As RangeItem) As Boolean
            If ((Low = otherRange.Low And _
                 Low = True) Or _
                (Low = otherRange.Low And _
                 Low = False And _
                 StartValue = otherRange.StartValue And _
                 IncludeStartValue = otherRange.IncludeStartValue)) Then
                Return True
            End If

            Return False
        End Function

        ' Compares the end range of a RangeItem with the current range
        ' item's end range.
        Public Function IsEndEqualTo(ByVal otherRange As RangeItem) As Boolean
            If ((High = otherRange.High And _
                  High = True) Or _
                 (High = otherRange.High And _
                  High = False And _
                  EndValue = otherRange.EndValue And _
                  IncludeEndValue = otherRange.IncludeEndValue)) Then
                Return True
            End If

            Return False
        End Function

        ' Determines if the current range is "undefined".
        Public ReadOnly Property IsUndefinedRange() As Boolean
            Get
                Return (Mask.Trim().Length = 0 And NoEdit = False)
            End Get
        End Property

        ' Initialises the current item's member values.
        Public Sub Reset()
            Low = True
            StartValue = 0.0
            IncludeStartValue = True
            High = True
            EndValue = 0.0
            IncludeEndValue = True
            Mask = String.Empty
            NoEdit = False
            UseFillChar = False
            FillChar = " "
            UsePrefix = False
            Prefix = String.Empty
            UseMultiplier = False
            Multiplier = 1.0
        End Sub
    End Class
#End Region

#Region " GeneralParameters Class "
    ' This class is used to pass all of the properties for the
    ' General panel to and from the panel.
    Public Class GeneralParameters
        Public FormatName As String
        Public PictureType As Int32
        Public ServerName As String
        Public LibraryName As String
        Public UseDefaultWidth As Boolean
        Public DefaultWidth As Int32
        Public UseMinimumWidth As Boolean
        Public MinimumWidth As Int32
        Public UseMaximumWidth As Boolean
        Public MaximumWidth As Int32

        Public Sub New()
            FormatName = String.Empty
            PictureType = 0     ' Std. numeric picture format
            ServerName = String.Empty
            LibraryName = String.Empty
            UseDefaultWidth = False
            DefaultWidth = 12
            UseMinimumWidth = False
            MinimumWidth = 1
            UseMaximumWidth = False
            MaximumWidth = 40
        End Sub
    End Class
#End Region

#Region " DefinitionParameters Class "
    ' This class is used to pass all of the properties for the
    ' range definition panel to and from the panel.
    Public Class DefinitionParameters
        Public RangeItems As ArrayList
        Public UseMissingLabel As Boolean
        Public MissingLabel As String
        Public UseOTHERLabel As Boolean
        Public OTHERLabel As String
        Public TestValues() As Double

        Public Sub New()
            RangeItems = New ArrayList
            RangeItems.Add(New RangeItem)
            UseMissingLabel = False
            MissingLabel = String.Empty
            UseOTHERLabel = False
            OTHERLabel = String.Empty
            TestValues = New Double() {}
        End Sub
    End Class
#End Region

    Public Class PicFormat
        Implements SAS.Shared.AddIns.ISASTask
        Implements SAS.Shared.AddIns.ISASTaskAddIn
        Implements SAS.Shared.AddIns.ISASTaskDescription

#Region " Private Members "
        ' Consumer object supplied by host application.
        Private _consumer As SAS.Shared.AddIns.ISASTaskConsumer = Nothing

        ' Panel parameters.
        Private GeneralParams As GeneralParameters = New GeneralParameters
        Private DefinitionParams As DefinitionParameters = New DefinitionParameters
        Private CurrentPage As Int32 = 1

        ' The task's form label.
        Private _label As String = "Create Picture Format"
#End Region

#Region " ISASTask Members "
        ' The text to display as the run log when no SAS code is
        ' used. 
        Overridable ReadOnly Property RunLog() As String Implements SAS.Shared.AddIns.ISASTask.RunLog
            Get
                Return Nothing
            End Get
        End Property
        ' An XML description of the task state so that it can be
        ' persisted from run to run, and within the projects that
        ' contain the task. 
        Overridable Property XmlState() As String Implements SAS.Shared.AddIns.ISASTask.XmlState
            Get
                ' Get the XML string and return it.
                Return WriteXML()
            End Get
            Set(ByVal Value As String)
                ' Read all the property values from the XML string.
                ReadXML(Value)
            End Set
        End Property
        ' This routine is called at the end of each task session. It
        ' signals to the add-in it can perform any necessary
        ' post-processing cleanup. 
        Overridable Sub Terminate() Implements SAS.Shared.AddIns.ISASTask.Terminate
        End Sub
        ' Provides information about the output data that this 
        ' add-in expects to create or modify. 
        Overridable Function OutputDataInfo(ByVal Index As Integer, ByRef Source As String, ByRef Label As String) As SAS.Shared.AddIns.OutputData Implements SAS.Shared.AddIns.ISASTask.OutputDataInfo
            ' No output data created
            Source = Nothing
            Label = Nothing
            Return SAS.Shared.AddIns.OutputData.Unknown
        End Function
        ' The SAS code associated with the task.
        Overridable ReadOnly Property SASCode() As String Implements SAS.Shared.AddIns.ISASTask.SasCode
            Get
                ' Get the code from the Shared method on the view.
                Return BuildSASCode(GeneralParams, DefinitionParams)
            End Get
        End Property
        ' Called by the application when the task is about to be
        ' used.
        Overridable Function Initialize() As Boolean Implements SAS.Shared.AddIns.ISASTask.Initialize
            Return True
        End Function
        ' Show the task form.
        Overridable Function Show(ByVal Owner As IWin32Window) As SAS.Shared.AddIns.ShowResult Implements SAS.Shared.AddIns.ISASTask.Show
            If (_consumer Is Nothing) Then
                Throw New SAS.Shared.AddIns.AddInException("No consumer available.", SAS.Shared.AddIns.ErrorType.None)
            End If

            Dim dlg As PicFormatWizardForm = New PicFormatWizardForm
            ' Set the task's main form's title.
            dlg.Text = _label
            ' Pass parameters to the dialog's panels
            dlg.Consumer = _consumer
            dlg.GeneralParams = GeneralParams
            dlg.DefinitionParams = DefinitionParams
            dlg.CurrentPage = CurrentPage

            ' Show the dialog
            If (DialogResult.OK = dlg.ShowDialog(Owner)) Then
                ' Since it appears that the dialog was not cancelled
                ' we need to store the new parameter values returned
                ' from the view (task dialog).
                GeneralParams = dlg.GeneralParams
                DefinitionParams = dlg.DefinitionParams
                CurrentPage = dlg.CurrentPage
                ' If the server has changed, then we need to let
                ' the consumer know about it.
                If (_consumer.AssignedServer <> GeneralParams.ServerName) Then
                    _consumer.AssignedServer = GeneralParams.ServerName
                End If

                ' Tell the caller that we need to run.
                Return SAS.Shared.AddIns.ShowResult.RunNow
            Else
                ' Tell the consumer that the task was cancelled.
                Return SAS.Shared.AddIns.ShowResult.Canceled
            End If
        End Function
        ' The label of an instance of the task as it appears in the project. 
        Overridable Property Label() As String Implements SAS.Shared.AddIns.ISASTask.Label
            Get
                Return _label
            End Get
            Set(ByVal Value As String)
                _label = Value
            End Set
        End Property

        ' Indicates how many output datasets will be generated.
        Overridable ReadOnly Property OutputDataCount() As Integer Implements SAS.Shared.AddIns.ISASTask.OutputDataCount
            Get
                ' We do not produce data sets as output.
                Return 0
            End Get
        End Property
#End Region
#Region " ISASTaskAddIn Members "
        ' THIS PROPERTY IS NOT CURRENTLY USED AND IS RESERVED FOR FUTURE USE.
        Overridable ReadOnly Property VisibleInManager() As Boolean Implements SAS.Shared.AddIns.ISASTaskAddIn.VisibleInManager
            Get
                Return True
            End Get
        End Property
        ' Returns the name of the add-in. This value is shown in the
        ' Add-In Manager and might be shown within the host application.
        Overridable ReadOnly Property AddInName() As String Implements SAS.Shared.AddIns.ISASTaskAddIn.AddInName
            Get
                Return "Create Picture Format"
            End Get
        End Property

        ' Disconnects, or unloads, the add-in from the host application. 
        Overridable Sub Disconnect() Implements SAS.Shared.AddIns.ISASTaskAddIn.Disconnect
            _consumer = Nothing
        End Sub

        ' Returns a list of culture codes that represent the
        ' languages/cultures that this add-in can support. 
        Overridable Function Languages(ByRef Items() As String) As Integer Implements SAS.Shared.AddIns.ISASTaskAddIn.Languages
            Items = New String() {"en-US"}
            Return 1
        End Function

        ' Returns the description of the add-in. This value is shown in
        ' the Add-In Manager and might be shown within the host
        ' application. 
        Overridable ReadOnly Property AddInDescription() As String Implements SAS.Shared.AddIns.ISASTaskAddIn.AddInDescription
            Get
                Return "Creates a PICTURE based SAS format."
            End Get
        End Property

        ' Connects, or loads, the add-in for use by the host application. 
        Public Overridable Function Connect(ByVal Consumer As SAS.Shared.AddIns.ISASTaskConsumer) As Boolean Implements SAS.Shared.AddIns.ISASTaskAddIn.Connect
            _consumer = Consumer
            If (Consumer Is Nothing) Then
                Throw New SAS.Shared.AddIns.AddInException("Did not receive a valid consumer in the Connect function.", SAS.Shared.AddIns.ErrorType.None)
            End If

            Return True
        End Function

        ' Current language (culture) that the add-in should operate in. 
        Public Overridable WriteOnly Property Language() As String Implements SAS.Shared.AddIns.ISASTaskAddIn.Language
            Set(ByVal Value As String)
                ' no implementation
            End Set
        End Property
#End Region
#Region " ISASTaskDescription Members "
        ' SAS products that required on the server to use this add-in. 
        Overridable ReadOnly Property ProductsRequired() As String Implements SAS.Shared.AddIns.ISASTaskDescription.ProductsRequired
            Get
                Return "BASE"
            End Get
        End Property
        ' SAS products that are optional on the server to use this add-in. 
        Overridable ReadOnly Property ProductsOptional() As String Implements SAS.Shared.AddIns.ISASTaskDescription.ProductsOptional
            Get
                Return String.Empty
            End Get
        End Property

        ' List of SAS procedures used by this add-in. 
        Overridable ReadOnly Property ProcsUsed() As String Implements SAS.Shared.AddIns.ISASTaskDescription.ProcsUsed
            Get
                Return "FORMAT"
            End Get
        End Property

        ' Indicates whether the category is a standard SAS-supplied
        ' category.
        Overridable ReadOnly Property StandardCategory() As Boolean Implements SAS.Shared.AddIns.ISASTaskDescription.StandardCategory
            Get
                Return False
            End Get
        End Property

        ' Indicates whether this add-in creates SAS ODS results. 
        Overridable ReadOnly Property GeneratesListOutput() As Boolean Implements SAS.Shared.AddIns.ISASTaskDescription.GeneratesListOutput
            Get
                ' This task does not generate any List/ODS output.
                Return False
            End Get
        End Property
        ' The add-in name as shown in the application menus and
        ' task list.
        Overridable ReadOnly Property TaskName() As String Implements SAS.Shared.AddIns.ISASTaskDescription.TaskName
            Get
                Return "Create Picture Format"
            End Get
        End Property
        ' The validation status associated with this add-in. 
        Overridable ReadOnly Property Validation() As String Implements SAS.Shared.AddIns.ISASTaskDescription.Validation
            Get
                Return "SAS Sample"
            End Get
        End Property

        ' The name of the category for this add-in. 
        Overridable ReadOnly Property TaskCategory() As String Implements SAS.Shared.AddIns.ISASTaskDescription.TaskCategory
            Get
                Return "Add-in Examples"
            End Get
        End Property
        ' The name of the .NET assembly that contains the add-in
        ' icon. 
        Overridable ReadOnly Property IconAssembly() As String Implements SAS.Shared.AddIns.ISASTaskDescription.IconAssembly
            Get
                ' Return the full path/name of this assembly
                Return System.Reflection.Assembly.GetExecutingAssembly().Location
            End Get
        End Property
        ' The name of the add-in icon resource. 
        Overridable ReadOnly Property IconName() As String Implements SAS.Shared.AddIns.ISASTaskDescription.IconName
            Get
                ' return the name of the icon within our assembly
                ' including namespace qualifiers
                Return "SASVB.CustomTasks.PicFormat.ico"
            End Get
        End Property

        ' A unique identifier in GUID format. Each add-in must supply
        ' a unique Clsid.
        Overridable ReadOnly Property Clsid() As String Implements SAS.Shared.AddIns.ISASTaskDescription.Clsid
            Get
                Return "C1999BE1-D97B-469d-9A70-AA0F89810B4E"
            End Get
        End Property
        ' THIS PROPERTY IS NOT CURRENTLY USED AND IS RESERVED FOR FUTURE USE.
        Overridable ReadOnly Property TaskType() As SAS.Shared.AddIns.ShowType Implements SAS.Shared.AddIns.ISASTaskDescription.TaskType
            Get
                Return SAS.Shared.AddIns.ShowType.Wizard
            End Get
        End Property
        ' Indicates whether the add-in requires one or more input data
        ' sets to operate. 
        Overridable ReadOnly Property RequiresData() As Boolean Implements SAS.Shared.AddIns.ISASTaskDescription.RequiresData
            Get
                ' No data sets are needed to create a PICTURE format. 
                Return False
            End Get
        End Property
        ' Indicates how many analysis variables (numeric) are required
        ' in the input data for the add-in to run. 
        Overridable ReadOnly Property NumericColsRequired() As Integer Implements SAS.Shared.AddIns.ISASTaskDescription.NumericColsRequired
            Get
                Return 0
            End Get
        End Property
        ' The major version number for this add-in. This is shown in
        ' the Task by Name window within SAS Enterprise Guide.
        Overridable ReadOnly Property MajorVersion() As Integer Implements SAS.Shared.AddIns.ISASTaskDescription.MajorVersion
            Get
                Return 1
            End Get
        End Property
        ' The minor version number for this add-in. This is shown in
        ' the Task by Name window within SAS Enterprise Guide. 
        Overridable ReadOnly Property MinorVersion() As Integer Implements SAS.Shared.AddIns.ISASTaskDescription.MinorVersion
            Get
                Return 0
            End Get
        End Property
        ' A short description of the add-in's function or purpose. 
        Overridable ReadOnly Property TaskDescription() As String Implements SAS.Shared.AddIns.ISASTaskDescription.TaskDescription
            Get
                Return "Creates a PICTURE based SAS format."
            End Get
        End Property
        ' Whether this add-in generates SAS program code. 
        Overridable ReadOnly Property GeneratesSasCode() As Boolean Implements SAS.Shared.AddIns.ISASTaskDescription.GeneratesSasCode
            Get
                Return True
            End Get
        End Property
        ' Name used when constructing an instance of the add-in. 
        Overridable ReadOnly Property FriendlyName() As String Implements SAS.Shared.AddIns.ISASTaskDescription.FriendlyName
            Get
                Return "Create Picture Format"
            End Get
        End Property
        ' A detailed description of the function and role of the
        ' add-in. 
        Overridable ReadOnly Property WhatIsDescription() As String Implements SAS.Shared.AddIns.ISASTaskDescription.WhatIsDescription
            Get
                Return "Creates a PICTURE based SAS format."
            End Get
        End Property
#End Region
#Region " Save and Load XML State "
        Private Function WriteXML() As String
            Dim sw As StringWriter = New StringWriter
            Dim writer As XmlTextWriter = New XmlTextWriter(sw)
            writer.WriteStartElement("PicFormat")

            writer.WriteElementString("CurrentPage", XmlConvert.ToString(CurrentPage))

            writer.WriteElementString("FormatName", GeneralParams.FormatName)
            writer.WriteElementString("PictureType", XmlConvert.ToString(GeneralParams.PictureType))
            writer.WriteElementString("ServerName", GeneralParams.ServerName)
            writer.WriteElementString("LibraryName", GeneralParams.LibraryName)
            writer.WriteElementString("UseDefaultWidth", XmlConvert.ToString(GeneralParams.UseDefaultWidth))
            writer.WriteElementString("DefaultWidth", XmlConvert.ToString(GeneralParams.DefaultWidth))
            writer.WriteElementString("UseMinimumWidth", XmlConvert.ToString(GeneralParams.UseMinimumWidth))
            writer.WriteElementString("MinimumWidth", XmlConvert.ToString(GeneralParams.MinimumWidth))
            writer.WriteElementString("UseMaximumWidth", XmlConvert.ToString(GeneralParams.UseMaximumWidth))
            writer.WriteElementString("MaximumWidth", XmlConvert.ToString(GeneralParams.MaximumWidth))

            ' Serialize the contents of the RangeItem ArrayList. 
            writer.WriteStartElement("RangeItems")
            writer.WriteElementString("RangeItemCount", XmlConvert.ToString(DefinitionParams.RangeItems.Count))
            Dim ri As RangeItem
            For rangeIndex As Int32 = 0 To DefinitionParams.RangeItems.Count - 1
                writer.WriteStartElement(String.Format("RangeItem{0}", rangeIndex.ToString(NumberFormatInfo.InvariantInfo)))
                ri = DefinitionParams.RangeItems(rangeIndex)
                writer.WriteElementString("Low", XmlConvert.ToString(ri.Low))
                writer.WriteElementString("StartValue", XmlConvert.ToString(ri.StartValue))
                writer.WriteElementString("IncludeStartValue", XmlConvert.ToString(ri.IncludeStartValue))
                writer.WriteElementString("High", XmlConvert.ToString(ri.High))
                writer.WriteElementString("EndValue", XmlConvert.ToString(ri.EndValue))
                writer.WriteElementString("IncludeEndValue", XmlConvert.ToString(ri.IncludeEndValue))
                writer.WriteElementString("Mask", ri.Mask)
                writer.WriteElementString("NoEdit", XmlConvert.ToString(ri.NoEdit))
                writer.WriteElementString("UsePrefix", XmlConvert.ToString(ri.UsePrefix))
                writer.WriteElementString("Prefix", ri.Prefix)
                writer.WriteElementString("UseFillChar", XmlConvert.ToString(ri.UseFillChar))
                writer.WriteElementString("FillChar", ri.FillChar)
                writer.WriteElementString("UseMultiplier", XmlConvert.ToString(ri.UseMultiplier))
                writer.WriteElementString("Multiplier", ri.Multiplier)
                writer.WriteEndElement() ' RangeItem{0}
            Next
            writer.WriteEndElement() ' RangeItems

            writer.WriteElementString("UseMissingLabel", XmlConvert.ToString(DefinitionParams.UseMissingLabel))
            writer.WriteElementString("MissingLabel", DefinitionParams.MissingLabel)

            writer.WriteElementString("UseOTHERLabel", XmlConvert.ToString(DefinitionParams.UseOTHERLabel))
            writer.WriteElementString("OTHERLabel", DefinitionParams.OTHERLabel)

            ' Serialize the TestValues Double array.
            writer.WriteStartElement("TestValueItems")
            writer.WriteElementString("TestValueItemCount", XmlConvert.ToString(DefinitionParams.TestValues.GetLength(0)))
            For index As Int32 = 0 To DefinitionParams.TestValues.GetLength(0) - 1
                writer.WriteElementString(String.Format("TestValueItem{0}", index.ToString(NumberFormatInfo.InvariantInfo)), XmlConvert.ToString(DefinitionParams.TestValues(index)))
            Next
            writer.WriteEndElement() ' TestValueItems

            writer.WriteEndElement() ' PicFormat
            writer.Close()
            Return sw.ToString()
        End Function

        Private Sub ReadXML(ByVal xml As String)
            If (Not xml Is Nothing And xml.Length > 0) Then
                Try
                    Dim sr As StringReader = New StringReader(xml)
                    Dim reader As XmlTextReader = New XmlTextReader(sr)
                    reader.ReadStartElement("PicFormat")

                    CurrentPage = XmlConvert.ToInt32(reader.ReadElementString("CurrentPage"))

                    GeneralParams.FormatName = reader.ReadElementString("FormatName")
                    GeneralParams.PictureType = XmlConvert.ToInt32(reader.ReadElementString("PictureType"))
                    GeneralParams.ServerName = reader.ReadElementString("ServerName")
                    GeneralParams.LibraryName = reader.ReadElementString("LibraryName")
                    GeneralParams.UseDefaultWidth = XmlConvert.ToBoolean(reader.ReadElementString("UseDefaultWidth"))
                    GeneralParams.DefaultWidth = XmlConvert.ToDouble(reader.ReadElementString("DefaultWidth"))
                    GeneralParams.UseMinimumWidth = XmlConvert.ToBoolean(reader.ReadElementString("UseMinimumWidth"))
                    GeneralParams.MinimumWidth = XmlConvert.ToDouble(reader.ReadElementString("MinimumWidth"))
                    GeneralParams.UseMaximumWidth = XmlConvert.ToBoolean(reader.ReadElementString("UseMaximumWidth"))
                    GeneralParams.MaximumWidth = XmlConvert.ToDouble(reader.ReadElementString("MaximumWidth"))

                    ' Recreate the RangeItem ArrayList.
                    reader.ReadStartElement("RangeItems")
                    Dim rangeItemCount As Int32 = XmlConvert.ToInt32(reader.ReadElementString("RangeItemCount"))
                    Dim ri As RangeItem
                    DefinitionParams.RangeItems.Clear()
                    For rangeIndex As Int32 = 0 To rangeItemCount - 1
                        reader.ReadStartElement(String.Format("RangeItem{0}", rangeIndex.ToString(NumberFormatInfo.InvariantInfo)))
                        ri = New RangeItem
                        ri.Low = XmlConvert.ToBoolean(reader.ReadElementString("Low"))
                        ri.StartValue = XmlConvert.ToDouble(reader.ReadElementString("StartValue"))
                        ri.IncludeStartValue = XmlConvert.ToBoolean(reader.ReadElementString("IncludeStartValue"))
                        ri.High = XmlConvert.ToBoolean(reader.ReadElementString("High"))
                        ri.EndValue = XmlConvert.ToDouble(reader.ReadElementString("EndValue"))
                        ri.IncludeEndValue = XmlConvert.ToBoolean(reader.ReadElementString("IncludeEndValue"))
                        ri.Mask = reader.ReadElementString("Mask")
                        ri.NoEdit = XmlConvert.ToBoolean(reader.ReadElementString("NoEdit"))
                        ri.UsePrefix = XmlConvert.ToBoolean(reader.ReadElementString("UsePrefix"))
                        ri.Prefix = reader.ReadElementString("Prefix")
                        ri.UseFillChar = XmlConvert.ToBoolean(reader.ReadElementString("UseFillChar"))
                        ri.FillChar = reader.ReadElementString("FillChar")
                        ri.UseMultiplier = XmlConvert.ToBoolean(reader.ReadElementString("UseMultiplier"))
                        ri.Multiplier = reader.ReadElementString("Multiplier")
                        DefinitionParams.RangeItems.Add(ri)
                        reader.ReadEndElement()
                    Next
                    reader.ReadEndElement()

                    DefinitionParams.UseMissingLabel = XmlConvert.ToBoolean(reader.ReadElementString("UseMissingLabel"))
                    DefinitionParams.MissingLabel = reader.ReadElementString("MissingLabel")

                    DefinitionParams.UseOTHERLabel = XmlConvert.ToBoolean(reader.ReadElementString("UseOTHERLabel"))
                    DefinitionParams.OTHERLabel = reader.ReadElementString("OTHERLabel")

                    ' Recreate the TestValues Double array.
                    reader.ReadStartElement("TestValueItems")
                    Dim testValueItemCount As Int32 = XmlConvert.ToInt32(reader.ReadElementString("TestValueItemCount"))
                    ReDim DefinitionParams.TestValues(testValueItemCount - 1)
                    For index As Int32 = 0 To testValueItemCount - 1
                        DefinitionParams.TestValues(index) = XmlConvert.ToDouble(reader.ReadElementString(String.Format("TestValueItem{0}", index.ToString(NumberFormatInfo.InvariantInfo))))
                    Next
                    reader.ReadEndElement() ' TestValueItems

                    reader.ReadEndElement()
                    reader.Close()
                Catch
                End Try
            End If
        End Sub
#End Region

#Region " Shared Code Generation Methods "
        Public Shared Function BuildSASCode(ByVal GeneralParams As GeneralParameters, ByVal DefinitionParams As DefinitionParameters) As String
            Dim sb As StringBuilder = New StringBuilder
            Dim separator As String
            ' Create the code based on the current properties
            sb.Append("PROC FORMAT")

            ' If the output library is WORK, then we don't need to specify
            ' the LIBRARY= option  since WORK is the default.
            If (GeneralParams.LibraryName <> "WORK") Then
                sb.AppendFormat(" LIBRARY={0}", GeneralParams.LibraryName)
            End If

            sb.AppendFormat(";{0}", vbCrLf)

            sb.AppendFormat("    PICTURE {0}", GeneralParams.FormatName)
            ' We only need to write out the PICTURE options' parentheses 
            ' if one or more PICTURE options have been selected.
            If (GeneralParams.UseDefaultWidth = True Or _
                GeneralParams.UseMinimumWidth = True Or _
                GeneralParams.UseMaximumWidth = True) Then
                ' The separator variable is used to place spaces between
                ' consecutive PICTURE options.
                separator = String.Empty

                sb.Append(" (")
                If (GeneralParams.UseDefaultWidth = True) Then
                    sb.AppendFormat("{0}DEFAULT={1}", separator, GeneralParams.DefaultWidth)
                    separator = " "
                End If
                If (GeneralParams.UseMinimumWidth = True) Then
                    sb.AppendFormat("{0}MIN={1}", separator, GeneralParams.MinimumWidth)
                    separator = " "
                End If
                If (GeneralParams.UseMaximumWidth = True) Then
                    sb.AppendFormat("{0}MAX={1}", separator, GeneralParams.MaximumWidth)
                    separator = " "
                End If
                sb.Append(")")
            End If
            sb.Append(vbCrLf)

            ' Now we call the method for creating the string containing
            ' the code for the actual range definitions.
            sb.Append(BuildSASCodeForRange(DefinitionParams, GeneralParams.PictureType))

            sb.AppendFormat("{0}{1}", "        ;", vbCrLf)
            sb.AppendFormat("{0}{1}{2}", "RUN;", vbCrLf, vbCrLf)
            ' And we're done!
            Return sb.ToString()
        End Function

        Public Shared Function BuildSASCodeForRange(ByVal dp As DefinitionParameters, ByVal pictureType As Int32) As String
            Dim sb As StringBuilder = New StringBuilder
            Dim separator As String = String.Empty
            Dim workString As String

            ' We need to loop through each RangeItem in the RangeItems ArrayList
            ' and generate a range expression for each defined range.
            For Each ri As RangeItem In dp.RangeItems
                If (ri.IsUndefinedRange = False) Then
                    ' Start by supplying a bit of indentation spacing.
                    sb.Append("        ")
                    If (ri.Low = True) Then
                        sb.Append("LOW")
                    Else
                        ' Format the Double value depending on the format type.
                        Select Case (pictureType)
                            Case 0 ' Numeric
                                sb.Append(ri.StartValue.ToString(NumberFormatInfo.InvariantInfo))
                            Case 1 ' Date
                                sb.Append(UtilityMethods.SASDateToString(ri.StartValue, True))
                            Case 2 ' Time
                                sb.Append(UtilityMethods.SASTimeToString(ri.StartValue, True))
                            Case 3 ' DateTime
                                sb.Append(UtilityMethods.SASDateTimeToString(ri.StartValue, True))
                        End Select
                        If (ri.IncludeStartValue = False) Then
                            sb.Append("<")
                        End If
                    End If
                    sb.Append("-")
                    If (ri.High = True) Then
                        sb.Append("HIGH")
                    Else
                        If (ri.IncludeEndValue = False) Then
                            sb.Append("<")
                        End If
                        ' Format the Double value depending on the format type.
                        Select Case (pictureType)
                            Case 0 ' Numeric
                                sb.Append(ri.EndValue.ToString(NumberFormatInfo.InvariantInfo))
                            Case 1 ' Date
                                sb.Append(UtilityMethods.SASDateToString(ri.EndValue, True))
                            Case 2 ' Time
                                sb.Append(UtilityMethods.SASTimeToString(ri.EndValue, True))
                            Case 3 ' DateTime
                                sb.Append(UtilityMethods.SASDateTimeToString(ri.EndValue, True))
                        End Select
                    End If

                    ' PROC FORMAT treats a string value of '' (an empty string)
                    ' as if it were '''' resulting in a single quote ('). So
                    ' instead we need to make sure that we supply a quoted space.
                    workString = IIf(ri.Mask.Length = 0, " ", ri.Mask)
                    sb.AppendFormat(" = {0}", UtilityMethods.QuoteString(workString, False))

                    ' We only need to write out the range options' parentheses 
                    ' if one or more range options are defined.
                    If (ri.NoEdit = True Or _
                        ri.UsePrefix = True Or _
                        ri.UseFillChar = True Or _
                        pictureType <> 0 Or _
                        (pictureType = 0 And ri.UseMultiplier = True)) Then
                        separator = String.Empty

                        sb.Append(" (")
                        If (ri.NoEdit = True) Then
                            sb.AppendFormat("{0}NOEDIT", separator)
                            separator = " "
                        End If
                        If (ri.UsePrefix = True) Then
                            workString = IIf(ri.Prefix.Length = 0, " ", ri.Prefix)
                            sb.AppendFormat("{0}PREFIX={1}", separator, UtilityMethods.QuoteString(workString, False))
                            separator = " "
                        End If
                        If (ri.UseFillChar = True) Then
                            sb.AppendFormat("{0}FILL={1}", separator, UtilityMethods.QuoteString(ri.FillChar, False))
                            separator = " "
                        End If
                        If (pictureType <> 0) Then
                            Select Case pictureType
                                Case 1 ' Date
                                    sb.AppendFormat("{0}DATATYPE=DATE", separator)
                                Case 2 ' Time
                                    sb.AppendFormat("{0}DATATYPE=TIME", separator)
                                Case 3 ' DateTime
                                    sb.AppendFormat("{0}DATATYPE=DATETIME", separator)
                            End Select
                            separator = " "
                        End If
                        ' It only makes sense to push the MULTIPLIER= option if
                        ' the picture format type is numeric (not for date, time
                        ' or datetime formats). 
                        If (pictureType = 0 And ri.UseMultiplier = True) Then
                            sb.AppendFormat("{0}MULTIPLIER={1}", separator, ri.Multiplier.ToString(NumberFormatInfo.InvariantInfo))
                            separator = " "
                        End If
                        sb.Append(")")
                    End If
                    sb.Append(vbCrLf)
                End If
            Next

            If (dp.UseMissingLabel = True) Then
                workString = IIf(dp.MissingLabel.Length = 0, " ", dp.MissingLabel)
                sb.AppendFormat("        . = {0} (NOEDIT){1}", UtilityMethods.QuoteString(workString, False), vbCrLf)
            End If

            If (dp.UseOTHERLabel = True) Then
                workString = IIf(dp.OTHERLabel.Length = 0, " ", dp.OTHERLabel)
                sb.AppendFormat("        OTHER = {0} (NOEDIT){1}", UtilityMethods.QuoteString(workString, False), vbCrLf)
            Else
                ' If OTHER has not been specified, then for the case of date,
                ' time and datetime type picture formats, we will push code
                ' to provide a format appropriate for that 'data' type to be
                ' used as the default formatting mask.
                Select Case pictureType
                    Case 1 ' Date
                        sb.AppendFormat("        OTHER = '%0d%b%Y' (DATATYPE=DATE){0}", vbCrLf)
                    Case 2 ' Time
                        sb.AppendFormat("        OTHER = '%0H:%0M:%0S' (DATATYPE=TIME){0}", vbCrLf)
                    Case 3 ' DateTime
                        sb.AppendFormat("        OTHER = '%0d%b%Y:%0H:%0M:%0S' (DATATYPE=DATETIME){0}", vbCrLf)
                End Select
            End If

            ' And we're done!
            Return sb.ToString()
        End Function
#End Region

    End Class
End Namespace
