' =========================================================
'   
'                Create Picture Format Task
'
'                          a SAS Custom Task Example 
'
' Copyright (C) 2006 by SAS Institute Inc., Cary, NC 27513
'
' ---------------------------------------------------------
' This is the view (main form) for the task. In this
' instance the task is in the form of a two page (panel)
' wizard style UI.
' The form itself acts as the container for the panels of
' the form, as well as acting as the conduit for any
' communications between the model and the panels, or
' between the panels themselves.
' =========================================================

Imports System.Drawing
Imports System.Globalization
Imports System.IO
Imports System.Text
Imports System.Windows.Forms

Namespace SASVB.CustomTasks
    Public Class PicFormatWizardForm
        Inherits System.Windows.Forms.Form

#Region " Private Constants "
        ' The total number of wizard pages.
        Private Const _TotalPages As Integer = 2
#End Region

#Region " Member Variables "
        ' The panel objects representing pages 1 and 2 of the
        ' wizard.
        Private WithEvents _Panel1 As GeneralPanel = Nothing
        Private WithEvents _Panel2 As DefinitionPanel = Nothing

        ' Define the code preview Form, although we won't be
        ' creating it until it's needed.
        Private WithEvents _CodePreviewForm As CodePreviewForm = Nothing

        ' The currently displayed wizard page number.
        Private _CurrentPage As Int32 = 0
        ' Used to keep track of the state of the code preview form.
        Private _CodePreviewVisible As Boolean = False

        Private _Consumer As SAS.Shared.Addins.ISASTaskConsumer
#End Region

#Region " Public Properties "
        Public Property Consumer() As SAS.Shared.Addins.ISASTaskConsumer
            Get
                Return _Consumer
            End Get
            Set(ByVal Value As SAS.Shared.Addins.ISASTaskConsumer)
                _Consumer = Value
                ' If the wizard panels are defined, then we need to pass
                ' the consumer on to them.
                If Not (_Panel1 Is Nothing) Then
                    _Panel1.Consumer = _Consumer
                End If
                If Not (_Panel2 Is Nothing) Then
                    _Panel2.Consumer = _Consumer
                End If
            End Set
        End Property

        Public Property GeneralParams() As GeneralParameters
            Get
                ' Returns the current parameter values for the general
                ' (1st) panel.
                Return _Panel1.GeneralParams
            End Get
            Set(ByVal Value As GeneralParameters)
                ' Store the general panel property values.
                _Panel1.GeneralParams = Value
                ' Pass the picture type on to the range definition
                ' panel as well.
                _Panel2.PictureType = Value.PictureType
            End Set
        End Property

        Public Property DefinitionParams() As DefinitionParameters
            Get
                ' Returns the current parameter values for the range
                ' definition (2nd) panel.
                Return _Panel2.DefinitionParams
            End Get
            Set(ByVal Value As DefinitionParameters)
                ' Store the range definition panel property values.
                _Panel2.DefinitionParams = Value
            End Set
        End Property

        Public Property CurrentPage() As Integer
            Get
                ' Return the currently active wizard page number.
                Return _CurrentPage
            End Get
            Set(ByVal Value As Integer)
                ' Check that the supplied page number is valid.
                If (Value < 1 Or Value > _TotalPages) Then
                    Throw New ArgumentOutOfRangeException("CurrentPage", Value, "An attempt was made to change to an invalid Wizard page number.")
                End If

                ' Hide the current page.
                If (_CurrentPage >= 1 And _CurrentPage <= _TotalPages) Then
                    Select Case (_CurrentPage)
                        Case 1
                            _Panel1.Visible = False
                        Case 2
                            _Panel2.Visible = False
                        Case Else
                            Throw New Exception("An unexpected number of Wizard panels exist.")
                    End Select
                End If

                ' Enable the new page and set the panel title on
                ' the wizard header.
                Select Case (Value)
                    Case 1
                        _Panel1.Visible = True
                        lblPanelTitle.Text = "General Format Information"
                    Case 2
                        _Panel2.Visible = True
                        lblPanelTitle.Text = "Define Ranges and Masks"
                    Case Else
                        Throw New Exception("An unexpected number of Wizard panels exist.")
                End Select
                ' Also update the page number on the wizard header.
                lblPageNo.Text = Value.ToString()

                ' Now store the new current page number.
                _CurrentPage = Value

                ' Set the Next/Back push button states
                btnBack.Enabled = (_CurrentPage <> 1)
                btnNext.Enabled = (_CurrentPage <> _TotalPages)
            End Set
        End Property
#End Region

#Region " Windows Form Designer generated code "

        Public Sub New()
            MyBase.New()

            'This call is required by the Windows Form Designer.
            InitializeComponent()

            ' Load the icon for the form's control menu on the title bar.
            Me.Icon = New Icon(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream("SASVB.CustomTasks.PicFormat.ico"))

            ' Load the SAS Logo bitmap.
            picSASLogo.Image = New Bitmap(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream("SASVB.CustomTasks.SASLogo.bmp"))

            'Create the wizard task panels.
            _Panel1 = New GeneralPanel
            _Panel1.Name = "_Panel1"
            _Panel1.Location = New Point(0, picStepHeader.Top + picStepHeader.Height)
            _Panel1.Size = New Size(Me.ClientSize.Width, gboxControlArea.Top - _Panel1.Location.Y)
            _Panel1.Anchor = AnchorStyles.Left + AnchorStyles.Top + AnchorStyles.Right + AnchorStyles.Bottom
            _Panel1.Visible = False
            _Panel1.Consumer = _Consumer
            _Panel1.TabIndex = 5

            _Panel2 = New DefinitionPanel
            _Panel2.Name = "_Panel2"
            _Panel2.Location = New Point(0, picStepHeader.Top + picStepHeader.Height)
            _Panel2.Size = New Size(Me.ClientSize.Width, gboxControlArea.Top - _Panel1.Location.Y)
            _Panel2.Anchor = AnchorStyles.Left + AnchorStyles.Top + AnchorStyles.Right + AnchorStyles.Bottom
            _Panel2.Visible = False
            _Panel2.Consumer = _Consumer
            _Panel2.TabIndex = 5

            ' Add the new panels to the control collection for the form.
            Me.Controls.Add(_Panel1)
            Me.Controls.Add(_Panel2)

            ' Set the current page to page 1.
            CurrentPage = 1
        End Sub

        'Form overrides dispose to clean up the component list.
        Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
            If disposing Then
                If Not (components Is Nothing) Then
                    components.Dispose()
                End If
            End If
            MyBase.Dispose(disposing)
        End Sub

        'Required by the Windows Form Designer
        Private components As System.ComponentModel.IContainer

        'NOTE: The following procedure is required by the Windows Form Designer
        'It can be modified using the Windows Form Designer.  
        'Do not modify it using the code editor.
        Friend WithEvents lblPageNo As System.Windows.Forms.Label
        Friend WithEvents lblPanelTitle As System.Windows.Forms.Label
        Friend WithEvents picStepHeader As System.Windows.Forms.PictureBox
        Friend WithEvents lblOfPages As System.Windows.Forms.Label
        Friend WithEvents picSASLogo As System.Windows.Forms.PictureBox
        Friend WithEvents gboxControlArea As System.Windows.Forms.GroupBox
        Friend WithEvents btnHelp As System.Windows.Forms.Button
        Friend WithEvents btnCancel As System.Windows.Forms.Button
        Friend WithEvents btnFinish As System.Windows.Forms.Button
        Friend WithEvents btnNext As System.Windows.Forms.Button
        Friend WithEvents btnBack As System.Windows.Forms.Button
        Friend WithEvents chkPreviewCode As System.Windows.Forms.CheckBox
        <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
            Me.picStepHeader = New System.Windows.Forms.PictureBox
            Me.lblPageNo = New System.Windows.Forms.Label
            Me.lblOfPages = New System.Windows.Forms.Label
            Me.lblPanelTitle = New System.Windows.Forms.Label
            Me.picSASLogo = New System.Windows.Forms.PictureBox
            Me.gboxControlArea = New System.Windows.Forms.GroupBox
            Me.btnHelp = New System.Windows.Forms.Button
            Me.btnCancel = New System.Windows.Forms.Button
            Me.btnFinish = New System.Windows.Forms.Button
            Me.btnNext = New System.Windows.Forms.Button
            Me.btnBack = New System.Windows.Forms.Button
            Me.chkPreviewCode = New System.Windows.Forms.CheckBox
            Me.SuspendLayout()
            '
            'picStepHeader
            '
            Me.picStepHeader.Anchor = CType(((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.picStepHeader.BackColor = System.Drawing.SystemColors.Window
            Me.picStepHeader.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle
            Me.picStepHeader.Location = New System.Drawing.Point(-8, 0)
            Me.picStepHeader.Name = "picStepHeader"
            Me.picStepHeader.Size = New System.Drawing.Size(624, 56)
            Me.picStepHeader.TabIndex = 0
            Me.picStepHeader.TabStop = False
            '
            'lblPageNo
            '
            Me.lblPageNo.BackColor = System.Drawing.SystemColors.Window
            Me.lblPageNo.Font = New System.Drawing.Font("Microsoft Sans Serif", 14.25!, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, CType(0, Byte))
            Me.lblPageNo.Location = New System.Drawing.Point(8, 16)
            Me.lblPageNo.Name = "lblPageNo"
            Me.lblPageNo.Size = New System.Drawing.Size(24, 24)
            Me.lblPageNo.TabIndex = 1
            Me.lblPageNo.Text = "1"
            Me.lblPageNo.TextAlign = System.Drawing.ContentAlignment.MiddleRight
            '
            'lblOfPages
            '
            Me.lblOfPages.BackColor = System.Drawing.SystemColors.Window
            Me.lblOfPages.Font = New System.Drawing.Font("Microsoft Sans Serif", 9.75!, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, CType(0, Byte))
            Me.lblOfPages.Location = New System.Drawing.Point(32, 16)
            Me.lblOfPages.Name = "lblOfPages"
            Me.lblOfPages.Size = New System.Drawing.Size(40, 24)
            Me.lblOfPages.TabIndex = 2
            Me.lblOfPages.Text = "of 2"
            Me.lblOfPages.TextAlign = System.Drawing.ContentAlignment.MiddleLeft
            '
            'lblPanelTitle
            '
            Me.lblPanelTitle.Anchor = CType(((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.lblPanelTitle.BackColor = System.Drawing.SystemColors.Window
            Me.lblPanelTitle.Font = New System.Drawing.Font("Microsoft Sans Serif", 9.75!, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, CType(0, Byte))
            Me.lblPanelTitle.Location = New System.Drawing.Point(80, 16)
            Me.lblPanelTitle.Name = "lblPanelTitle"
            Me.lblPanelTitle.Size = New System.Drawing.Size(392, 24)
            Me.lblPanelTitle.TabIndex = 3
            Me.lblPanelTitle.Text = "General Format Information"
            Me.lblPanelTitle.TextAlign = System.Drawing.ContentAlignment.MiddleLeft
            '
            'picSASLogo
            '
            Me.picSASLogo.Anchor = CType((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.picSASLogo.BackColor = System.Drawing.SystemColors.Window
            Me.picSASLogo.Location = New System.Drawing.Point(512, 12)
            Me.picSASLogo.Name = "picSASLogo"
            Me.picSASLogo.Size = New System.Drawing.Size(80, 32)
            Me.picSASLogo.SizeMode = System.Windows.Forms.PictureBoxSizeMode.CenterImage
            Me.picSASLogo.TabIndex = 4
            Me.picSASLogo.TabStop = False
            '
            'gboxControlArea
            '
            Me.gboxControlArea.Anchor = CType(((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.gboxControlArea.Location = New System.Drawing.Point(0, 380)
            Me.gboxControlArea.Name = "gboxControlArea"
            Me.gboxControlArea.Size = New System.Drawing.Size(608, 8)
            Me.gboxControlArea.TabIndex = 6
            Me.gboxControlArea.TabStop = False
            '
            'btnHelp
            '
            Me.btnHelp.Anchor = CType((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.btnHelp.Enabled = False
            Me.btnHelp.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.btnHelp.Location = New System.Drawing.Point(528, 400)
            Me.btnHelp.Name = "btnHelp"
            Me.btnHelp.Size = New System.Drawing.Size(75, 24)
            Me.btnHelp.TabIndex = 12
            Me.btnHelp.Text = "Help"
            '
            'btnCancel
            '
            Me.btnCancel.Anchor = CType((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.btnCancel.CausesValidation = False
            Me.btnCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel
            Me.btnCancel.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.btnCancel.Location = New System.Drawing.Point(448, 400)
            Me.btnCancel.Name = "btnCancel"
            Me.btnCancel.Size = New System.Drawing.Size(75, 24)
            Me.btnCancel.TabIndex = 11
            Me.btnCancel.Text = "Cancel"
            '
            'btnFinish
            '
            Me.btnFinish.Anchor = CType((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.btnFinish.DialogResult = System.Windows.Forms.DialogResult.OK
            Me.btnFinish.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.btnFinish.Location = New System.Drawing.Point(368, 400)
            Me.btnFinish.Name = "btnFinish"
            Me.btnFinish.Size = New System.Drawing.Size(75, 24)
            Me.btnFinish.TabIndex = 10
            Me.btnFinish.Text = "&Finish"
            '
            'btnNext
            '
            Me.btnNext.Anchor = CType((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.btnNext.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.btnNext.Location = New System.Drawing.Point(288, 400)
            Me.btnNext.Name = "btnNext"
            Me.btnNext.Size = New System.Drawing.Size(75, 24)
            Me.btnNext.TabIndex = 9
            Me.btnNext.Text = "&Next >"
            '
            'btnBack
            '
            Me.btnBack.Anchor = CType((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.btnBack.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.btnBack.Location = New System.Drawing.Point(208, 400)
            Me.btnBack.Name = "btnBack"
            Me.btnBack.Size = New System.Drawing.Size(75, 24)
            Me.btnBack.TabIndex = 8
            Me.btnBack.Text = "< &Back"
            '
            'chkPreviewCode
            '
            Me.chkPreviewCode.Anchor = CType((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Left), System.Windows.Forms.AnchorStyles)
            Me.chkPreviewCode.Appearance = System.Windows.Forms.Appearance.Button
            Me.chkPreviewCode.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.chkPreviewCode.Location = New System.Drawing.Point(8, 400)
            Me.chkPreviewCode.Name = "chkPreviewCode"
            Me.chkPreviewCode.Size = New System.Drawing.Size(112, 24)
            Me.chkPreviewCode.TabIndex = 7
            Me.chkPreviewCode.Text = "Preview &Code"
            Me.chkPreviewCode.TextAlign = System.Drawing.ContentAlignment.MiddleCenter
            '
            'PicFormatWizardForm
            '
            Me.AcceptButton = Me.btnFinish
            Me.AutoScaleBaseSize = New System.Drawing.Size(5, 13)
            Me.CancelButton = Me.btnCancel
            Me.ClientSize = New System.Drawing.Size(608, 438)
            Me.Controls.Add(Me.chkPreviewCode)
            Me.Controls.Add(Me.btnHelp)
            Me.Controls.Add(Me.gboxControlArea)
            Me.Controls.Add(Me.picSASLogo)
            Me.Controls.Add(Me.lblPanelTitle)
            Me.Controls.Add(Me.lblOfPages)
            Me.Controls.Add(Me.lblPageNo)
            Me.Controls.Add(Me.picStepHeader)
            Me.Controls.Add(Me.btnCancel)
            Me.Controls.Add(Me.btnFinish)
            Me.Controls.Add(Me.btnNext)
            Me.Controls.Add(Me.btnBack)
            Me.MaximizeBox = False
            Me.MinimizeBox = False
            Me.Name = "PicFormatWizardForm"
            Me.ShowInTaskbar = False
            Me.SizeGripStyle = System.Windows.Forms.SizeGripStyle.Show
            Me.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent
            Me.Text = "PicFormatWizardForm"
            Me.ResumeLayout(False)

        End Sub
#End Region

#Region " Child Control Event Handlers "
        Private Sub btnBack_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnBack.Click
            ' Go back a page if it's valid to.
            If (_CurrentPage > 1) Then
                CurrentPage = _CurrentPage - 1
            End If
        End Sub

        Private Sub btnNext_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnNext.Click
            ' Go forward a page if it's valid to.
            If (_CurrentPage < _TotalPages) Then
                CurrentPage = _CurrentPage + 1
            End If

        End Sub

        Private Sub chkPreviewCode_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles chkPreviewCode.CheckedChanged
            If (chkPreviewCode.Checked = True) Then
                ' We have to display the code preview window, so start by
                ' createing the form if it doesn't already exist.
                If (_CodePreviewForm Is Nothing) Then
                    _CodePreviewForm = New CodePreviewForm
                End If
                ' Load the current task generated code into the dialog's
                ' text control, remove the selection an make it an
                ' always-on-top window.
                _CodePreviewForm.txtCode.Text = PicFormat.BuildSASCode(GeneralParams, DefinitionParams)
                _CodePreviewForm.txtCode.SelectionStart = 0
                _CodePreviewForm.txtCode.SelectionLength = 0
                _CodePreviewForm.TopMost = True
                ' Show the form as a modless window.
                _CodePreviewForm.Show()
            Else
                ' It's time to get rid of the code preview window.
                If Not (_CodePreviewForm Is Nothing) Then
                    ' Close it and effectively destroy it.
                    _CodePreviewForm.Close()
                    _CodePreviewForm = Nothing
                End If
            End If
        End Sub

        Private Sub _CodePreviewForm_Closing(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles _CodePreviewForm.Closing
            ' This event will be fired when the user presses the
            ' close button on the top right of the dialog's title
            ' bar.
            ' We just need to change the Preview Code check button
            ' to false and the chkPreviewCode_CheckedChanged event
            ' handler will take care of destroying the form for us.
            chkPreviewCode.Checked = False
        End Sub

        Private Sub _Panel1_ParametersUpdated(ByVal sender As Object, ByVal ea As System.EventArgs) Handles _Panel1.ParametersUpdated
            ' This event handler is activated whenever any of the
            ' properties on the general panel are modified. This
            ' gives us the opportuity to update the code in the
            ' code preview window (if it's enabled) and to enable
            ' or disable the Finish button as needed.
            If (chkPreviewCode.Checked = True) Then
                If Not (_CodePreviewForm Is Nothing) Then
                    _CodePreviewForm.txtCode.Text = PicFormat.BuildSASCode(GeneralParams, DefinitionParams)
                    _CodePreviewForm.txtCode.SelectionStart = 0
                    _CodePreviewForm.txtCode.SelectionLength = 0
                End If
            End If

            btnFinish.Enabled = (_Panel1.IsAbleToBeRun = True And _Panel2.IsAbleToBeRun = True)
        End Sub

        Private Sub _Panel2_ParametersUpdated(ByVal sender As Object, ByVal ea As System.EventArgs) Handles _Panel2.ParametersUpdated
            ' This event handler is activated whenever any of the
            ' properties on the range definition panel are modified.
            ' This gives us the opportuity to update the code in the
            ' code preview window (if it's enabled) and to enable
            ' or disable the Finish button as needed.
            If (chkPreviewCode.Checked = True) Then
                If Not (_CodePreviewForm Is Nothing) Then
                    _CodePreviewForm.txtCode.Text = PicFormat.BuildSASCode(GeneralParams, DefinitionParams)
                    _CodePreviewForm.txtCode.SelectionStart = 0
                    _CodePreviewForm.txtCode.SelectionLength = 0
                End If
            End If

            btnFinish.Enabled = (_Panel1.IsAbleToBeRun = True And _Panel2.IsAbleToBeRun = True)
        End Sub

        Private Sub _Panel1_PictureTypeChanged(ByVal sender As Object, ByVal newType As Integer) Handles _Panel1.PictureTypeChanged
            ' The general panel is reporting that the picture type
            ' has changed, so we need to pass the new value on to
            ' the range definitions panel.
            _Panel2.PictureType = newType
        End Sub

        Private Sub _Panel1_ServerNameChanged(ByVal sender As Object, ByVal newServerName As String) Handles _Panel1.ServerNameChanged
            ' The general panel is reporting that the server name
            ' has changed, so we need to pass the new value on to
            ' the range definitions panel.
            _Panel2.ServerName = newServerName
        End Sub

        Private Sub _Panel1_DefaultWidthChanged(ByVal sender As Object, ByVal newDefaultWidth As Integer) Handles _Panel1.DefaultWidthChanged
            ' The general panel is reporting that the default width
            ' has changed, so we need to pass the new value on to
            ' the range definitions panel.
            _Panel2.DefaultWidth = newDefaultWidth
        End Sub
#End Region

#Region " Overridden Methods "
        Protected Overrides Sub OnClosing(ByVal e As System.ComponentModel.CancelEventArgs)
            ' If the task form is closing, then we need to make
            ' sure that the code preview window is also closed. 
            If Not (_CodePreviewForm Is Nothing) Then
                _CodePreviewForm.Close()
                _CodePreviewForm = Nothing
            End If
        End Sub
#End Region

    End Class
End Namespace
