' =========================================================
'   
'                Create Picture Format Task
'
'                          a SAS Custom Task Example 
'
' Copyright (C) 2006 by SAS Institute Inc., Cary, NC 27513
'
' ---------------------------------------------------------
' This form is used to specify the properties for an
' individual range.
' When it first comes up it will display the current range
' values and will check any modifications made to the range
' against the limits range. The limits range defines the 
' smallest and largest values acceptable for the range so
' as to prevent the defined range from overlapping any
' other defined range. 
' =========================================================

Imports System.Globalization
Imports System.Drawing
Imports System.Windows.Forms

Namespace SASVB.CustomTasks
    Public Class RangeEditor
        Inherits System.Windows.Forms.Form

#Region " Member Variables "
        ' These variables are for storing Property values.
        Private _OptionsAreShown As Boolean = False
        Private _RangeLimits As RangeItem = New RangeItem
        Private _OutputRangeItem As RangeItem = New RangeItem
        Private _AddMode As Boolean = True
        Private _PictureType As Int32 = -1
#End Region

#Region " Properties "
        ' Shows or hides the options part of the dialog.
        Public Property OptionsAreShown() As Boolean
            Get
                Return _OptionsAreShown
            End Get
            Set(ByVal Value As Boolean)
                ' Hide or show the options and resize the window
                ' by calling our helper method.
                ShowOptions(Value)
            End Set
        End Property

        ' Returns the currently defined RangeItem.
        Public ReadOnly Property RangeItem() As RangeItem
            Get
                Return _OutputRangeItem
            End Get
        End Property

        ' Specifies whether the form is to be in Add mode or
        ' Modify mode. (There isn't actually too much
        ' difference between the two modes.)
        Public Property AddMode() As Boolean
            Get
                Return _AddMode
            End Get
            Set(ByVal Value As Boolean)
                If (_AddMode <> Value) Then
                    _AddMode = Value

                    ' Update the title bar text and the button
                    ' text depending on the mode we're in.
                    If (_AddMode = True) Then
                        Me.Text = "Add New Range"
                        btnAdd.Text = "Add"
                    Else
                        Me.Text = "Modify Existing Range"
                        btnAdd.Text = "Modify"
                    End If
                End If
            End Set
        End Property

        ' The PictureType propert affects the way we display the
        ' range values, the controls we provide for modifying
        ' the values, and the menu items displayed from the
        ' picture mask Directives listed in the context menu.
        Public Property PictureType() As Int32
            Get
                Return _PictureType
            End Get
            Set(ByVal Value As Int32)
                ' Start by checking that the suplied value for the
                ' picture type is within range.
                If (Value < 0 Or Value > 3) Then
                    Return
                End If

                ' We only need to do something if the new value
                ' differs from the current one.
                If (_PictureType <> Value) Then
                    ' Store the new value.
                    _PictureType = Value

                    ' Enable or disable the picture mask Directives
                    ' listed in the context menu.
                    mitmLeadingSpace.Enabled = (Value = 0)
                    mitmLeadingZero.Enabled = (Value = 0)
                    mitmAbbrvWkDayName.Enabled = (Value = 1 Or Value = 3)
                    mitmFullWkDayName.Enabled = (Value = 1 Or Value = 3)
                    mitmAbbrevMonthName.Enabled = (Value = 1 Or Value = 3)
                    mitmFullMonthName.Enabled = (Value = 1 Or Value = 3)
                    mitmYearWOCentury.Enabled = (Value = 1 Or Value = 3)
                    mitmYearWCentury.Enabled = (Value = 1 Or Value = 3)
                    mitmMonth.Enabled = (Value = 1 Or Value = 3)
                    mitmDayOfMonth.Enabled = (Value = 1 Or Value = 3)
                    mitmDayOfYear.Enabled = (Value = 1 Or Value = 3)
                    mitmWeekday.Enabled = (Value = 1 Or Value = 3)
                    mitmWeekNumber.Enabled = (Value = 1 Or Value = 3)
                    mitmHour24.Enabled = (Value = 2 Or Value = 3)
                    mitmHour12.Enabled = (Value = 2 Or Value = 3)
                    mitmMinute.Enabled = (Value = 2 Or Value = 3)
                    mitmSecond.Enabled = (Value = 2 Or Value = 3)
                    mitmAMPM.Enabled = (Value = 2 Or Value = 3)
                    mitmPercent.Enabled = True

                    ' Enable the appropriate input controls for the
                    ' new picture type.
                    txtStartValue.Visible = (Value = 0)
                    dateStartValue.Visible = (Value = 1 Or Value = 3)
                    timeStartValue.Visible = (Value = 2 Or Value = 3)

                    txtEndValue.Visible = (Value = 0)
                    dateEndValue.Visible = (Value = 1 Or Value = 3)
                    timeEndValue.Visible = (Value = 2 Or Value = 3)

                    ' It just doesn't make sense to have a multiplier
                    ' specified on a date, time or datetime value, so
                    ' we're disabling the multiplier controls if the
                    ' picture type is not Numeric.
                    chkUseMultiplier.Enabled = (Value = 0)
                    txtMultiplier.Enabled = (Value = 0 And chkUseMultiplier.Checked = True)

                    ' Lastly, we need to adjust the positions of the
                    ' value input controls depending on the picture
                    ' type.
                    Select Case (Value)
                        Case 1
                            dateStartValue.Location = txtStartValue.Location
                            dateEndValue.Location = txtEndValue.Location
                        Case 2
                            timeStartValue.Location = txtStartValue.Location
                            timeEndValue.Location = txtEndValue.Location
                        Case 3
                            dateStartValue.Location = txtStartValue.Location
                            dateEndValue.Location = txtEndValue.Location
                            ' The time input control needs to start
                            ' immediately following the date contol. 
                            timeStartValue.Location = New Point(dateStartValue.Location.X + dateStartValue.Width, dateStartValue.Location.Y)
                            timeEndValue.Location = New Point(dateEndValue.Location.X + dateEndValue.Width, dateEndValue.Location.Y)
                    End Select
                End If
            End Set
        End Property

        Public Sub SetRanges(ByVal rangeLimits As RangeItem, ByVal inputRange As RangeItem)
            ' Start by checking for inconsistancies in the two supplied ranges.
            If (rangeLimits.Low = False) Then
                If (inputRange.Low = True Or _
                    rangeLimits.StartValue > inputRange.StartValue Or _
                    (rangeLimits.StartValue = inputRange.StartValue And _
                     rangeLimits.IncludeStartValue = False And _
                     inputRange.IncludeStartValue = True)) Then
                    Throw New Exception("There is a conflict between the start value in the range limits range and the input range.")
                End If
            End If

            If (rangeLimits.High = False) Then
                If (inputRange.High = True Or _
                    rangeLimits.EndValue < inputRange.EndValue Or _
                    (rangeLimits.EndValue = inputRange.EndValue And _
                     rangeLimits.IncludeEndValue = False And _
                     inputRange.IncludeEndValue = True)) Then
                    Throw New Exception("There is a conflict between the end value in the range limits range and the input range.")
                End If
            End If

            ' The limit range and the input range seem to be acceptable,
            ' so, now we store the range information.
            _RangeLimits = rangeLimits
            _OutputRangeItem = inputRange.Clone()

            ' Populate the input controls with the various values from
            ' the range input range.
            UpdateControlValues()

            If (_RangeLimits.Low = False) Then
                ' If Low is beyond the allowable range limits then we may
                ' as well make it unavailable.
                optLow.Enabled = False
                ' Also disable the include start value checkbox if it
                ' is not allowed for the lowest value.
                If (_RangeLimits.IncludeStartValue = False) Then
                    chkIncludeStartValue.Enabled = False
                End If
            End If

            If (_RangeLimits.High = False) Then
                ' If High is beyond the allowable range limits then we may
                ' as well make it unavailable.
                optHigh.Enabled = False
                ' Also disable the include end value checkbox if it
                ' is not allowed for the highest value.
                If (_RangeLimits.IncludeEndValue = False) Then
                    chkIncludeEndValue.Enabled = False
                End If
            End If
        End Sub
#End Region

#Region " Windows Form Designer generated code "

        Public Sub New()
            MyBase.New()

            'This call is required by the Windows Form Designer.
            InitializeComponent()

            ' Load the icon for the form's control menu on the title bar.
            Me.Icon = New Icon(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream("SASVB.CustomTasks.RangeEditor.ico"))

            ' Since the Form editor doesn't like us entering the Unicode
            ' character 0x25BA (the Black Right-Pointing Pointer) through
            ' the VS.NET UI, we'll simply assign the character at this
            ' point since the form has now been initialized.
            Me.btnMaskStringInsertCode.Text = ChrW(&H25BA)

            ' If the picture type has not yet been initialized, we'll
            ' make it a standard numeric type.
            If (_PictureType = -1) Then
                PictureType = 0
            End If

            ' Show/hide the options.
            ShowOptions(_OptionsAreShown)
        End Sub

        'Form overrides dispose to clean up the component list.
        Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
            If disposing Then
                If Not (components Is Nothing) Then
                    components.Dispose()
                End If
            End If
            MyBase.Dispose(disposing)
        End Sub

        'Required by the Windows Form Designer
        Private components As System.ComponentModel.IContainer

        'NOTE: The following procedure is required by the Windows Form Designer
        'It can be modified using the Windows Form Designer.  
        'Do not modify it using the code editor.
        Friend WithEvents btnAdd As System.Windows.Forms.Button
        Friend WithEvents btnCancel As System.Windows.Forms.Button
        Friend WithEvents grpRange As System.Windows.Forms.GroupBox
        Friend WithEvents grpStart As System.Windows.Forms.GroupBox
        Friend WithEvents txtStartValue As System.Windows.Forms.TextBox
        Friend WithEvents lblStartValue As System.Windows.Forms.Label
        Friend WithEvents optUseStartValue As System.Windows.Forms.RadioButton
        Friend WithEvents optLow As System.Windows.Forms.RadioButton
        Friend WithEvents chkIncludeStartValue As System.Windows.Forms.CheckBox
        Friend WithEvents grpEnd As System.Windows.Forms.GroupBox
        Friend WithEvents chkIncludeEndValue As System.Windows.Forms.CheckBox
        Friend WithEvents txtEndValue As System.Windows.Forms.TextBox
        Friend WithEvents lblEndValue As System.Windows.Forms.Label
        Friend WithEvents optUseEndValue As System.Windows.Forms.RadioButton
        Friend WithEvents optHigh As System.Windows.Forms.RadioButton
        Friend WithEvents lblMaskString As System.Windows.Forms.Label
        Friend WithEvents txtMaskString As System.Windows.Forms.TextBox
        Friend WithEvents chkNoEdit As System.Windows.Forms.CheckBox
        Friend WithEvents btnMaskStringHelp As System.Windows.Forms.Button
        Friend WithEvents btnShowOptions As System.Windows.Forms.Button
        Friend WithEvents grpOptions As System.Windows.Forms.GroupBox
        Friend WithEvents txtMultiplier As System.Windows.Forms.TextBox
        Friend WithEvents lblMultiplier As System.Windows.Forms.Label
        Friend WithEvents chkUseMultiplier As System.Windows.Forms.CheckBox
        Friend WithEvents txtFillChar As System.Windows.Forms.TextBox
        Friend WithEvents lblFillChar As System.Windows.Forms.Label
        Friend WithEvents chkUseFillChar As System.Windows.Forms.CheckBox
        Friend WithEvents lblPrefix As System.Windows.Forms.Label
        Friend WithEvents txtPrefix As System.Windows.Forms.TextBox
        Friend WithEvents chkUsePrefix As System.Windows.Forms.CheckBox
        Friend WithEvents btnMaskStringInsertCode As System.Windows.Forms.Button
        Friend WithEvents mitmAbbrvWkDayName As System.Windows.Forms.MenuItem
        Friend WithEvents mnuMaskStringPopup As System.Windows.Forms.ContextMenu
        Friend WithEvents mitmLeadingSpace As System.Windows.Forms.MenuItem
        Friend WithEvents mitmLeadingZero As System.Windows.Forms.MenuItem
        Friend WithEvents mitmFullWkDayName As System.Windows.Forms.MenuItem
        Friend WithEvents mitmAbbrevMonthName As System.Windows.Forms.MenuItem
        Friend WithEvents mitmFullMonthName As System.Windows.Forms.MenuItem
        Friend WithEvents mitmSepDate As System.Windows.Forms.MenuItem
        Friend WithEvents mitmYearWOCentury As System.Windows.Forms.MenuItem
        Friend WithEvents mitmYearWCentury As System.Windows.Forms.MenuItem
        Friend WithEvents mitmMonth As System.Windows.Forms.MenuItem
        Friend WithEvents mitmDayOfMonth As System.Windows.Forms.MenuItem
        Friend WithEvents mitmDayOfYear As System.Windows.Forms.MenuItem
        Friend WithEvents mitmWeekday As System.Windows.Forms.MenuItem
        Friend WithEvents mitmWeekNumber As System.Windows.Forms.MenuItem
        Friend WithEvents mitmSepTime As System.Windows.Forms.MenuItem
        Friend WithEvents mitmHour24 As System.Windows.Forms.MenuItem
        Friend WithEvents mitmHour12 As System.Windows.Forms.MenuItem
        Friend WithEvents mitmMinute As System.Windows.Forms.MenuItem
        Friend WithEvents mitmSecond As System.Windows.Forms.MenuItem
        Friend WithEvents mitmAMPM As System.Windows.Forms.MenuItem
        Friend WithEvents mitmSepSpecial As System.Windows.Forms.MenuItem
        Friend WithEvents mitmPercent As System.Windows.Forms.MenuItem
        Friend WithEvents dateEndValue As System.Windows.Forms.DateTimePicker
        Friend WithEvents timeEndValue As System.Windows.Forms.DateTimePicker
        Friend WithEvents dateStartValue As System.Windows.Forms.DateTimePicker
        Friend WithEvents timeStartValue As System.Windows.Forms.DateTimePicker
        <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
            Me.lblMaskString = New System.Windows.Forms.Label
            Me.txtMaskString = New System.Windows.Forms.TextBox
            Me.chkNoEdit = New System.Windows.Forms.CheckBox
            Me.btnMaskStringHelp = New System.Windows.Forms.Button
            Me.grpRange = New System.Windows.Forms.GroupBox
            Me.grpStart = New System.Windows.Forms.GroupBox
            Me.txtStartValue = New System.Windows.Forms.TextBox
            Me.lblStartValue = New System.Windows.Forms.Label
            Me.optUseStartValue = New System.Windows.Forms.RadioButton
            Me.optLow = New System.Windows.Forms.RadioButton
            Me.chkIncludeStartValue = New System.Windows.Forms.CheckBox
            Me.grpEnd = New System.Windows.Forms.GroupBox
            Me.chkIncludeEndValue = New System.Windows.Forms.CheckBox
            Me.txtEndValue = New System.Windows.Forms.TextBox
            Me.lblEndValue = New System.Windows.Forms.Label
            Me.optUseEndValue = New System.Windows.Forms.RadioButton
            Me.optHigh = New System.Windows.Forms.RadioButton
            Me.btnShowOptions = New System.Windows.Forms.Button
            Me.grpOptions = New System.Windows.Forms.GroupBox
            Me.txtMultiplier = New System.Windows.Forms.TextBox
            Me.lblMultiplier = New System.Windows.Forms.Label
            Me.chkUseMultiplier = New System.Windows.Forms.CheckBox
            Me.txtFillChar = New System.Windows.Forms.TextBox
            Me.lblFillChar = New System.Windows.Forms.Label
            Me.chkUseFillChar = New System.Windows.Forms.CheckBox
            Me.lblPrefix = New System.Windows.Forms.Label
            Me.txtPrefix = New System.Windows.Forms.TextBox
            Me.chkUsePrefix = New System.Windows.Forms.CheckBox
            Me.btnAdd = New System.Windows.Forms.Button
            Me.btnCancel = New System.Windows.Forms.Button
            Me.btnMaskStringInsertCode = New System.Windows.Forms.Button
            Me.mnuMaskStringPopup = New System.Windows.Forms.ContextMenu
            Me.mitmLeadingSpace = New System.Windows.Forms.MenuItem
            Me.mitmLeadingZero = New System.Windows.Forms.MenuItem
            Me.mitmSepDate = New System.Windows.Forms.MenuItem
            Me.mitmAbbrvWkDayName = New System.Windows.Forms.MenuItem
            Me.mitmFullWkDayName = New System.Windows.Forms.MenuItem
            Me.mitmAbbrevMonthName = New System.Windows.Forms.MenuItem
            Me.mitmFullMonthName = New System.Windows.Forms.MenuItem
            Me.mitmYearWOCentury = New System.Windows.Forms.MenuItem
            Me.mitmYearWCentury = New System.Windows.Forms.MenuItem
            Me.mitmMonth = New System.Windows.Forms.MenuItem
            Me.mitmDayOfMonth = New System.Windows.Forms.MenuItem
            Me.mitmDayOfYear = New System.Windows.Forms.MenuItem
            Me.mitmWeekday = New System.Windows.Forms.MenuItem
            Me.mitmWeekNumber = New System.Windows.Forms.MenuItem
            Me.mitmSepTime = New System.Windows.Forms.MenuItem
            Me.mitmHour24 = New System.Windows.Forms.MenuItem
            Me.mitmHour12 = New System.Windows.Forms.MenuItem
            Me.mitmMinute = New System.Windows.Forms.MenuItem
            Me.mitmSecond = New System.Windows.Forms.MenuItem
            Me.mitmAMPM = New System.Windows.Forms.MenuItem
            Me.mitmSepSpecial = New System.Windows.Forms.MenuItem
            Me.mitmPercent = New System.Windows.Forms.MenuItem
            Me.dateEndValue = New System.Windows.Forms.DateTimePicker
            Me.timeEndValue = New System.Windows.Forms.DateTimePicker
            Me.dateStartValue = New System.Windows.Forms.DateTimePicker
            Me.timeStartValue = New System.Windows.Forms.DateTimePicker
            Me.grpRange.SuspendLayout()
            Me.grpStart.SuspendLayout()
            Me.grpEnd.SuspendLayout()
            Me.grpOptions.SuspendLayout()
            Me.SuspendLayout()
            '
            'lblMaskString
            '
            Me.lblMaskString.Location = New System.Drawing.Point(8, 8)
            Me.lblMaskString.Name = "lblMaskString"
            Me.lblMaskString.Size = New System.Drawing.Size(96, 16)
            Me.lblMaskString.TabIndex = 0
            Me.lblMaskString.Text = "Mask string:"
            '
            'txtMaskString
            '
            Me.txtMaskString.Anchor = CType(((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.txtMaskString.Location = New System.Drawing.Point(104, 6)
            Me.txtMaskString.MaxLength = 40
            Me.txtMaskString.Name = "txtMaskString"
            Me.txtMaskString.Size = New System.Drawing.Size(174, 20)
            Me.txtMaskString.TabIndex = 1
            Me.txtMaskString.Text = ""
            '
            'chkNoEdit
            '
            Me.chkNoEdit.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.chkNoEdit.Location = New System.Drawing.Point(32, 30)
            Me.chkNoEdit.Name = "chkNoEdit"
            Me.chkNoEdit.Size = New System.Drawing.Size(328, 16)
            Me.chkNoEdit.TabIndex = 4
            Me.chkNoEdit.Text = "Treat the mask string as plain text, not a mask"
            '
            'btnMaskStringHelp
            '
            Me.btnMaskStringHelp.Anchor = CType((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.btnMaskStringHelp.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.btnMaskStringHelp.Location = New System.Drawing.Point(306, 4)
            Me.btnMaskStringHelp.Name = "btnMaskStringHelp"
            Me.btnMaskStringHelp.Size = New System.Drawing.Size(54, 24)
            Me.btnMaskStringHelp.TabIndex = 3
            Me.btnMaskStringHelp.Text = "Help"
            '
            'grpRange
            '
            Me.grpRange.Anchor = CType(((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.grpRange.Controls.Add(Me.grpStart)
            Me.grpRange.Controls.Add(Me.grpEnd)
            Me.grpRange.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.grpRange.Location = New System.Drawing.Point(8, 54)
            Me.grpRange.Name = "grpRange"
            Me.grpRange.Size = New System.Drawing.Size(352, 256)
            Me.grpRange.TabIndex = 5
            Me.grpRange.TabStop = False
            Me.grpRange.Text = "Range"
            '
            'grpStart
            '
            Me.grpStart.Anchor = CType(((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.grpStart.Controls.Add(Me.timeStartValue)
            Me.grpStart.Controls.Add(Me.dateStartValue)
            Me.grpStart.Controls.Add(Me.txtStartValue)
            Me.grpStart.Controls.Add(Me.lblStartValue)
            Me.grpStart.Controls.Add(Me.optUseStartValue)
            Me.grpStart.Controls.Add(Me.optLow)
            Me.grpStart.Controls.Add(Me.chkIncludeStartValue)
            Me.grpStart.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.grpStart.Location = New System.Drawing.Point(8, 16)
            Me.grpStart.Name = "grpStart"
            Me.grpStart.Size = New System.Drawing.Size(336, 112)
            Me.grpStart.TabIndex = 0
            Me.grpStart.TabStop = False
            Me.grpStart.Text = "Start"
            '
            'txtStartValue
            '
            Me.txtStartValue.Anchor = CType(((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.txtStartValue.Location = New System.Drawing.Point(120, 60)
            Me.txtStartValue.Name = "txtStartValue"
            Me.txtStartValue.Size = New System.Drawing.Size(200, 20)
            Me.txtStartValue.TabIndex = 3
            Me.txtStartValue.Text = ""
            '
            'lblStartValue
            '
            Me.lblStartValue.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.lblStartValue.Location = New System.Drawing.Point(24, 62)
            Me.lblStartValue.Name = "lblStartValue"
            Me.lblStartValue.Size = New System.Drawing.Size(96, 16)
            Me.lblStartValue.TabIndex = 2
            Me.lblStartValue.Text = "Start value:"
            '
            'optUseStartValue
            '
            Me.optUseStartValue.Anchor = CType(((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.optUseStartValue.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.optUseStartValue.Location = New System.Drawing.Point(8, 40)
            Me.optUseStartValue.Name = "optUseStartValue"
            Me.optUseStartValue.Size = New System.Drawing.Size(320, 16)
            Me.optUseStartValue.TabIndex = 1
            Me.optUseStartValue.Text = "Specify the start value"
            '
            'optLow
            '
            Me.optLow.Anchor = CType(((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.optLow.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.optLow.Location = New System.Drawing.Point(8, 20)
            Me.optLow.Name = "optLow"
            Me.optLow.Size = New System.Drawing.Size(320, 16)
            Me.optLow.TabIndex = 0
            Me.optLow.Text = "Low (the smallest possible value)"
            '
            'chkIncludeStartValue
            '
            Me.chkIncludeStartValue.Anchor = CType(((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.chkIncludeStartValue.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.chkIncludeStartValue.Location = New System.Drawing.Point(32, 84)
            Me.chkIncludeStartValue.Name = "chkIncludeStartValue"
            Me.chkIncludeStartValue.Size = New System.Drawing.Size(296, 16)
            Me.chkIncludeStartValue.TabIndex = 4
            Me.chkIncludeStartValue.Text = "Include the start value within the range"
            '
            'grpEnd
            '
            Me.grpEnd.Anchor = CType(((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.grpEnd.Controls.Add(Me.timeEndValue)
            Me.grpEnd.Controls.Add(Me.dateEndValue)
            Me.grpEnd.Controls.Add(Me.chkIncludeEndValue)
            Me.grpEnd.Controls.Add(Me.txtEndValue)
            Me.grpEnd.Controls.Add(Me.lblEndValue)
            Me.grpEnd.Controls.Add(Me.optUseEndValue)
            Me.grpEnd.Controls.Add(Me.optHigh)
            Me.grpEnd.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.grpEnd.Location = New System.Drawing.Point(8, 136)
            Me.grpEnd.Name = "grpEnd"
            Me.grpEnd.Size = New System.Drawing.Size(336, 112)
            Me.grpEnd.TabIndex = 1
            Me.grpEnd.TabStop = False
            Me.grpEnd.Text = "End"
            '
            'chkIncludeEndValue
            '
            Me.chkIncludeEndValue.Anchor = CType(((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.chkIncludeEndValue.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.chkIncludeEndValue.Location = New System.Drawing.Point(32, 84)
            Me.chkIncludeEndValue.Name = "chkIncludeEndValue"
            Me.chkIncludeEndValue.Size = New System.Drawing.Size(296, 16)
            Me.chkIncludeEndValue.TabIndex = 6
            Me.chkIncludeEndValue.Text = "Include the end value within the range"
            '
            'txtEndValue
            '
            Me.txtEndValue.Anchor = CType(((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.txtEndValue.Location = New System.Drawing.Point(120, 60)
            Me.txtEndValue.Name = "txtEndValue"
            Me.txtEndValue.Size = New System.Drawing.Size(200, 20)
            Me.txtEndValue.TabIndex = 3
            Me.txtEndValue.Text = ""
            '
            'lblEndValue
            '
            Me.lblEndValue.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.lblEndValue.Location = New System.Drawing.Point(24, 62)
            Me.lblEndValue.Name = "lblEndValue"
            Me.lblEndValue.Size = New System.Drawing.Size(96, 16)
            Me.lblEndValue.TabIndex = 2
            Me.lblEndValue.Text = "End value:"
            '
            'optUseEndValue
            '
            Me.optUseEndValue.Anchor = CType(((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.optUseEndValue.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.optUseEndValue.Location = New System.Drawing.Point(8, 40)
            Me.optUseEndValue.Name = "optUseEndValue"
            Me.optUseEndValue.Size = New System.Drawing.Size(320, 16)
            Me.optUseEndValue.TabIndex = 1
            Me.optUseEndValue.Text = "Specify the end value"
            '
            'optHigh
            '
            Me.optHigh.Anchor = CType(((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.optHigh.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.optHigh.Location = New System.Drawing.Point(8, 20)
            Me.optHigh.Name = "optHigh"
            Me.optHigh.Size = New System.Drawing.Size(320, 16)
            Me.optHigh.TabIndex = 0
            Me.optHigh.Text = "High (the greatest possible value)"
            '
            'btnShowOptions
            '
            Me.btnShowOptions.Anchor = CType((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Left), System.Windows.Forms.AnchorStyles)
            Me.btnShowOptions.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.btnShowOptions.Location = New System.Drawing.Point(8, 496)
            Me.btnShowOptions.Name = "btnShowOptions"
            Me.btnShowOptions.Size = New System.Drawing.Size(120, 24)
            Me.btnShowOptions.TabIndex = 7
            Me.btnShowOptions.Text = "<<< Hide Options"
            '
            'grpOptions
            '
            Me.grpOptions.Anchor = CType(((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.grpOptions.Controls.Add(Me.txtMultiplier)
            Me.grpOptions.Controls.Add(Me.lblMultiplier)
            Me.grpOptions.Controls.Add(Me.chkUseMultiplier)
            Me.grpOptions.Controls.Add(Me.txtFillChar)
            Me.grpOptions.Controls.Add(Me.lblFillChar)
            Me.grpOptions.Controls.Add(Me.chkUseFillChar)
            Me.grpOptions.Controls.Add(Me.lblPrefix)
            Me.grpOptions.Controls.Add(Me.txtPrefix)
            Me.grpOptions.Controls.Add(Me.chkUsePrefix)
            Me.grpOptions.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.grpOptions.Location = New System.Drawing.Point(8, 320)
            Me.grpOptions.Name = "grpOptions"
            Me.grpOptions.Size = New System.Drawing.Size(352, 168)
            Me.grpOptions.TabIndex = 6
            Me.grpOptions.TabStop = False
            Me.grpOptions.Text = "Options"
            '
            'txtMultiplier
            '
            Me.txtMultiplier.Anchor = CType(((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.txtMultiplier.Location = New System.Drawing.Point(212, 136)
            Me.txtMultiplier.Name = "txtMultiplier"
            Me.txtMultiplier.Size = New System.Drawing.Size(128, 20)
            Me.txtMultiplier.TabIndex = 8
            Me.txtMultiplier.Text = "1"
            '
            'lblMultiplier
            '
            Me.lblMultiplier.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.lblMultiplier.Location = New System.Drawing.Point(24, 138)
            Me.lblMultiplier.Name = "lblMultiplier"
            Me.lblMultiplier.Size = New System.Drawing.Size(184, 16)
            Me.lblMultiplier.TabIndex = 7
            Me.lblMultiplier.Text = "Multiplier:"
            '
            'chkUseMultiplier
            '
            Me.chkUseMultiplier.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.chkUseMultiplier.Location = New System.Drawing.Point(8, 120)
            Me.chkUseMultiplier.Name = "chkUseMultiplier"
            Me.chkUseMultiplier.Size = New System.Drawing.Size(328, 16)
            Me.chkUseMultiplier.TabIndex = 6
            Me.chkUseMultiplier.Text = "Multiply the value before applying the mask"
            '
            'txtFillChar
            '
            Me.txtFillChar.Location = New System.Drawing.Point(212, 88)
            Me.txtFillChar.MaxLength = 1
            Me.txtFillChar.Name = "txtFillChar"
            Me.txtFillChar.Size = New System.Drawing.Size(24, 20)
            Me.txtFillChar.TabIndex = 5
            Me.txtFillChar.Text = ""
            '
            'lblFillChar
            '
            Me.lblFillChar.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.lblFillChar.Location = New System.Drawing.Point(24, 90)
            Me.lblFillChar.Name = "lblFillChar"
            Me.lblFillChar.Size = New System.Drawing.Size(184, 16)
            Me.lblFillChar.TabIndex = 4
            Me.lblFillChar.Text = "Fill character:"
            '
            'chkUseFillChar
            '
            Me.chkUseFillChar.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.chkUseFillChar.Location = New System.Drawing.Point(8, 72)
            Me.chkUseFillChar.Name = "chkUseFillChar"
            Me.chkUseFillChar.Size = New System.Drawing.Size(328, 16)
            Me.chkUseFillChar.TabIndex = 3
            Me.chkUseFillChar.Text = "Fill leading spaces with a character"
            '
            'lblPrefix
            '
            Me.lblPrefix.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.lblPrefix.Location = New System.Drawing.Point(24, 42)
            Me.lblPrefix.Name = "lblPrefix"
            Me.lblPrefix.Size = New System.Drawing.Size(184, 16)
            Me.lblPrefix.TabIndex = 1
            Me.lblPrefix.Text = "Prefix string (2 chars max):"
            '
            'txtPrefix
            '
            Me.txtPrefix.Location = New System.Drawing.Point(212, 40)
            Me.txtPrefix.MaxLength = 2
            Me.txtPrefix.Name = "txtPrefix"
            Me.txtPrefix.Size = New System.Drawing.Size(32, 20)
            Me.txtPrefix.TabIndex = 2
            Me.txtPrefix.Text = ""
            '
            'chkUsePrefix
            '
            Me.chkUsePrefix.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.chkUsePrefix.Location = New System.Drawing.Point(8, 24)
            Me.chkUsePrefix.Name = "chkUsePrefix"
            Me.chkUsePrefix.Size = New System.Drawing.Size(328, 16)
            Me.chkUsePrefix.TabIndex = 0
            Me.chkUsePrefix.Text = "Add a prefix to the formatted value"
            '
            'btnAdd
            '
            Me.btnAdd.Anchor = CType((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.btnAdd.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.btnAdd.Location = New System.Drawing.Point(208, 496)
            Me.btnAdd.Name = "btnAdd"
            Me.btnAdd.Size = New System.Drawing.Size(72, 24)
            Me.btnAdd.TabIndex = 8
            Me.btnAdd.Text = "Add"
            '
            'btnCancel
            '
            Me.btnCancel.Anchor = CType((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.btnCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel
            Me.btnCancel.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.btnCancel.Location = New System.Drawing.Point(288, 496)
            Me.btnCancel.Name = "btnCancel"
            Me.btnCancel.Size = New System.Drawing.Size(72, 24)
            Me.btnCancel.TabIndex = 9
            Me.btnCancel.Text = "Cancel"
            '
            'btnMaskStringInsertCode
            '
            Me.btnMaskStringInsertCode.AccessibleName = "Show mask string pop-up insert menu"
            Me.btnMaskStringInsertCode.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.btnMaskStringInsertCode.Font = New System.Drawing.Font("Arial", 8.25!)
            Me.btnMaskStringInsertCode.Location = New System.Drawing.Point(280, 6)
            Me.btnMaskStringInsertCode.Name = "btnMaskStringInsertCode"
            Me.btnMaskStringInsertCode.Size = New System.Drawing.Size(20, 20)
            Me.btnMaskStringInsertCode.TabIndex = 2
            Me.btnMaskStringInsertCode.Text = "?"
            '
            'mnuMaskStringPopup
            '
            Me.mnuMaskStringPopup.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.mitmLeadingSpace, Me.mitmLeadingZero, Me.mitmSepDate, Me.mitmAbbrvWkDayName, Me.mitmFullWkDayName, Me.mitmAbbrevMonthName, Me.mitmFullMonthName, Me.mitmYearWOCentury, Me.mitmYearWCentury, Me.mitmMonth, Me.mitmDayOfMonth, Me.mitmDayOfYear, Me.mitmWeekday, Me.mitmWeekNumber, Me.mitmSepTime, Me.mitmHour24, Me.mitmHour12, Me.mitmMinute, Me.mitmSecond, Me.mitmAMPM, Me.mitmSepSpecial, Me.mitmPercent})
            '
            'mitmLeadingSpace
            '
            Me.mitmLeadingSpace.Index = 0
            Me.mitmLeadingSpace.Text = "0   Show leading zero as space"
            '
            'mitmLeadingZero
            '
            Me.mitmLeadingZero.Index = 1
            Me.mitmLeadingZero.Text = "9   Show leading zero"
            '
            'mitmSepDate
            '
            Me.mitmSepDate.Index = 2
            Me.mitmSepDate.Text = "-"
            '
            'mitmAbbrvWkDayName
            '
            Me.mitmAbbrvWkDayName.Index = 3
            Me.mitmAbbrvWkDayName.Text = "%a   Locale's abbreviated weekday name"
            '
            'mitmFullWkDayName
            '
            Me.mitmFullWkDayName.Index = 4
            Me.mitmFullWkDayName.Text = "%A   Locale's full weekday name"
            '
            'mitmAbbrevMonthName
            '
            Me.mitmAbbrevMonthName.Index = 5
            Me.mitmAbbrevMonthName.Text = "%b   Locale's abbreviated month name"
            '
            'mitmFullMonthName
            '
            Me.mitmFullMonthName.Index = 6
            Me.mitmFullMonthName.Text = "%B   Locale's full month name"
            '
            'mitmYearWOCentury
            '
            Me.mitmYearWOCentury.Index = 7
            Me.mitmYearWOCentury.Text = "%y   Year without century as a decimal number (0-99), with no leading zero"
            '
            'mitmYearWCentury
            '
            Me.mitmYearWCentury.Index = 8
            Me.mitmYearWCentury.Text = "%Y   Year with century as a decimal number"
            '
            'mitmMonth
            '
            Me.mitmMonth.Index = 9
            Me.mitmMonth.Text = "%m   Month as a decimal number (1-12), with no leading zero"
            '
            'mitmDayOfMonth
            '
            Me.mitmDayOfMonth.Index = 10
            Me.mitmDayOfMonth.Text = "%d   Day of the month as a decimal number (1-31), with no leading zero"
            '
            'mitmDayOfYear
            '
            Me.mitmDayOfYear.Index = 11
            Me.mitmDayOfYear.Text = "%j   Day of the year as a decimal number (1-366), with no leading zero"
            '
            'mitmWeekday
            '
            Me.mitmWeekday.Index = 12
            Me.mitmWeekday.Text = "%w   Weekday as a decimal number (1= Sunday, 7=Saturday)"
            '
            'mitmWeekNumber
            '
            Me.mitmWeekNumber.Index = 13
            Me.mitmWeekNumber.Text = "%U   Week number of the year (Sunday as the first day of the week) as a decimal n" & _
            "umber (0-53), with no leading zero"
            '
            'mitmSepTime
            '
            Me.mitmSepTime.Index = 14
            Me.mitmSepTime.Text = "-"
            '
            'mitmHour24
            '
            Me.mitmHour24.Index = 15
            Me.mitmHour24.Text = "%H   Hour (24-hour clock) as a decimal number (0-23), with no leading zero"
            '
            'mitmHour12
            '
            Me.mitmHour12.Index = 16
            Me.mitmHour12.Text = "%I   Hour (12-hour clock) as a decimal number (1-12), with no leading zero"
            '
            'mitmMinute
            '
            Me.mitmMinute.Index = 17
            Me.mitmMinute.Text = "%M   Minute as a decimal number (0-59), with no leading zero"
            '
            'mitmSecond
            '
            Me.mitmSecond.Index = 18
            Me.mitmSecond.Text = "%S   Second as a decimal number (0-59), with no leading zero"
            '
            'mitmAMPM
            '
            Me.mitmAMPM.Index = 19
            Me.mitmAMPM.Text = "%p   Locale's equivalent of either AM or PM"
            '
            'mitmSepSpecial
            '
            Me.mitmSepSpecial.Index = 20
            Me.mitmSepSpecial.Text = "-"
            '
            'mitmPercent
            '
            Me.mitmPercent.Index = 21
            Me.mitmPercent.Text = "%%   % character"
            '
            'dateEndValue
            '
            Me.dateEndValue.Format = System.Windows.Forms.DateTimePickerFormat.Short
            Me.dateEndValue.Location = New System.Drawing.Point(120, 64)
            Me.dateEndValue.Name = "dateEndValue"
            Me.dateEndValue.Size = New System.Drawing.Size(96, 20)
            Me.dateEndValue.TabIndex = 4
            Me.dateEndValue.Visible = False
            '
            'timeEndValue
            '
            Me.timeEndValue.Format = System.Windows.Forms.DateTimePickerFormat.Time
            Me.timeEndValue.Location = New System.Drawing.Point(216, 64)
            Me.timeEndValue.Name = "timeEndValue"
            Me.timeEndValue.ShowUpDown = True
            Me.timeEndValue.Size = New System.Drawing.Size(104, 20)
            Me.timeEndValue.TabIndex = 5
            Me.timeEndValue.Visible = False
            '
            'dateStartValue
            '
            Me.dateStartValue.Format = System.Windows.Forms.DateTimePickerFormat.Short
            Me.dateStartValue.Location = New System.Drawing.Point(120, 64)
            Me.dateStartValue.Name = "dateStartValue"
            Me.dateStartValue.Size = New System.Drawing.Size(96, 20)
            Me.dateStartValue.TabIndex = 5
            Me.dateStartValue.Visible = False
            '
            'timeStartValue
            '
            Me.timeStartValue.Format = System.Windows.Forms.DateTimePickerFormat.Time
            Me.timeStartValue.Location = New System.Drawing.Point(216, 64)
            Me.timeStartValue.Name = "timeStartValue"
            Me.timeStartValue.ShowUpDown = True
            Me.timeStartValue.Size = New System.Drawing.Size(104, 20)
            Me.timeStartValue.TabIndex = 6
            Me.timeStartValue.Visible = False
            '
            'RangeEditor
            '
            Me.AcceptButton = Me.btnAdd
            Me.AutoScaleBaseSize = New System.Drawing.Size(5, 13)
            Me.CancelButton = Me.btnCancel
            Me.ClientSize = New System.Drawing.Size(368, 526)
            Me.Controls.Add(Me.btnMaskStringInsertCode)
            Me.Controls.Add(Me.lblMaskString)
            Me.Controls.Add(Me.txtMaskString)
            Me.Controls.Add(Me.btnMaskStringHelp)
            Me.Controls.Add(Me.chkNoEdit)
            Me.Controls.Add(Me.grpRange)
            Me.Controls.Add(Me.btnShowOptions)
            Me.Controls.Add(Me.grpOptions)
            Me.Controls.Add(Me.btnAdd)
            Me.Controls.Add(Me.btnCancel)
            Me.MaximizeBox = False
            Me.MinimizeBox = False
            Me.Name = "RangeEditor"
            Me.ShowInTaskbar = False
            Me.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent
            Me.Text = "Add New Range"
            Me.grpRange.ResumeLayout(False)
            Me.grpStart.ResumeLayout(False)
            Me.grpEnd.ResumeLayout(False)
            Me.grpOptions.ResumeLayout(False)
            Me.ResumeLayout(False)

        End Sub

#End Region

#Region " Control Event Handlers "
        Private Sub btnShowOptions_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnShowOptions.Click
            ' Call the helper method to hide or show the options
            ' as necessary.
            ShowOptions(_OptionsAreShown <> True)
        End Sub

        Private Sub optLow_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles optLow.CheckedChanged
            ' Store the new state of the Low option
            _OutputRangeItem.Low = (optLow.Checked = True)
            ' If Low is selected then we disable all of the
            ' low value input fields since their values are
            ' not going to be needed.
            txtStartValue.Enabled = (optLow.Checked = False)
            dateStartValue.Enabled = (optLow.Checked = False)
            timeStartValue.Enabled = (optLow.Checked = False)
            chkIncludeStartValue.Enabled = (optLow.Checked = False)
        End Sub

        Private Sub optHigh_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles optHigh.CheckedChanged
            ' Store the new state of the High option
            _OutputRangeItem.High = (optHigh.Checked = True)
            ' If High is selected then we disable all of the
            ' high value input fields since their values are
            ' not going to be needed.
            txtEndValue.Enabled = (optHigh.Checked = False)
            dateEndValue.Enabled = (optHigh.Checked = False)
            timeEndValue.Enabled = (optHigh.Checked = False)
            chkIncludeEndValue.Enabled = (optHigh.Checked = False)
        End Sub

        Private Sub txtStartValue_Validating(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles txtStartValue.Validating
            Dim doubleValue As Double = Double.NaN
            Dim rangeMsg As String

            ' Check that the value entered is a proper Double value and
            ' extract the Double if it is.
            If (Double.TryParse(txtStartValue.Text, Globalization.NumberStyles.Float, NumberFormatInfo.InvariantInfo, doubleValue) = False) Then
                ' The validation failed.
                e.Cancel = True
                MessageBox.Show(Me, "The value entered in the Start Value field is invalid." + vbCrLf + _
                        "It must consist of a single integer, floating point, or scientific notation value.", _
                        "Invalid Numeric Value")
                Return
            End If

            ' If the Low range limit is not set and the Double value
            ' entered is less than the range limit, then we have a
            ' problem. 
            If (_RangeLimits.Low = False And doubleValue < _RangeLimits.StartValue) Then
                ' The validation has failed.
                e.Cancel = True
                If (_RangeLimits.High = True) Then
                    rangeMsg = String.Format("The value must be greater than or equal to {0}.", _RangeLimits.StartValue.ToString())
                Else
                    rangeMsg = String.Format("The value must be greater than or equal to {0} and less than or equal to {1}.", _RangeLimits.StartValue.ToString(), _RangeLimits.EndValue.ToString())
                End If
                MessageBox.Show(Me, "The value entered in the Start Value field is out of range." + _
                        vbCrLf + rangeMsg, "Invalid Numeric Value")
                Return
            End If

            ' If we got here, then the value should be acceptable.
            e.Cancel = False
        End Sub

        Private Sub txtStartValue_Validated(ByVal sender As Object, ByVal e As System.EventArgs) Handles txtStartValue.Validated
            Dim doubleValue As Double = Double.NaN

            ' Extract the Double value from the textbox.
            If (Double.TryParse(txtStartValue.Text, Globalization.NumberStyles.Float, NumberFormatInfo.InvariantInfo, doubleValue) = False) Then
                Throw New Exception("Somehow, we got to the Validated event handler for txtStartValue with an invalid double string. This would be one of those internal errors!")
            End If

            ' We are assuming here that all the range validation was
            ' taken care of in the Validating event handler.
            _OutputRangeItem.StartValue = doubleValue
            ' While where at it, we'll update the date and time controls
            ' with the new value as well.
            dateStartValue.Value = UtilityMethods.SASDateToDotNetDateTime(doubleValue)
            timeStartValue.Value = dateStartValue.Value

            ' If the new value is equal to the range limit value and
            ' the range limit does not include the start value, then
            ' we need to make sure that we don't include it either.
            If (doubleValue = _RangeLimits.StartValue) Then
                If (_RangeLimits.IncludeStartValue = False) Then
                    If (_OutputRangeItem.IncludeStartValue = True) Then
                        chkIncludeStartValue.Checked = False
                    End If
                    chkIncludeStartValue.Enabled = False
                End If
            Else
                If (chkIncludeStartValue.Enabled = False) Then
                    chkIncludeStartValue.Enabled = True
                End If
            End If
        End Sub

        Private Sub txtStartValue_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles txtStartValue.TextChanged
            Dim doubleValue As Double = Double.NaN

            ' Although the enabling or disabling of the include start 
            ' checkbox is handled more fully in the Validated handler,
            ' we want to do some preliminary enablement/disablement
            ' based on the current value typed in the field. We want
            ' to do this so that the user will be able to interact
            ' with the checkbox without having to move focus from the
            ' textbox first.
            If (Double.TryParse(txtStartValue.Text, Globalization.NumberStyles.Float, NumberFormatInfo.InvariantInfo, doubleValue) = False) Then
                If (chkIncludeStartValue.Enabled = False) Then
                    chkIncludeStartValue.Enabled = True
                End If
            Else
                If (doubleValue = _RangeLimits.StartValue) Then
                    If (_RangeLimits.IncludeStartValue = False And chkIncludeStartValue.Enabled = True) Then
                        chkIncludeStartValue.Enabled = False
                    End If
                Else
                    If (chkIncludeStartValue.Enabled = False) Then
                        chkIncludeStartValue.Enabled = True
                    End If
                End If
            End If
        End Sub

        Private Sub chkIncludeStartValue_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles chkIncludeStartValue.CheckedChanged
            ' Store the checkbox state in the range item.
            _OutputRangeItem.IncludeStartValue = chkIncludeStartValue.Checked
        End Sub

        Private Sub txtEndValue_Validating(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles txtEndValue.Validating
            Dim doubleValue As Double = Double.NaN
            Dim rangeMsg As String

            ' Check that the value entered is a proper Double value and
            ' extract the Double if it is.
            If (Double.TryParse(txtEndValue.Text, Globalization.NumberStyles.Float, NumberFormatInfo.InvariantInfo, doubleValue) = False) Then
                ' The validation failed.
                e.Cancel = True
                MessageBox.Show(Me, "The value entered in the End Value field is invalid." + vbCrLf + _
                        "It must consist of a single integer, floating point, or scientific notation value.", _
                        "Invalid Numeric Value")
                Return
            End If

            ' If the High range limit is not set and the Double value
            ' entered is greater than the range limit, then we have a
            ' problem. 
            If (_RangeLimits.High = False And doubleValue > _RangeLimits.EndValue) Then
                ' The validation has failed.
                e.Cancel = True
                If (_RangeLimits.Low = True) Then
                    rangeMsg = String.Format("The value must be less than or equal to {0}.", _RangeLimits.EndValue.ToString())
                Else
                    rangeMsg = String.Format("The value must be greater than or equal to {0} and less than or equal to {1}.", _RangeLimits.StartValue.ToString(), _RangeLimits.EndValue.ToString())
                End If
                MessageBox.Show(Me, "The value entered in the End Value field is out of range." + _
                        vbCrLf + rangeMsg, "Invalid Numeric Value")
                Return
            End If

            ' The value appears to be acceptable.
            e.Cancel = False
        End Sub

        Private Sub txtEndValue_Validated(ByVal sender As Object, ByVal e As System.EventArgs) Handles txtEndValue.Validated
            Dim doubleValue As Double = Double.NaN

            ' Extract the Double value from the textbox.
            If (Double.TryParse(txtEndValue.Text, Globalization.NumberStyles.Float, NumberFormatInfo.InvariantInfo, doubleValue) = False) Then
                Throw New Exception("Somehow, we got to the Validated event handler for txtEndValue with an invalid double string. This would be one of those internal errors!")
            End If

            ' We are assuming here that all the range validation was
            ' taken care of in the Validating event handler.
            _OutputRangeItem.EndValue = doubleValue
            ' While where at it, we'll update the date and time controls
            ' with the new value as well.
            dateEndValue.Value = UtilityMethods.SASDateToDotNetDateTime(doubleValue)
            timeEndValue.Value = dateEndValue.Value

            ' If the new value is equal to the range limit value and
            ' the range limit does not include the end value, then
            ' we need to make sure that we don't include it either.
            If (doubleValue = _RangeLimits.EndValue) Then
                If (_RangeLimits.IncludeEndValue = False) Then
                    If (_OutputRangeItem.IncludeEndValue = True) Then
                        chkIncludeEndValue.Checked = False
                    End If
                    chkIncludeEndValue.Enabled = False
                End If
            Else
                If (chkIncludeEndValue.Enabled = False) Then
                    chkIncludeEndValue.Enabled = True
                End If
            End If
        End Sub

        Private Sub txtEndValue_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles txtEndValue.TextChanged
            Dim doubleValue As Double = Double.NaN

            ' Although the enabling or disabling of the include end
            ' checkbox is handled more fully in the Validated handler,
            ' we want to do some preliminary enablement/disablement
            ' based on the current value typed in the field. We want
            ' to do this so that the user will be able to interact
            ' with the checkbox without having to move focus from the
            ' textbox first.
            If (Double.TryParse(txtEndValue.Text, Globalization.NumberStyles.Float, NumberFormatInfo.InvariantInfo, doubleValue) = False) Then
                If (chkIncludeEndValue.Enabled = False) Then
                    chkIncludeEndValue.Enabled = True
                End If
            Else
                If (doubleValue = _RangeLimits.EndValue) Then
                    If (_RangeLimits.IncludeEndValue = False And chkIncludeEndValue.Enabled = True) Then
                        chkIncludeEndValue.Enabled = False
                    End If
                Else
                    If (chkIncludeEndValue.Enabled = False) Then
                        chkIncludeEndValue.Enabled = True
                    End If
                End If
            End If
        End Sub

        Private Sub chkIncludeEndValue_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles chkIncludeEndValue.CheckedChanged
            ' Store the checkbox state in the range item.
            _OutputRangeItem.IncludeEndValue = chkIncludeEndValue.Checked
        End Sub

        Private Sub dateStartValue_Validating(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles dateStartValue.Validating
            Dim doubleValue As Double = Double.NaN
            Dim rangeMsg As String

            ' The validation of the actual date value is not necessary
            ' because the input is restricted to valid values by the
            ' DateTimePicker control.
            '
            ' Convert the .NET date or datetime into as SAS Date or
            ' DateTime value.
            Select Case (_PictureType)
                Case 1 ' Date
                    doubleValue = UtilityMethods.DotNetDateTimeToSASDate(dateStartValue.Value)
                Case 3 ' DateTime
                    doubleValue = UtilityMethods.DotNetDateAndTimeToSASDateTime(dateStartValue.Value, timeStartValue.Value)
                Case Else
                    Throw New Exception("Somehow we managed to get into the dateStartValue_Validating event with a non-date picture type.")
            End Select

            ' If the Low range limit is not set and the Double value
            ' entered is less than the range limit, then we have a
            ' problem. 
            If (_RangeLimits.Low = False And doubleValue < _RangeLimits.StartValue) Then
                ' The validation has failed.
                e.Cancel = True
                ' Format the appropriate error message.
                If (_RangeLimits.High = True) Then
                    Dim valueString As String

                    If (_PictureType = 1) Then
                        valueString = UtilityMethods.SASDateToString(_RangeLimits.StartValue, False)
                    Else
                        valueString = UtilityMethods.SASDateTimeToString(_RangeLimits.StartValue, False)
                    End If
                    rangeMsg = String.Format("The value must be greater than or equal to {0}.", valueString)
                Else
                    Dim startValueString As String
                    Dim endValueString As String

                    If (_PictureType = 1) Then
                        startValueString = UtilityMethods.SASDateToString(_RangeLimits.StartValue, False)
                        endValueString = UtilityMethods.SASDateToString(_RangeLimits.EndValue, False)
                    Else
                        startValueString = UtilityMethods.SASDateTimeToString(_RangeLimits.StartValue, False)
                        endValueString = UtilityMethods.SASDateTimeToString(_RangeLimits.EndValue, False)
                    End If
                    rangeMsg = String.Format("The value must be greater than or equal to {0} and less than or equal to {1}.", startValueString, endValueString)
                End If

                MessageBox.Show(Me, "The value entered in the Start Value field is out of range." + _
                        vbCrLf + rangeMsg, "Invalid Value")
                Return
            End If

            ' If we got here, then value is acceptable.
            e.Cancel = False
        End Sub

        Private Sub dateStartValue_Validated(ByVal sender As Object, ByVal e As System.EventArgs) Handles dateStartValue.Validated
            ' We are assuming here that all the range validation was
            ' taken care of in the Validating event handler.
            '
            ' Get and store the new range start value.
            Select Case (_PictureType)
                Case 1 ' Date
                    _OutputRangeItem.StartValue = UtilityMethods.DotNetDateTimeToSASDate(dateStartValue.Value)
                Case 3 ' DateTime
                    _OutputRangeItem.StartValue = UtilityMethods.DotNetDateAndTimeToSASDateTime(dateStartValue.Value, timeStartValue.Value)
                Case Else
                    Throw New Exception("Somehow we managed to get into the dateStartValue_Validated event with a non-date picture type.")
            End Select
            ' While where at it, we'll update the numeric control
            ' with the new value as well.
            txtStartValue.Text = _OutputRangeItem.StartValue.ToString()

            ' If the new value is equal to the range limit value and
            ' the range limit does not include the start value, then
            ' we need to make sure that we don't include it either.
            If (_OutputRangeItem.StartValue = _RangeLimits.StartValue) Then
                If (_RangeLimits.IncludeStartValue = False) Then
                    If (_OutputRangeItem.IncludeStartValue = True) Then
                        chkIncludeStartValue.Checked = False
                    End If
                    chkIncludeStartValue.Enabled = False
                End If
            Else
                If (chkIncludeStartValue.Enabled = False) Then
                    chkIncludeStartValue.Enabled = True
                End If
            End If
        End Sub

        Private Sub timeStartValue_Validating(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles timeStartValue.Validating
            Dim doubleValue As Double = Double.NaN
            Dim rangeMsg As String

            ' The validation of the actual time value is not necessary
            ' because the input is restricted to valid values by the
            ' DateTimePicker control.
            '
            ' Convert the .NET time or datetime into as SAS Time or
            ' DateTime value.
            Select Case (_PictureType)
                Case 2 ' Time
                    doubleValue = UtilityMethods.DotNetDateTimeToSASTime(timeStartValue.Value)
                Case 3 ' DateTime
                    doubleValue = UtilityMethods.DotNetDateAndTimeToSASDateTime(dateStartValue.Value, timeStartValue.Value)
                Case Else
                    Throw New Exception("Somehow we managed to get into the timeStartValue_Validating event with a non-time picture type.")
            End Select

            ' If the Low range limit is not set and the Double value
            ' entered is less than the range limit, then we have a
            ' problem. 
            If (_RangeLimits.Low = False And doubleValue < _RangeLimits.StartValue) Then
                ' The validation has failed.
                e.Cancel = True
                ' Format the appropriate error message.
                If (_RangeLimits.High = True) Then
                    Dim valueString As String

                    If (_PictureType = 2) Then
                        valueString = UtilityMethods.SASTimeToString(_RangeLimits.EndValue, False)
                    Else
                        valueString = UtilityMethods.SASDateTimeToString(_RangeLimits.EndValue, False)
                    End If
                    rangeMsg = String.Format("The value must be less than or equal to {0}.", valueString)
                Else
                    Dim startValueString As String
                    Dim endValueString As String

                    If (_PictureType = 2) Then
                        startValueString = UtilityMethods.SASTimeToString(_RangeLimits.StartValue, False)
                        endValueString = UtilityMethods.SASTimeToString(_RangeLimits.EndValue, False)
                    Else
                        startValueString = UtilityMethods.SASDateTimeToString(_RangeLimits.StartValue, False)
                        endValueString = UtilityMethods.SASDateTimeToString(_RangeLimits.EndValue, False)
                    End If
                    rangeMsg = String.Format("The value must be greater than or equal to {0} and less than or equal to {1}.", startValueString, endValueString)
                End If

                MessageBox.Show(Me, "The value entered in the Start Value field is out of range." + _
                        vbCrLf + rangeMsg, "Invalid Value")
                Return
            End If

            ' If we got here, then value is acceptable.
            e.Cancel = False
        End Sub

        Private Sub timeStartValue_Validated(ByVal sender As Object, ByVal e As System.EventArgs) Handles timeStartValue.Validated
            ' We are assuming here that all the range validation was
            ' taken care of in the Validating event handler.
            '
            ' Get and store the new range start value.
            Select Case (_PictureType)
                Case 2 ' Time
                    _OutputRangeItem.StartValue = UtilityMethods.DotNetDateTimeToSASTime(timeStartValue.Value)
                Case 3 ' DateTime
                    _OutputRangeItem.StartValue = UtilityMethods.DotNetDateAndTimeToSASDateTime(dateStartValue.Value, timeStartValue.Value)
                Case Else
                    Throw New Exception("Somehow we managed to get into the timeStartValue_Validated event with a non-time picture type.")
            End Select
            ' While where at it, we'll update the numeric control
            ' with the new value as well.
            txtStartValue.Text = _OutputRangeItem.StartValue.ToString()

            ' If the new value is equal to the range limit value and
            ' the range limit does not include the start value, then
            ' we need to make sure that we don't include it either.
            If (_OutputRangeItem.StartValue = _RangeLimits.StartValue) Then
                If (_RangeLimits.IncludeStartValue = False) Then
                    If (_OutputRangeItem.IncludeStartValue = True) Then
                        chkIncludeStartValue.Checked = False
                    End If
                    chkIncludeStartValue.Enabled = False
                End If
            Else
                If (chkIncludeStartValue.Enabled = False) Then
                    chkIncludeStartValue.Enabled = True
                End If
            End If
        End Sub

        Private Sub dateEndValue_Validating(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles dateEndValue.Validating
            Dim doubleValue As Double = Double.NaN
            Dim rangeMsg As String

            ' The validation of the actual date value is not necessary
            ' because the input is restricted to valid values by the
            ' DateTimePicker control.
            '
            ' Convert the .NET date or datetime into as SAS Date or
            ' DateTime value.
            Select Case (_PictureType)
                Case 1 ' Date
                    doubleValue = UtilityMethods.DotNetDateTimeToSASDate(dateEndValue.Value)
                Case 3 ' DateTime
                    doubleValue = UtilityMethods.DotNetDateAndTimeToSASDateTime(dateEndValue.Value, timeEndValue.Value)
                Case Else
                    Throw New Exception("Somehow we managed to get into the dateEndValue_Validating event with a non-date picture type.")
            End Select

            ' If the High range limit is not set and the Double value
            ' entered is greater than the range limit, then we have a
            ' problem. 
            If (_RangeLimits.High = False And doubleValue > _RangeLimits.EndValue) Then
                ' The validation has failed.
                e.Cancel = True
                ' Format the appropriate error message.
                If (_RangeLimits.Low = True) Then
                    Dim valueString As String

                    If (_PictureType = 1) Then
                        valueString = UtilityMethods.SASDateToString(_RangeLimits.EndValue, False)
                    Else
                        valueString = UtilityMethods.SASDateTimeToString(_RangeLimits.EndValue, False)
                    End If
                    rangeMsg = String.Format("The value must be less than or equal to {0}.", valueString)
                Else
                    Dim startValueString As String
                    Dim endValueString As String

                    If (_PictureType = 1) Then
                        startValueString = UtilityMethods.SASDateToString(_RangeLimits.StartValue, False)
                        endValueString = UtilityMethods.SASDateToString(_RangeLimits.EndValue, False)
                    Else
                        startValueString = UtilityMethods.SASDateTimeToString(_RangeLimits.StartValue, False)
                        endValueString = UtilityMethods.SASDateTimeToString(_RangeLimits.EndValue, False)
                    End If
                    rangeMsg = String.Format("The value must be greater than or equal to {0} and less than or equal to {1}.", startValueString, endValueString)
                End If

                MessageBox.Show(Me, "The value entered in the End Value field is out of range." + _
                        vbCrLf + rangeMsg, "Invalid Value")
                Return
            End If

            ' If we got here, then value is acceptable.
            e.Cancel = False
        End Sub

        Private Sub dateEndValue_Validated(ByVal sender As Object, ByVal e As System.EventArgs) Handles dateEndValue.Validated
            ' We are assuming here that all the range validation was
            ' taken care of in the Validating event handler.
            '
            ' Get and store the new range end value.
            Select Case (_PictureType)
                Case 1 ' Date
                    _OutputRangeItem.EndValue = UtilityMethods.DotNetDateTimeToSASDate(dateEndValue.Value)
                Case 3 ' Time
                    _OutputRangeItem.EndValue = UtilityMethods.DotNetDateAndTimeToSASDateTime(dateEndValue.Value, timeEndValue.Value)
                Case Else
                    Throw New Exception("Somehow we managed to get into the dateEndValue_Validated event with a non-date picture type.")
            End Select
            ' While where at it, we'll update the numeric control
            ' with the new value as well.
            txtEndValue.Text = _OutputRangeItem.EndValue.ToString()

            ' If the new value is equal to the range limit value and
            ' the range limit does not include the end value, then
            ' we need to make sure that we don't include it either.
            If (_OutputRangeItem.EndValue = _RangeLimits.EndValue) Then
                If (_RangeLimits.IncludeEndValue = False) Then
                    If (_OutputRangeItem.IncludeEndValue = True) Then
                        chkIncludeEndValue.Checked = False
                    End If
                    chkIncludeEndValue.Enabled = False
                End If
            Else
                If (chkIncludeEndValue.Enabled = False) Then
                    chkIncludeEndValue.Enabled = True
                End If
            End If
        End Sub

        Private Sub timeEndValue_Validating(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles timeEndValue.Validating
            Dim doubleValue As Double = Double.NaN
            Dim rangeMsg As String

            ' The validation of the actual time value is not necessary
            ' because the input is restricted to valid values by the
            ' DateTimePicker control.
            '
            ' Convert the .NET time or datetime into as SAS Time or
            ' DateTime value.
            Select Case (_PictureType)
                Case 2 ' Time
                    doubleValue = UtilityMethods.DotNetDateTimeToSASTime(timeEndValue.Value)
                Case 3 ' DateTime
                    doubleValue = UtilityMethods.DotNetDateAndTimeToSASDateTime(timeEndValue.Value, timeStartValue.Value)
                Case Else
                    Throw New Exception("Somehow we managed to get into the dateEndValue_Validating event with a non-date picture type.")
            End Select

            ' If the High range limit is not set and the Double value
            ' entered is greater than the range limit, then we have a
            ' problem. 
            If (_RangeLimits.High = False And doubleValue > _RangeLimits.EndValue) Then
                ' The validation has failed.
                e.Cancel = True
                ' Format the appropriate error message.
                If (_RangeLimits.Low = True) Then
                    Dim valueString As String

                    If (_PictureType = 2) Then
                        valueString = UtilityMethods.SASTimeToString(_RangeLimits.StartValue, False)
                    Else
                        valueString = UtilityMethods.SASDateTimeToString(_RangeLimits.StartValue, False)
                    End If
                    rangeMsg = String.Format("The value must be less than or equal to {0}.", valueString)
                Else
                    Dim startValueString As String
                    Dim endValueString As String

                    If (_PictureType = 1) Then
                        startValueString = UtilityMethods.SASTimeToString(_RangeLimits.StartValue, False)
                        endValueString = UtilityMethods.SASTimeToString(_RangeLimits.EndValue, False)
                    Else
                        startValueString = UtilityMethods.SASDateTimeToString(_RangeLimits.StartValue, False)
                        endValueString = UtilityMethods.SASDateTimeToString(_RangeLimits.EndValue, False)
                    End If
                    rangeMsg = String.Format("The value must be greater than or equal to {0} and less than or equal to {1}.", startValueString, endValueString)
                End If

                MessageBox.Show(Me, "The value entered in the End Value field is out of range." + _
                        vbCrLf + rangeMsg, "Invalid Value")
                Return
            End If

            ' If we got here, then value is acceptable.
            e.Cancel = False
        End Sub

        Private Sub timeEndValue_Validated(ByVal sender As Object, ByVal e As System.EventArgs) Handles timeEndValue.Validated
            ' We are assuming here that all the range validation was
            ' taken care of in the Validating event handler.
            '
            ' Get and store the new range end value.
            Select Case (_PictureType)
                Case 2 ' Time
                    _OutputRangeItem.EndValue = UtilityMethods.DotNetDateTimeToSASTime(timeEndValue.Value)
                Case 3 ' DateTime
                    _OutputRangeItem.EndValue = UtilityMethods.DotNetDateAndTimeToSASDateTime(dateEndValue.Value, timeEndValue.Value)
                Case Else
                    Throw New Exception("Somehow we managed to get into the timeEndValue_Validated event with a non-date picture type.")
            End Select
            ' While where at it, we'll update the numeric control
            ' with the new value as well.
            txtEndValue.Text = _OutputRangeItem.EndValue.ToString()

            ' If the new value is equal to the range limit value and
            ' the range limit does not include the end value, then
            ' we need to make sure that we don't include it either.
            If (_OutputRangeItem.EndValue = _RangeLimits.EndValue) Then
                If (_RangeLimits.IncludeEndValue = False) Then
                    If (_OutputRangeItem.IncludeEndValue = True) Then
                        chkIncludeEndValue.Checked = False
                    End If
                    chkIncludeEndValue.Enabled = False
                End If
            Else
                If (chkIncludeEndValue.Enabled = False) Then
                    chkIncludeEndValue.Enabled = True
                End If
            End If
        End Sub

        Private Sub txtMaskString_Validated(ByVal sender As Object, ByVal e As System.EventArgs) Handles txtMaskString.Validated
            ' Store the picture mask string.
            _OutputRangeItem.Mask = txtMaskString.Text
        End Sub

        Private Sub chkNoEdit_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles chkNoEdit.CheckedChanged
            ' Store the NOEDIT flag.
            _OutputRangeItem.NoEdit = chkNoEdit.Checked
            ' If the NOEDIT flag is set, then we want to disable the
            ' button that allows the insertion of Directive codes.
            btnMaskStringInsertCode.Enabled = (chkNoEdit.Checked = False)
        End Sub

        Private Sub chkUsePrefix_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles chkUsePrefix.CheckedChanged
            ' Store the Use Prefix flag.
            _OutputRangeItem.UsePrefix = chkUsePrefix.Checked
        End Sub

        Private Sub txtPrefix_Validated(ByVal sender As Object, ByVal e As System.EventArgs) Handles txtPrefix.Validated
            ' Store the new prefix string.
            _OutputRangeItem.Prefix = txtPrefix.Text
        End Sub

        Private Sub chkUseFillChar_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles chkUseFillChar.CheckedChanged
            ' Store the Use Fill Char flag.
            _OutputRangeItem.UseFillChar = chkUseFillChar.Checked
        End Sub

        Private Sub txtFillChar_Validated(ByVal sender As Object, ByVal e As System.EventArgs) Handles txtFillChar.Validated
            Dim fillChar As Char()
            ' Store the fill character.
            fillChar = txtFillChar.Text.ToCharArray()
            _OutputRangeItem.FillChar = fillChar(0)
        End Sub

        Private Sub chkUseMultiplier_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles chkUseMultiplier.CheckedChanged
            ' Store the Use Multiplier flag.
            _OutputRangeItem.UseMultiplier = chkUseMultiplier.Checked
        End Sub

        Private Sub txtMultiplier_Validating(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles txtMultiplier.Validating
            Dim doubleValue As Double
            Dim messageString As String

            ' It's time to validate the value entered in the
            ' Multiplier textbox.
            If (txtMultiplier.Text.Trim().Length > 0) Then
                ' The textbox isn't empty, so let's try to get a
                ' number from it. 
                If (Double.TryParse(txtMultiplier.Text, Globalization.NumberStyles.Float, NumberFormatInfo.InvariantInfo, doubleValue) = True) Then
                    If (doubleValue <> 0) Then
                        ' A number was succesfully parsed from
                        ' the textbox.
                        e.Cancel = False
                    Else
                        ' The number cannot be 0, so validation
                        ' has failed.
                        e.Cancel = True
                        messageString = "The value entered in the Multiplier field is zero, which is invalid ."
                    End If
                Else
                    ' We couldn't parse an acceptable number from
                    ' the textbox.
                    e.Cancel = True
                    messageString = "The value entered in the Multiplier field is invalid."
                End If
            Else
                ' The text box was empty, which is unacceptable.
                e.Cancel = True
                messageString = "There is no value entered in the Multiplier field."
            End If

            ' If the validation failed, then we want to let the
            ' user know about it.
            If (e.Cancel = True) Then
                MessageBox.Show(Me, messageString + vbCrLf + _
                        "The value  must consist of a single non-zero integer, floating point," + vbCrLf + _
                        "or scientific notation value. Entering a value of 1 will display the" + vbCrLf + _
                        "formatted value with the decimal position unaltered.", _
                        "Invalid Numeric Value")
            End If
        End Sub

        Private Sub txtMultiplier_Validated(ByVal sender As Object, ByVal e As System.EventArgs) Handles txtMultiplier.Validated
            Dim doubleValue As Double

            ' We are assuming here that the validation of the value has
            ' been performed in the Validating event handler.
            '
            ' Parse the number from the textbox.
            If (Double.TryParse(txtMultiplier.Text, Globalization.NumberStyles.Float, NumberFormatInfo.InvariantInfo, doubleValue) = False) Then
                Throw New Exception("Somehow, we got to the Validated event handler for txtMultiplier with an invalid double string. This would be one of those internal errors!")
            End If

            ' Store the multiplier value.
            _OutputRangeItem.Multiplier = doubleValue
        End Sub

        Private Sub btnAdd_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnAdd.Click
            ' Before we can dismiss the dialog, we need to ensure
            ' that if the picture mask field is blank, then it has
            ' been done on purpose.
            If (_OutputRangeItem.IsUndefinedRange = True) Then
                If (MessageBox.Show(Me, "This range cannot be added to the format because the Mask field is blank" + vbCrLf + _
                        "without the option to treat the Mask value as a text string selected." + vbCrLf + _
                        "Do you want to the empty Mask value to be treated as a text string?", _
                        "Mask Settings Conflict", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) = DialogResult.Yes) Then
                    ' The user does want a blank mask and so we need
                    ' to set the NOEDIT flag for the range.
                    chkNoEdit.Checked = True
                Else
                    ' The user just forgot to enter the picture mask
                    ' string, so we abort the dismissing of the dialog
                    ' and set focus to the picture mask textbox.
                    txtMaskString.Focus()
                    Return
                End If
            End If

            ' Time for the last bit of validation. At this point we
            ' want to check that the range does not consist of a
            ' single value which has been excluded from the range
            ' (in effect creating an empty range).
            If (_OutputRangeItem.Low = False And _OutputRangeItem.High = False And _
                _OutputRangeItem.StartValue = _OutputRangeItem.EndValue And _
                _OutputRangeItem.IncludeStartValue = False And _
                _OutputRangeItem.IncludeEndValue = False) Then
                MessageBox.Show(Me, "Because the Start Value and End Values are the same and both of the values have" + vbCrLf + _
                    "been excluded from the range, the range, in effect, contains no values." + vbCrLf + _
                    "Either modify the range or press the Cancel button to dismiss the dialog.", _
                    "Empty Range", MessageBoxButtons.OK, MessageBoxIcon.Warning)
                ' Set focus to the appropriate start control for the
                ' picture type.
                Select Case (_PictureType)
                    Case 0 ' Numeric
                        txtStartValue.Focus()
                    Case 1 Or 3 ' Date or DateTime
                        dateStartValue.Focus()
                    Case 2 ' Time
                        timeStartValue.Focus()
                End Select
                Return
            End If

            ' Everything seems to be in order with the range, so it's
            ' time to dismiss the dialog signalling our success.
            Me.DialogResult = DialogResult.OK
        End Sub

        Private Sub btnMaskStringHelp_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnMaskStringHelp.Click
            Dim dlg As RangesHelpForm = New RangesHelpForm

            ' Show the simple picture format mask help dialog.
            dlg.ShowDialog(Me)
        End Sub

        Private Sub btnMaskStringInsertCode_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnMaskStringInsertCode.Click
            ' Show the Directives context menu.
            mnuMaskStringPopup.Show(Me, New Point(btnMaskStringInsertCode.Left + btnMaskStringInsertCode.Width, btnMaskStringInsertCode.Top))
        End Sub
#End Region

#Region " Private Methods "
        Private Sub ShowOptions(ByVal Show As Boolean)
            ' Show or hide the Options groupbox (and it's contents)
            ' depending on the supplied flag.
            grpOptions.Visible = Show
            ' Store the flag so that we can toggle it next time the
            ' Show/Hide button is pressed.
            _OptionsAreShown = Show

            ' Now perform the calculations needed to resize the
            ' dialog window to expand to show the options, or collapse
            ' to hide the space where the options were.
            If (Show = True) Then
                ' Explaination of the following calculation:
                ' (Me.Height - Me.ClientSize.Height) gives us the height
                '     of the dialog's titlebar and window frames.
                ' (Me.ClientSize.Height - btnAdd.Top) gives us the distance
                '     from the top of the "Add" button to the bottom of
                '     the client area.
                ' So putting it all together, if we are to display the
                ' Options, the height of the window needs to be the 
                ' height of the title bar and window frames plus the
                ' distance from the Add button to the bottom of the client
                ' area plus the distance from the top of the client area
                ' to the bottom of the Options groupbox plus a gap of 10
                ' pixels between the Options groupbox and the Add button.
                Me.Height = (Me.Height - Me.ClientSize.Height) + (Me.ClientSize.Height - btnAdd.Top) + grpOptions.Top + grpOptions.Height + 10
                btnShowOptions.Text = "<<< Hide Options"
                ' Make sure that we haven't dropped off the bottom of the screen
                If (Me.Top + Me.Height > SystemInformation.WorkingArea.Bottom) Then
                    ' It appears that we have, so we need to reposition
                    ' the window so that the bottom is visible.
                    Me.Top = SystemInformation.WorkingArea.Bottom - Me.Height
                End If
            Else
                ' Using the description above for a description of the 
                ' first two parts of the following calculation, if we
                ' are to collapse the window to hide the options, the
                ' calculation can be described as:
                ' The height of the window needs to be the height of 
                ' the title bar and window frames plus the distance from
                ' the Add button to the bottom of the client area plus
                ' the distance from the top of the client area to the
                ' bottom of the range groupbox, plus a gap of 10 pixels
                ' between the groupbox and the Add button.
                Me.Height = (Me.Height - Me.ClientSize.Height) + (Me.ClientSize.Height - btnAdd.Top) + grpRange.Top + grpRange.Height + 10
                btnShowOptions.Text = "Show Options >>>"
            End If
        End Sub

        Private Sub UpdateControlValues()
            ' We need to set the input control values from
            ' the current range item's property values.
            If (_OutputRangeItem.Low = True) Then
                optLow.Select()
            Else
                optUseStartValue.Select()
            End If
            txtStartValue.Text = _OutputRangeItem.StartValue.ToString()
            dateStartValue.Value = UtilityMethods.SASDateTimeToDotNetDateTime(_OutputRangeItem.StartValue)
            timeStartValue.Value = dateStartValue.Value
            chkIncludeStartValue.Checked = _OutputRangeItem.IncludeStartValue

            If (_OutputRangeItem.High = True) Then
                optHigh.Select()
            Else
                optUseEndValue.Select()
            End If
            txtEndValue.Text = _OutputRangeItem.EndValue.ToString()
            dateEndValue.Value = UtilityMethods.SASDateTimeToDotNetDateTime(_OutputRangeItem.EndValue)
            timeEndValue.Value = dateEndValue.Value
            chkIncludeEndValue.Checked = _OutputRangeItem.IncludeEndValue

            txtMaskString.Text = _OutputRangeItem.Mask
            chkNoEdit.Checked = _OutputRangeItem.NoEdit

            chkUsePrefix.Checked = _OutputRangeItem.UsePrefix
            txtPrefix.Text = _OutputRangeItem.Prefix

            chkUseFillChar.Checked = _OutputRangeItem.UseFillChar
            txtFillChar.Text = _OutputRangeItem.FillChar.ToString().Trim()

            chkUseMultiplier.Checked = _OutputRangeItem.UseMultiplier
            txtMultiplier.Text = _OutputRangeItem.Multiplier.ToString()
        End Sub
#End Region

#Region " Overriden Methods "
        Protected Overrides Sub OnActivated(ByVal e As System.EventArgs)
            ' For some reason we need to set focus to the Mask field
            ' here, otherwise, focus keeps going to the High option
            ' button when the form first appears.
            txtMaskString.Focus()
        End Sub
#End Region

#Region "Context Menu Item Event Handlers"
        Private Sub mitmLeadingSpace_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mitmLeadingSpace.Click
            ' Inserts the mask character for showing a space instead
            ' of a zero in place of the currently selected text in 
            ' the picture mask textbox. Focus is then set back to
            ' the textbox.
            txtMaskString.SelectedText = "0"
            txtMaskString.Focus()
        End Sub

        Private Sub mitmLeadingZero_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mitmLeadingZero.Click
            ' Inserts the mask character for showing a zero wherever
            ' a digit would normally not be shown, in place of the
            ' currently selected text in the picture mask textbox.
            ' Focus is then set back to the textbox.
            txtMaskString.SelectedText = "9"
            txtMaskString.Focus()
        End Sub

        Private Sub mitmAbbrvWkDayName_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mitmAbbrvWkDayName.Click
            ' Inserts the mask directive for showing a locale
            ' specific abbreviated week day name, in place of the
            ' currently selected text in the picture mask textbox.
            ' Focus is then set back to the textbox.
            txtMaskString.SelectedText = "%a"
            txtMaskString.Focus()
        End Sub

        Private Sub mitmFullWkDayName_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mitmFullWkDayName.Click
            ' Inserts the mask directive for showing a locale
            ' specific full week day name, in place of the
            ' currently selected text in the picture mask textbox.
            ' Focus is then set back to the textbox.
            txtMaskString.SelectedText = "%A"
            txtMaskString.Focus()
        End Sub

        Private Sub mitmAbbrevMonthName_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mitmAbbrevMonthName.Click
            ' Inserts the mask directive for showing a locale
            ' specific abbreviated month name, in place of the
            ' currently selected text in the picture mask textbox.
            ' Focus is then set back to the textbox.
            txtMaskString.SelectedText = "%b"
            txtMaskString.Focus()
        End Sub

        Private Sub mitmFullMonthName_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mitmFullMonthName.Click
            ' Inserts the mask directive for showing a locale
            ' specific full month name, in place of the currently
            ' selected text in the picture mask textbox. Focus
            ' is then set back to the textbox.
            txtMaskString.SelectedText = "%B"
            txtMaskString.Focus()
        End Sub

        Private Sub mitmYearWOCentury_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mitmYearWOCentury.Click
            ' Inserts the mask directive for showing the year
            ' number, without the century digits, in place of
            ' the currently selected text in the picture mask
            ' textbox. Focus is then set back to the textbox.
            txtMaskString.SelectedText = "%y"
            txtMaskString.Focus()
        End Sub

        Private Sub mitmYearWCentury_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mitmYearWCentury.Click
            ' Inserts the mask directive for showing the year
            ' number, including the century digits, in place of
            ' the currently selected text in the picture mask
            ' textbox. Focus is then set back to the textbox.
            txtMaskString.SelectedText = "%Y"
            txtMaskString.Focus()
        End Sub

        Private Sub mitmMonth_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mitmMonth.Click
            ' Inserts the mask directive for showing a month
            ' number in place of the currently selected text
            ' in the picture mask textbox. Focus is then set
            ' back to the textbox.
            txtMaskString.SelectedText = "%m"
            txtMaskString.Focus()
        End Sub

        Private Sub mitmDayOfMonth_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mitmDayOfMonth.Click
            ' Inserts the mask directive for showing the day
            ' of the month in place of the currently selected text
            ' in the picture mask textbox. Focus is then set
            ' back to the textbox.
            txtMaskString.SelectedText = "%d"
            txtMaskString.Focus()
        End Sub

        Private Sub mitmDayOfYear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mitmDayOfYear.Click
            ' Inserts the mask directive for showing the day
            ' of the year in place of the currently selected text
            ' in the picture mask text box. Focus is then set
            ' back to the textbox.
            txtMaskString.SelectedText = "%j"
            txtMaskString.Focus()
        End Sub

        Private Sub mitmWeekday_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mitmWeekday.Click
            ' Inserts the mask directive for showing a locale
            ' specific full week day name, in place of the
            ' currently selected text in the picture mask
            ' textbox. Focus is then set back to the textbox.
            txtMaskString.SelectedText = "%w"
            txtMaskString.Focus()
        End Sub

        Private Sub mitmWeekNumber_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mitmWeekNumber.Click
            ' Inserts the mask directive for showing the
            ' week number in place of the currently
            ' selected text in the picture mask textbox.
            ' Focus is then set back to the textbox.
            txtMaskString.SelectedText = "%U"
            txtMaskString.Focus()
        End Sub

        Private Sub mitmHour24_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mitmHour24.Click
            ' Inserts the mask directive for showing the
            ' hour number in 24-hour format in place of
            ' the currently selected text in the picture
            ' mask textbox. Focus is then set back to the
            ' textbox.
            txtMaskString.SelectedText = "%H"
            txtMaskString.Focus()
        End Sub

        Private Sub mitmHour12_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mitmHour12.Click
            ' Inserts the mask directive for showing the
            ' hour number in 12-hour format in place of
            ' the currently selected text in the picture
            ' mask textbox. Focus is then set back to the
            ' textbox.
            txtMaskString.SelectedText = "I"
            txtMaskString.Focus()
        End Sub

        Private Sub mitmMinute_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mitmMinute.Click
            ' Inserts the mask directive for showing the
            ' minutes number in place of the currently
            ' selected text in the picture mask textbox.
            ' Focus is then set back to the textbox.
            txtMaskString.SelectedText = "%M"
            txtMaskString.Focus()
        End Sub

        Private Sub mitmSecond_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mitmSecond.Click
            ' Inserts the mask directive for showing the
            ' seconds number in place of the currently
            ' selected text in the picture mask textbox.
            ' Focus is then set back to the textbox.
            txtMaskString.SelectedText = "%S"
            txtMaskString.Focus()
        End Sub

        Private Sub mitmAMPM_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mitmAMPM.Click
            ' Inserts the mask directive for showing the
            ' locale speific AM/PM indicator in place of
            ' the currently selected text in the picture
            ' mask textbox. Focus is then set back to the
            ' textbox.
            txtMaskString.SelectedText = "%p"
            txtMaskString.Focus()
        End Sub

        Private Sub mitmPercent_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mitmPercent.Click
            ' Inserts the mask directive for showing a
            ' percent (%) sign in place of the currently
            ' selected text in the picture mask textbox.
            ' Focus is then set back to the textbox.
            txtMaskString.SelectedText = "%%"
            txtMaskString.Focus()
        End Sub
#End Region
    End Class
End Namespace

