' =========================================================
'   
'                Create Picture Format Task
'
'                          a SAS Custom Task Example 
'
' Copyright (C) 2006 by SAS Institute Inc., Cary, NC 27513
'
' ---------------------------------------------------------
' The TestRanges from allows the user to specify values
' that are passed through a temporary version of the
' PICTURE format (as it is currently defined) and shows the
' resulting formatted value. This allows the format to be
' tested before the task is run.
' =========================================================

Imports System.Collections
Imports System.Drawing
Imports System.Globalization
Imports System.Windows.Forms

Namespace SASVB.CustomTasks
    Public Class TestRanges
        Inherits System.Windows.Forms.Form

#Region " Member variables "
        ' The values for these member variables are provided by the
        ' parent prior to the loading of the form via the associated
        ' properties. 
        Private _consumer As SAS.Shared.AddIns.ISASTaskConsumer = Nothing
        Private _serverName As String = String.Empty
        Private _pictureType As Int32 = 0
        Private _defaultWidth As Int32 = 12
        Private _formatCode As String = String.Empty

        ' This flag indicates whether the temporary test format has
        ' been defined on the server yet.
        Private tempFormatCreated As Boolean = False
#End Region

#Region " Properties "
        Public WriteOnly Property Consumer() As SAS.Shared.AddIns.ISASTaskConsumer
            Set(ByVal Value As SAS.Shared.AddIns.ISASTaskConsumer)
                ' Store the consumer object for use when we need to
                ' create the temporary test format, or format one of
                ' the test values.
                _consumer = Value
            End Set
        End Property

        Public WriteOnly Property ServerName() As String
            Set(ByVal Value As String)
                ' Store the consumer object for use when we need to
                ' create the temporary test format, or format one of
                ' the test values.
                _serverName = Value
            End Set
        End Property

        Public WriteOnly Property PictureType() As Int32
            Set(ByVal Value As Int32)
                ' Start by checking that the new picture type value is
                ' valid.
                If (Value < 0 Or Value > 3) Then
                    Throw New ArgumentException("The value of PictureType must be 0, 1, 2 or 3.", "PictureType")
                End If

                ' Store the new value for use when we add new test values
                ' to the list of test values.
                _pictureType = Value

                ' Enable/diable the value entry controls based on the
                ' picture format type.
                txtTestValue.Visible = (_pictureType = 0)
                dateTestValue.Visible = (_pictureType = 1 Or _pictureType = 3)
                timeTestValue.Visible = (_pictureType = 2 Or _pictureType = 3)

                ' Position the date and time controls based on the picture
                ' format type.
                If (_pictureType = 1 Or _pictureType = 3) Then
                    dateTestValue.Location = New Point(txtTestValue.Left, txtTestValue.Top)
                End If
                If (_pictureType = 2) Then
                    timeTestValue.Location = New Point(txtTestValue.Left, txtTestValue.Top)
                Else
                    If (_pictureType = 3) Then
                        timeTestValue.Location = New Point(dateTestValue.Left + dateTestValue.Width + 8, txtTestValue.Top)
                    End If
                End If
            End Set
        End Property

        Public WriteOnly Property FormatCode() As String
            Set(ByVal Value As String)
                ' Save the SAS format code until it's time to define
                ' the temporary format on the server.
                _formatCode = Value
            End Set
        End Property

        Public WriteOnly Property DefaultWidth() As Int32
            Set(ByVal Value As Int32)
                ' Check that the new default width value is valid.
                If (Value < 1 Or Value > 40) Then
                    Throw New ArgumentException("The value for DefaultWidth must be between 1 and 40 (inclusive).", "DefaultWidth")
                End If
                ' Store it away until we're ready to format values.
                _defaultWidth = Value
            End Set
        End Property

        Public Property TestValues() As Double()
            Get
                ' Create a new Double array for the new test values
                ' to be returned in.
                Dim workDouble(lvwTestValues.Items.Count - 1) As Double

                ' Load up each of the test Double values into the array.
                For itemIdx As Int32 = 0 To lvwTestValues.Items.Count - 1
                    workDouble(itemIdx) = CType(lvwTestValues.Items(itemIdx).Tag, Double)
                Next

                ' Return the test Double array. 
                Return workDouble
            End Get

            Set(ByVal Value As Double())
                ' Start by ensuring that there are no old values left
                ' in the list view.
                For Each lvi As ListViewItem In lvwTestValues.Items
                    lvi.Tag = Nothing
                Next
                lvwTestValues.Items.Clear()

                ' Loop through each of the Double values in the array of
                ' new test values.
                For valueIdx As Int32 = 0 To Value.GetLength(0) - 1
                    Dim lvi As ListViewItem = New ListViewItem

                    ' Format the first subitem for the list item based
                    ' on the current picture type.
                    Select Case (_pictureType)
                        Case 0 ' Numeric
                            lvi.SubItems(0).Text = Value(valueIdx).ToString()
                        Case 1 ' Date
                            lvi.SubItems(0).Text = UtilityMethods.SASDateToString(Value(valueIdx), False)
                        Case 2 ' Time
                            lvi.SubItems(0).Text = UtilityMethods.SASTimeToString(Value(valueIdx), False)
                        Case 3 ' DateTime
                            lvi.SubItems(0).Text = UtilityMethods.SASDateTimeToString(Value(valueIdx), False)
                    End Select
                    ' For the second subitem, get the formatted string
                    ' by passing the Double value through the test
                    ' version of the format on the server.
                    lvi.SubItems.Add(GetFormattedValue(Value(valueIdx)))
                    ' Store the test value in the Tag property of the 
                    ' list view item.
                    lvi.Tag = Value(valueIdx)

                    ' Add the list view item to the list view control.
                    lvwTestValues.Items.Add(lvi)
                Next

                ' Make sure that the first item in the list view is selected.
                If (lvwTestValues.Items.Count > 0) Then
                    lvwTestValues.Items(0).Selected = True
                End If
            End Set
        End Property
#End Region

#Region " Windows Form Designer generated code "

        Public Sub New()
            MyBase.New()

            'This call is required by the Windows Form Designer.
            InitializeComponent()

            ' Load the icon for the form's control menu on the title bar.
            Me.Icon = New Icon(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream("SASVB.CustomTasks.TestRanges.ico"))

            ' Initialise the date and times displayed within the date
            ' and time selection controls to the current date and time.
            dateTestValue.Value = UtilityMethods.DotNetDateTimeDropMillisecs(DateTime.Now)
            timeTestValue.Value = dateTestValue.Value
        End Sub

        'Form overrides dispose to clean up the component list.
        Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
            If disposing Then
                If Not (components Is Nothing) Then
                    components.Dispose()
                End If
            End If
            MyBase.Dispose(disposing)
        End Sub

        'Required by the Windows Form Designer
        Private components As System.ComponentModel.IContainer

        'NOTE: The following procedure is required by the Windows Form Designer
        'It can be modified using the Windows Form Designer.  
        'Do not modify it using the code editor.
        Friend WithEvents lvwTestValues As System.Windows.Forms.ListView
        Friend WithEvents colHeader1 As System.Windows.Forms.ColumnHeader
        Friend WithEvents colHeader2 As System.Windows.Forms.ColumnHeader
        Friend WithEvents lblTestValue As System.Windows.Forms.Label
        Friend WithEvents txtTestValue As System.Windows.Forms.TextBox
        Friend WithEvents dateTestValue As System.Windows.Forms.DateTimePicker
        Friend WithEvents btnClose As System.Windows.Forms.Button
        Friend WithEvents timeTestValue As System.Windows.Forms.DateTimePicker
        Friend WithEvents btnAdd As System.Windows.Forms.Button
        Friend WithEvents btnModify As System.Windows.Forms.Button
        Friend WithEvents btnRemove As System.Windows.Forms.Button
        Friend WithEvents lblInfoText As System.Windows.Forms.Label
        <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
            Me.lvwTestValues = New System.Windows.Forms.ListView
            Me.colHeader1 = New System.Windows.Forms.ColumnHeader
            Me.colHeader2 = New System.Windows.Forms.ColumnHeader
            Me.lblTestValue = New System.Windows.Forms.Label
            Me.txtTestValue = New System.Windows.Forms.TextBox
            Me.dateTestValue = New System.Windows.Forms.DateTimePicker
            Me.btnAdd = New System.Windows.Forms.Button
            Me.btnModify = New System.Windows.Forms.Button
            Me.btnRemove = New System.Windows.Forms.Button
            Me.btnClose = New System.Windows.Forms.Button
            Me.timeTestValue = New System.Windows.Forms.DateTimePicker
            Me.lblInfoText = New System.Windows.Forms.Label
            Me.SuspendLayout()
            '
            'lvwTestValues
            '
            Me.lvwTestValues.AccessibleName = "Test values list"
            Me.lvwTestValues.Anchor = CType((((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Bottom) _
                        Or System.Windows.Forms.AnchorStyles.Left) _
                        Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.lvwTestValues.Columns.AddRange(New System.Windows.Forms.ColumnHeader() {Me.colHeader1, Me.colHeader2})
            Me.lvwTestValues.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable
            Me.lvwTestValues.HideSelection = False
            Me.lvwTestValues.Location = New System.Drawing.Point(8, 88)
            Me.lvwTestValues.MultiSelect = False
            Me.lvwTestValues.Name = "lvwTestValues"
            Me.lvwTestValues.Size = New System.Drawing.Size(344, 136)
            Me.lvwTestValues.TabIndex = 0
            Me.lvwTestValues.View = System.Windows.Forms.View.Details
            '
            'colHeader1
            '
            Me.colHeader1.Text = "Test Value"
            Me.colHeader1.Width = 160
            '
            'colHeader2
            '
            Me.colHeader2.Text = "Formatted Value"
            Me.colHeader2.Width = 160
            '
            'lblTestValue
            '
            Me.lblTestValue.Anchor = CType((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Left), System.Windows.Forms.AnchorStyles)
            Me.lblTestValue.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.lblTestValue.Location = New System.Drawing.Point(8, 240)
            Me.lblTestValue.Name = "lblTestValue"
            Me.lblTestValue.Size = New System.Drawing.Size(96, 16)
            Me.lblTestValue.TabIndex = 1
            Me.lblTestValue.Text = "Test &value:"
            '
            'txtTestValue
            '
            Me.txtTestValue.Anchor = CType((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Left), System.Windows.Forms.AnchorStyles)
            Me.txtTestValue.Location = New System.Drawing.Point(104, 238)
            Me.txtTestValue.Name = "txtTestValue"
            Me.txtTestValue.Size = New System.Drawing.Size(144, 20)
            Me.txtTestValue.TabIndex = 2
            Me.txtTestValue.Text = "0"
            '
            'dateTestValue
            '
            Me.dateTestValue.Anchor = CType((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Left), System.Windows.Forms.AnchorStyles)
            Me.dateTestValue.CustomFormat = ""
            Me.dateTestValue.Format = System.Windows.Forms.DateTimePickerFormat.Short
            Me.dateTestValue.Location = New System.Drawing.Point(40, 296)
            Me.dateTestValue.Name = "dateTestValue"
            Me.dateTestValue.Size = New System.Drawing.Size(112, 20)
            Me.dateTestValue.TabIndex = 3
            Me.dateTestValue.Visible = False
            '
            'btnAdd
            '
            Me.btnAdd.Anchor = CType((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Left), System.Windows.Forms.AnchorStyles)
            Me.btnAdd.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.btnAdd.Location = New System.Drawing.Point(88, 264)
            Me.btnAdd.Name = "btnAdd"
            Me.btnAdd.Size = New System.Drawing.Size(56, 24)
            Me.btnAdd.TabIndex = 5
            Me.btnAdd.Text = "&Add"
            '
            'btnModify
            '
            Me.btnModify.Anchor = CType((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Left), System.Windows.Forms.AnchorStyles)
            Me.btnModify.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.btnModify.Location = New System.Drawing.Point(152, 264)
            Me.btnModify.Name = "btnModify"
            Me.btnModify.Size = New System.Drawing.Size(56, 24)
            Me.btnModify.TabIndex = 6
            Me.btnModify.Text = "&Modify"
            '
            'btnRemove
            '
            Me.btnRemove.Anchor = CType((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Left), System.Windows.Forms.AnchorStyles)
            Me.btnRemove.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.btnRemove.Location = New System.Drawing.Point(216, 264)
            Me.btnRemove.Name = "btnRemove"
            Me.btnRemove.Size = New System.Drawing.Size(56, 24)
            Me.btnRemove.TabIndex = 7
            Me.btnRemove.Text = "&Remove"
            '
            'btnClose
            '
            Me.btnClose.Anchor = CType((System.Windows.Forms.AnchorStyles.Bottom Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
            Me.btnClose.CausesValidation = False
            Me.btnClose.DialogResult = System.Windows.Forms.DialogResult.OK
            Me.btnClose.FlatStyle = System.Windows.Forms.FlatStyle.System
            Me.btnClose.Location = New System.Drawing.Point(288, 296)
            Me.btnClose.Name = "btnClose"
            Me.btnClose.Size = New System.Drawing.Size(64, 24)
            Me.btnClose.TabIndex = 8
            Me.btnClose.Text = "&Close"
            '
            'timeTestValue
            '
            Me.timeTestValue.Format = System.Windows.Forms.DateTimePickerFormat.Time
            Me.timeTestValue.Location = New System.Drawing.Point(160, 296)
            Me.timeTestValue.Name = "timeTestValue"
            Me.timeTestValue.ShowUpDown = True
            Me.timeTestValue.Size = New System.Drawing.Size(104, 20)
            Me.timeTestValue.TabIndex = 4
            '
            'lblInfoText
            '
            Me.lblInfoText.AccessibleName = "Information"
            Me.lblInfoText.Location = New System.Drawing.Point(8, 0)
            Me.lblInfoText.Name = "lblInfoText"
            Me.lblInfoText.Size = New System.Drawing.Size(344, 88)
            Me.lblInfoText.TabIndex = 9
            Me.lblInfoText.Text = "Entering values in this dialog will cause them to be passed to the SAS server to " & _
            "be formatted using the format as it is currently defined. The formatted value re" & _
            "turned from the server will be displayed in the second column, beside the origin" & _
            "al value, within the list."
            Me.lblInfoText.TextAlign = System.Drawing.ContentAlignment.MiddleLeft
            '
            'TestRanges
            '
            Me.AutoScaleBaseSize = New System.Drawing.Size(5, 13)
            Me.CancelButton = Me.btnClose
            Me.ClientSize = New System.Drawing.Size(360, 326)
            Me.Controls.Add(Me.lblInfoText)
            Me.Controls.Add(Me.timeTestValue)
            Me.Controls.Add(Me.btnClose)
            Me.Controls.Add(Me.btnAdd)
            Me.Controls.Add(Me.dateTestValue)
            Me.Controls.Add(Me.txtTestValue)
            Me.Controls.Add(Me.lblTestValue)
            Me.Controls.Add(Me.lvwTestValues)
            Me.Controls.Add(Me.btnModify)
            Me.Controls.Add(Me.btnRemove)
            Me.Name = "TestRanges"
            Me.ShowInTaskbar = False
            Me.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent
            Me.Text = "Test Current Range Definition"
            Me.ResumeLayout(False)

        End Sub

#End Region

#Region " Overrides "
        Protected Overrides Sub OnLoad(ByVal e As System.EventArgs)
            ' We can't do anything if we haven't got the consumer,
            ' server name or format code. 
            If (_consumer Is Nothing) Then
                Throw New Exception("Expected to have a valid ISASTaskConsumer at this point.")
            End If
            If (_serverName.Length = 0) Then
                Throw New Exception("Expected to have a valid SAS server name at this point.")
            End If
            If (_formatCode.Length = 0) Then
                Throw New Exception("Expected to have some PROC FORMAT range code at this point.")
            End If

            ' We appear to have enough to proceed, so we need to send
            ' the code off to the server so as to generate the 
            ' temporary test format.
            _consumer.Submit.SubmitCode(String.Format("PROC FORMAT;{0}    PICTURE _EG_FMT_{0}{1};{0}RUN;{0}{0}", vbCrLf, _formatCode), _serverName, False)
            ' Set the flag to indicate that the temporary format has
            ' been created.
            tempFormatCreated = True

            ' Loop through the list view and create formatted values
            ' of the test Double value and store it in the list view
            ' item's second subitem.
            For Each lvi As ListViewItem In lvwTestValues.Items
                lvi.SubItems.Item(1).Text = GetFormattedValue(CType(lvi.Tag, Double))
            Next

            ' Set the button states based on current list view selections.
            UpdateButtonStates()
        End Sub

        Protected Overrides Sub OnClosing(ByVal e As System.ComponentModel.CancelEventArgs)
            ' The form is closing, so if we have created a temporary
            ' format on the SAS server, we want to make sure that it's
            ' destroyed.
            If (tempFormatCreated = True) Then
                ' Submit PROC CATALOG code to delete the test format.
                _consumer.Submit.SubmitCode(String.Format("PROC CATALOG CAT=WORK.FORMATS;{0}    DELETE _EG_FMT_ / ENTRYTYPE=FORMAT;{0}RUN;{0}{0}", vbCrLf), _serverName, False)
                ' Reset our internal flag.
                tempFormatCreated = False
            End If
        End Sub
#End Region

#Region " Private Helper Methods "
        ' This method returns the string resulting from passing the
        ' Double value parameter through the temporary test format.
        Private Function GetFormattedValue(ByVal value As Double) As String
            ' All of the following array are required as the parameters
            ' to the FormatService methods.
            Dim FormatTypes() As Int32 = New Int32() {0}   ' 0 = format, 1 = informat
            Dim IDs As Array
            Dim FailureMessages As Array
            Dim FormatNames() As String = New String() {"_EG_FMT_"}
            Dim Values(,) As Double = New Double(,) {{value}}
            Dim Widths() As Int32 = New Int32() {_defaultWidth}
            Dim Decimals() As Int32 = New Int32() {0}
            Dim Missings(,) As Byte = New Byte(,) {{0}}
            Dim FormattedValues As Array
            Dim ExceptionRowIndicies As Array
            Dim ExceptionColumnIndicies As Array
            Dim ExceptionIsError As Array
            Dim ExceptionMessages As Array


            ' We can't do a thing without a valid consumer object and
            ' a temporary format definition on the server.
            If (_consumer Is Nothing Or tempFormatCreated = False) Then
                Return String.Empty
            End If

            Try
                ' We need a SAS Workspace in order to be able to call
                ' the FormatService methods.
                Dim sasWs As SAS.IWorkspace = CType(_consumer.Workspace(_serverName), SAS.IWorkspace)

                ' We can't format the value if we haven't got a SAS
                ' Workspace.
                If (sasWs Is Nothing) Then
                    Return String.Empty
                End If

                ' Load our test format.
                sasWs.Utilities.FormatService.Load(FormatTypes, FormatNames, IDs, FailureMessages)
                If (FailureMessages.GetLength(0) <> 0) Then
                    Return String.Empty
                End If

                ' Make the call to the format service interface to get the
                ' formatted version of the numeric value.
                sasWs.Utilities.FormatService.FormatNumericValues(IDs, _
                    Widths, Decimals, Values, Missings, _
                    FormattedValues, ExceptionRowIndicies, _
                    ExceptionColumnIndicies, ExceptionIsError, _
                    ExceptionMessages)

                ' Whether it worked or not, we're done with the previously
                ' loaded format, so get the format service to unload it.
                sasWs.Utilities.FormatService.Unload(IDs)
            Catch e As System.Exception
                Throw New Exception("An exception was thrown while attempting to format a test value.", e)
            End Try

            ' Check for the FormatService having caused a SAS exception.
            If (ExceptionRowIndicies.GetLength(0) <> 0) Then
                Throw New Exception("The SAS Server generated an exception while attempting to format a test value.")
            End If

            ' Return the formatted value.
            Return CType(FormattedValues.GetValue(0, 0), String)
        End Function

        Private Sub UpdateButtonStates()
            ' Determine if there is a selected item in the list view.
            Dim itemSelected = (lvwTestValues.SelectedIndices.Count > 0)

            ' Only enable the Modify and Remove buttons if an item
            ' is selected.
            btnModify.Enabled = (itemSelected = True)
            btnRemove.Enabled = (itemSelected = True)
        End Sub
#End Region

#Region " Subcontrol Event Handlers "
        Private Sub lvwTestValues_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles lvwTestValues.SelectedIndexChanged
            ' The index of the currently selected list view item appears
            ' to have changed.
            If (lvwTestValues.SelectedIndices.Count = 0) Then
                ' No items are now selected, so we enter an initial
                ' value into each of the value input controls.
                txtTestValue.Text = "0"
                dateTestValue.Value = UtilityMethods.DotNetDateTimeDropMillisecs(DateTime.Now)
                timeTestValue.Value = dateTestValue.Value
            Else
                ' A new item has been selected, so we need to take
                ' the new item's associated Double value and place
                ' it in the appropriate input control for the
                ' picture type.
                Select Case (_pictureType)
                    Case 0 ' Numeric
                        txtTestValue.Text = lvwTestValues.SelectedItems(0).Text
                    Case 1 ' Date
                        dateTestValue.Value = UtilityMethods.SASDateToDotNetDateTime(CType(lvwTestValues.SelectedItems(0).Tag, Double))
                    Case 2 ' Time
                        timeTestValue.Value = UtilityMethods.SASTimeToDotNetDateTime(CType(lvwTestValues.SelectedItems(0).Tag, Double))
                    Case 3 ' DateTime
                        dateTestValue.Value = UtilityMethods.SASDateTimeToDotNetDateTime(CType(lvwTestValues.SelectedItems(0).Tag, Double))
                        timeTestValue.Value = dateTestValue.Value
                End Select
            End If

            ' Now enable or diasble the buttons based on the current
            ' list view selection.
            UpdateButtonStates()
        End Sub


        Private Sub txtTestValue_Validating(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles txtTestValue.Validating
            Dim doubleValue As Double

            ' Check that we have a valid Double value and convert it
            ' from a String to a Double if it is valid.
            If (Double.TryParse(txtTestValue.Text, Globalization.NumberStyles.Float, NumberFormatInfo.InvariantInfo, doubleValue) = False) Then
                ' The entered value was invalid, so we need to cancel
                ' the validation.
                e.Cancel = True
                MessageBox.Show(Me, "The value entered in the Test Value field is invalid." + vbCrLf + _
                        "It must consist of a single integer, floating point, or scientific notation value.", _
                        "Invalid Numeric Value")
            End If

            ' The validation succeeded.
            e.Cancel = False
        End Sub

        Private Sub btnAdd_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnAdd.Click
            Dim doubleValue As Double
            Dim lvi As ListViewItem = New ListViewItem

            ' Get the new value to be added from the input controls
            ' and store it in a Double variable. Also insert the text
            ' representation of the value into the first subitem of
            ' the new list view item. 
            Select Case (_pictureType)
                Case 0
                    If (Double.TryParse(txtTestValue.Text, Globalization.NumberStyles.Float, NumberFormatInfo.InvariantInfo, doubleValue) = False) Then
                        Throw New Exception("The Test Value field contains an invalid number. We should not have reached this point if it did.")
                    End If
                    lvi.SubItems(0).Text = doubleValue.ToString()
                Case 1
                    doubleValue = UtilityMethods.DotNetDateTimeToSASDate(dateTestValue.Value)
                    lvi.SubItems(0).Text = UtilityMethods.SASDateToString(doubleValue, False)
                Case 2
                    doubleValue = UtilityMethods.DotNetDateTimeToSASTime(timeTestValue.Value)
                    lvi.SubItems(0).Text = UtilityMethods.SASTimeToString(doubleValue, False)
                Case 3
                    doubleValue = UtilityMethods.DotNetDateAndTimeToSASDateTime(dateTestValue.Value, timeTestValue.Value)
                    lvi.SubItems(0).Text = UtilityMethods.SASDateTimeToString(doubleValue, False)
            End Select

            ' Store the Double value in the Tag property of the
            ' new list view item.
            lvi.Tag = doubleValue
            ' Add the formatted value of the Double as the second
            ' subitem of the new list view item.
            lvi.SubItems.Add(GetFormattedValue(doubleValue))

            ' Add the new list view item to the list view control.
            lvwTestValues.Items.Add(lvi)

            ' Make the new item the selected item in the control.
            lvi.Selected = True
        End Sub

        Private Sub btnModify_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnModify.Click
            Dim doubleValue As Double

            ' Get the new value to be added from the input controls
            ' and store it in a Double variable. Also update the text
            ' representation of the value in the first subitem of the
            ' currently selected list view item. 
            Select Case (_pictureType)
                Case 0
                    If (Double.TryParse(txtTestValue.Text, Globalization.NumberStyles.Float, NumberFormatInfo.InvariantInfo, doubleValue) = False) Then
                        Throw New Exception("The Test Value field contains an invalid number. We should not have reached this point if it did.")
                    End If
                    lvwTestValues.SelectedItems(0).SubItems(0).Text = doubleValue.ToString()
                Case 1
                    doubleValue = UtilityMethods.DotNetDateTimeToSASDate(dateTestValue.Value)
                    lvwTestValues.SelectedItems(0).SubItems(0).Text = UtilityMethods.SASDateToString(doubleValue, False)
                Case 2
                    doubleValue = UtilityMethods.DotNetDateTimeToSASTime(timeTestValue.Value)
                    lvwTestValues.SelectedItems(0).SubItems(0).Text = UtilityMethods.SASTimeToString(doubleValue, False)
                Case 3
                    doubleValue = UtilityMethods.DotNetDateAndTimeToSASDateTime(dateTestValue.Value, timeTestValue.Value)
                    lvwTestValues.SelectedItems(0).SubItems(0).Text = UtilityMethods.SASDateTimeToString(doubleValue, False)
            End Select

            ' Update the selected list view item's Tag property with
            ' the new Double value.
            lvwTestValues.SelectedItems(0).Tag = doubleValue
            ' Format the Double value and update the second subitem's
            ' text with it.
            lvwTestValues.SelectedItems(0).SubItems(1).Text = GetFormattedValue(doubleValue)
        End Sub

        Private Sub btnRemove_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnRemove.Click
            Dim index As Int32 = lvwTestValues.SelectedIndices(0)
            Dim lvi As ListViewItem = lvwTestValues.SelectedItems(0)
            ' Remove the reference to the Double from the selected
            ' list view item.
            lvi.Tag = Nothing
            ' Now, remove the item from the list view.
            lvi.Remove()

            ' Finally, we want to ensure that, so long as there
            ' are still items within the list, another item is
            ' selected.
            If (lvwTestValues.Items.Count > 0) Then
                ' If there was no item after the one that was
                ' just deleted, then we need to select the one
                ' that came immediately before it (which will
                ' now be the last one).
                If (index > lvwTestValues.Items.Count - 1) Then
                    index = lvwTestValues.Items.Count - 1
                End If
                lvwTestValues.Items(index).Selected = True
            End If
        End Sub
#End Region

    End Class
End Namespace